*-----------------------------------------------------------------------
      subroutine define_new_window(xmin, ymin, xmax, ymax, 
     +                             xwmin, ywmin, xwmax, ywmax,
     +                             uin, uout)
*-----------------------------------------------------------------------
* Asks the user to define the new window.
*
* xmin: Minimum x coordinate of the full (default) window.
* ymin: Minimum y coordinate of the full (default) window
* xmax: Maximum x coordinate of the full (default) window
* ymax: Maximum y coordinate of the full (default) window.
*
* xwmin: Minimum x coordinate of the user-defined window
* ywmin: Minimum y coordinate of the user-defined window
* xwmax: Maximum x coordinate of the user-defined window
* ywmax: Maximum y coordinate of the user-defined window
*
* uin : Input unit number.
* uout: Output unit number.
*
* Author: Rodrigo A. Marin
* Date  : January 19 1993
*-----------------------------------------------------------------------


      include 'postconst.h'
      include 'pubfunc.h'

      double precision xmin, ymin, xmax, ymax
      double precision xwmin, ywmin, xwmax, ywmax
      integer uin, uout

      double precision values(4)
      double precision temp
      integer nval

      character prompt*(MAXPROMPTLENGTH)
      character hcopyfname*(MAXSTRINGLENGTH)

      logical quit

      quit = .FALSE.

      write(uout, *)
      write(uout, 100) xwmin, ywmin, xwmax, ywmax

      write(uout, *)
      write(uout, 150)
      write(prompt, 200) xmin, ymin, xmax, ymax

      do while (.not. quit)
         write(uout, *)
         call readndouble(prompt, strlen(prompt), 4, .TRUE., nval,
     +                    values, uin, uout)

         if (nval .eq. 0) then
             xwmin = xmin
             ywmin = ymin
             xwmax = xmax
             ywmax = ymax
    
             quit = .TRUE.
         else if (nval .ne. 4) then
                  call reperror(16)
         else
           if (values(1) .gt. values(3)) then
               temp = values(1)
               values(1) = values(3)
               values(3) = temp
           end if

           if (values(2) .gt. values(4)) then
               temp = values(2)
               values(2) = values(4)
               values(4) = temp
           end if

           xwmin = values(1)
           ywmin = values(2)
           xwmax = values(3)
           ywmax = values(4)
       
           quit = .TRUE.
         end if
      end do

      return
100   format('The current window is (xmin= ', f8.3, ', ymin= ', f8.3,
     +       ') - (xmax= ', f8.3, ', ymax= ', f8.3, ')')
150   format('Window is defined as xmin, ymin, xmax, ymax')
200   format('Please define the new window [',
     +       f8.3, ',', f8.3, ',', f8.3, ',', f8.3, ']: ')
      end

*-----------------------------------------------------------------------
      subroutine displot
*-----------------------------------------------------------------------
* Plots the displaced shape of the border of the topology, using a
* magnification factor given by the user.
*
* Author: Rodrigo A. Marin
* Date  : November 29 1992
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      logical quit1
      logical err
      integer option
      integer i
      integer numcont
      integer nummancont
      integer numautcont

      integer invalid(1)
      integer defvar(1)
      logical scale_drawn
 
      character prompt*(MAXPROMPTLENGTH)
      character hcopyfname*(MAXSTRINGLENGTH)

      real ref_pt(2)

      double precision xminwall, yminwall

      double precision maxdisp

      numvar = 1
      itraknum(1) = 1
      defvar(1) = 1
      invalid(1) = 38
      conttype = 'a'

      scale_posx = 0.0
      scale_posy = 0.9
     

      if (.not. sol_file_open) then
          call open_solution_file(ibin, iunit, iout)
      end if

      write(iout, *)
      write(iout, *) 'Please use the graphics window'
      write(iout, *)

      call clear_messages()
        
      call find_iteration(timetype,time,newtime,
     +                lastzpos,newzpos,casvel,
     +                lastitnum,newitnum,
     +                t,ut,vt,sigma,epsln,evp,seff,
     +                epefft,eeff,epdot,uts,ratio,
     +                e0,fa,tle,elastic,eflow,dmgcrt,emod,
     +                ctime,nn,nns,itnum,
     +                zpos, ibin,iout)

      timetype = 't'

c-----------------------------------------------------------------------
c        Assign tempvar to desired plot variable
      call assign (tempvar,t,ut,vt,sigma,epsln,evp,
     +             prst2,seff,epefft,eeff,epdot,uts,
     +             ratio,e0,fa,tle,elastic,eflow,dmgcrt,
     +             emod,ctime,nn,nns,
     +             itraknum)
c-----------------------------------------------------------------------
c        Determine location and value of max and min variables
      call minmax2(tempvar,tmax,tmin,imax,imin,nn)
c-----------------------------------------------------------------------

      nummancont = 2
      mancontval(1) = tmin
      mancontval(2) = tmax

      numautcont = 10 
      numcont = numautcont

      quit1 = .FALSE.
      scale_drawn = .FALSE.

      call setplotconfig(mtitle, 'y', 'n', 'n', 'y', 'y', 'n',
     +                   intlab, disp_border_plot)

      curstate = STATE_DISPLOT_00
      newmenu = 8

      ref_pt(1) = -1.0
      ref_pt(2) = 1.0

      call set_menu(newmenu, ref_pt)
      call read_menu(option)
 
      do while (.not. quit1)
         if (curstate .eq. STATE_DISPLOT_00) then
             if (option .eq. 1) then
                 curstate = STATE_DISPLOT_01

                 ref_pt(1) = -1.0
                 ref_pt(2) = 1.0

                 newmenu = 2
             else if (option .eq. 2) then
                  curstate = STATE_DISPLOT_02

                  ref_pt(1) = MENUBAR_OPT_WIDTH_FRAC * (option - 1)
     +                        - 1.0
                  ref_pt(2) = 1.0

                  newmenu = 3
             else if (option .eq. 3) then
                  call read_variables(1, 1, invalid, 1, defvar,
     +                                numvar, itraknum)

c-----------------------------------------------------------------------
c        Assign tempvar to desired plot variable
                  call assign (tempvar,t,ut,vt,sigma,epsln,evp,
     +                         prst2,seff,epefft,eeff,epdot,uts,
     +                         ratio,e0,fa,tle,elastic,eflow,dmgcrt,
     +                         emod,ctime,nn,nns,
     +                         itraknum)
c-----------------------------------------------------------------------
c        Determine location and value of max and min variables
                  call minmax2(tempvar,tmax,tmin,imax,imin,nn)
c-----------------------------------------------------------------------

                  curstate = STATE_DISPLOT_00
                  newmenu = 8

                  ref_pt(1) = -1.0
                  ref_pt(2) = 1.0
             else if (option .eq. 4) then
                  curstate = STATE_DISPLOT_03

                  ref_pt(1) = MENUBAR_OPT_WIDTH_FRAC * (option - 1)
     +                        - 1.0
                  ref_pt(2) = 1.0

                  newmenu = 4
             else if (option .eq. 5) then
                  call annotate_plot()
                  curstate = STATE_DISPLOT_00

                  ref_pt(1) = -1.0
                  ref_pt(2) = 1.0

                  newmenu = 1
             else if (option .eq. 6) then
                  curstate = STATE_DISPLOT_06

                  ref_pt(1) = MENUBAR_OPT_WIDTH_FRAC * (option - 1)
     +                        - 1.0
                  ref_pt(2) = 1.0

                  newmenu = 7
             else if (option .eq. 7) then
                  curstate = STATE_DISPLOT_04

                  ref_pt(1) = MENUBAR_OPT_WIDTH_FRAC * (option - 1)
     +                        - 1.0
                  ref_pt(2) = 1.0

                  newmenu = 5
             end if
         else if (curstate .eq. STATE_DISPLOT_01) then
                  if (option .eq. 1) then
                      if (nodplot .eq. 'y') then
                          nodplot = 'n'
                      else 
                          nodplot = 'y'
                      end if
                  else if (option .eq. 2) then
                           if (nodnum .eq. 'y') then
                               nodnum = 'n'
                           else
                               nodnum = 'y'
                           end if
                  else if (option .eq. 3) then
                           if (elplot .eq. 'y') then
                               elplot = 'n'
                           else
                               elplot = 'y'
                           end if
                  else if (option .eq. 4) then
                           if (boundplt .eq. 'y') then
                               boundplt = 'n'
                           else
                               boundplt = 'y'
                           end if
                  else if (option .eq. 5) then
                           if (wallplot .eq. 'y') then
                               wallplot = 'n'
                           else
                               wallplot = 'y'
                           end if
                  else if (option .eq. 6) then
                           if (disp_border_plot .eq. 'y') then
                               disp_border_plot = 'n'
                           else
                               disp_border_plot = 'y'
                           end if
                  else if (option .eq. 7) then
                           write(prompt, 200)

                           defmag = read_double(READ_DEFPOSX,
     +                                          READ_DEFPOSY,
     +                                          prompt, strlen(prompt),
     +                                          .TRUE., .FALSE., defmag, 
     +                                          '(F8.3)', ' ')
                  else if (option .eq. 9) then
                           call read_scale_position(scale_posx,
     +                                              scale_posy,
     +                                              scale_drawn,
     +                                              xwmin, xwmax,
     +                                              ywmin, ywmax)
                  end if

                  curstate = STATE_DISPLOT_00
                  newmenu = 8

                  ref_pt(1) = -1.0
                  ref_pt(2) = 1.0
         else if (curstate .eq. STATE_DISPLOT_02) then
                  if (option .ne. MENU_NO_OPT_SEL) then
                      call ask_iter(option, time, newtime, lastzpos, 
     +                              newzpos, casvel, lastitnum, 
     +                              newitnum,iunit, iout, ibin, 
     +                              timetype)

                      if (option .ne. 4) then
                          call find_iteration(timetype,time,newtime,
     +                                        lastzpos,newzpos,casvel,
     +                                        lastitnum,newitnum,
     +                                        t,ut,vt,sigma,epsln,evp,
     +                                        seff,epefft,eeff,epdot,
     +                                        uts,ratio,e0,fa,tle,
     +                                        elastic,eflow,dmgcrt,emod,
     +                                        ctime,nn,nns,itnum,zpos,
     +                                        ibin,iout)
                          write(iout, *)
                          write(iout, *)
     +                    'Plot will be done for the following data:'
                          write(iout, *)
                          write(iout, 300)'Iteration : ', itnum
                          write(iout, 400)'Time      : ', ctime
                          write(iout, 400)'Z-position: ', zpos

c-----------------------------------------------------------------------
c        Assign tempvar to desired plot variable
                          call assign (tempvar,t,ut,vt,sigma,epsln,evp,
     +                                 prst2,seff,epefft,eeff,epdot,uts,
     +                                 ratio,e0,fa,tle,elastic,eflow,
     +                                 dmgcrt,emod,ctime,nn,nns,
     +                                 itraknum)
c-----------------------------------------------------------------------
c        Determine location and value of max and min variables
                          call minmax2(tempvar,tmax,tmin,imax,imin,nn)
c-----------------------------------------------------------------------
                      end if
                  end if

                  curstate = STATE_DISPLOT_00
                  newmenu = 8

                  ref_pt(1) = -1.0
                  ref_pt(2) = 1.0
         else if (curstate .eq. STATE_DISPLOT_03) then
                  if (option .eq. 1) then
                      conttype = 'a'
                  else if (option .eq. 2) then
                           conttype = 'v'
                  else if (option .eq. 3) then
                           if (contlabl .eq. 'y') then
                               contlabl = 'n'
                           else
                               contlabl = 'y'
                           end if
                  else if (option .eq. 4) then
                           call read_label_increment(intlab, intlab)
                  end if

                  if ((option .eq. 1) .or. (option .eq. 2)) then
c-----------------------------------------------------------------------
c        Display minimum and maximum possible contour values

                      write(iout, *)
                      write(iout, 400)
     +                'Minimum possible contour value: ', tmin
                      write(iout, 400)
     +                'Maximum possible contour value: ', tmax
                      write(iout, *)
                      call conlabl1(autcontval, mancontval,iunit, iout, 
     +                              contlabl,intlab, charlabl, conttype,
     +                              numautcont, nummancont, tmin,
     +                              tmax, ychoose, ymaxtemp)
                  end if

                  curstate = STATE_DISPLOT_00
                  newmenu = 8

                  ref_pt(1) = -1.0
                  ref_pt(2) = 1.0
         else if (curstate .eq. STATE_DISPLOT_04) then
                  if (option .ne. MENU_NO_OPT_SEL) then
                      call window_definition(option)
                      if (scale_drawn) then
                          call draw_scale(defmag, xwmin, ywmin,
     +                                    xwmax, ywmax, dsqrt(maxdisp),
     +                                    scale_posx, scale_posy,
     +                                    '?Picture/plot/cur_wall', 'm')
                      end if
                  end if

                  curstate = STATE_DISPLOT_00
                  newmenu = 8

                  ref_pt(1) = -1.0
                  ref_pt(2) = 1.0
         else if (curstate .eq. STATE_DISPLOT_06) then
                  curstate = STATE_DISPLOT_00
                  newmenu = 8

                  ref_pt(1) = -1.0
                  ref_pt(2) = 1.0

                  if (option .eq. 1) then
                      call update_displot(nummancont, numautcont,
     +                                    maxdisp)
                      call draw_scale(defmag, xwmin, ywmin,
     +                                xwmax, ywmax, dsqrt(maxdisp),
     +                                scale_posx, scale_posy,
     +                                '?Picture/plot/cur_wall', 'm')
                      scale_drawn = .TRUE.
                  else if (option .eq. 2) then
                           call clear_plot_area()
                           call clear_info_area()
                  else if (option .eq. 3) then
                           call generate_hardcopy(iunit, iout,
     +                                            hcopyfname)
                           if (disp_border_plot .eq. 'n') then
                               call save_contour_data(hcopyfname, 
     +                                                numcont)
                           end if
                  else if (option .eq. 4) then
                           call generate_pict_file(iunit, iout,
     +                                             hcopyfname)
                           if (disp_border_plot .eq. 'n') then
                               call save_contour_data(hcopyfname, 
     +                                                numcont)
                           end if
                  else if (option .eq. 5) then
                           quit1 = .TRUE.
                           curstate =  STATE_NONGRAPH
                           newmenu = MENU_NO_MENU
                  end if
         end if

         call set_menu(newmenu, ref_pt)

         if (newmenu .ne. MENU_NO_MENU) then
             call read_menu(option)
         end if

      end do

      return
100   format('Increment between labels')
200   format('Enter Distortion Magnification Factor')
300   format(A, I8)
400   format(A, E14.5)
      end

*-----------------------------------------------------------------------
      subroutine update_displot(nummancont, numautcont,
     +                          maxdisp)
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'
      
      integer nummancont, numautcont
      double precision maxdisp

      integer numcont
      double precision xminwall, yminwall
      double precision value
      double precision aux

      call conlabl2(autcontval,mancontval,numautcont,
     +              nummancont,tmax,tmin,imax,imin,nn,
     +              icont, iout,contlabl, charlabl,
     +              conttype)
 
      if (conttype .eq. 'a') then
          numcont = numautcont
      else
          numcont = nummancont
      end if

      maxdisp = 0.0

      do 200 i = 1, nn
         xdist(i) = x(i) + defmag * ut(i)
         ydist(i) = y(i) + defmag * vt(i)

         aux = sqrdist(x(i), y(i), x(i) + ut(i), y(i) + vt(i))

         if (aux .gt. maxdisp) then
             maxdisp = aux
         end if
200   continue

c-----------------------------------------------------------------------
c  Determine largest and smallest x and y values in mesh
      call minmax(xdist,nnodes,xmin,xmax)
      call minmax(ydist,nnodes,ymin,ymax)
                   
      if (wallplot .eq. 'y') then
          call mincoord_mold_wall(zpos, defmag, xminwall, yminwall)

          if (xminwall .lt. xmin) then
              xmin = xminwall
          end if

          if (yminwall .lt. ymin) then
              ymin = yminwall
           end if
      end if

c-----------------------------------------------------------------------
c        Initialize terminal

      call hf_flush_contents('?Picture/plot...', 'geometry')
      call hf_flush_contents('?Picture/plot/annots', 'segment')

      call setup_window(xwmin,ywmin,xwmax,ywmax)
      call hf_open_segment('?Picture/plot')

c----------------Plot nodes and nodes numbers if requested--------------
      if (nodplot .eq. 'y') call plotn(xdist,ydist,xwmin, ywmin,
     +                                 xwmax, ywmax, nn,nodnum)
c
c----------------Draw elements if requested-----------------------------
      if (elplot .eq. 'y') 
     +    call eplot(xdist,ydist,nn,nel,node,intrash)
c
c----------------Draw boundary if requested-----------------------------
      if (boundplt .eq. 'y') 
     +    call border(xdist,ydist,nn,nbe,nbnc,idash)
c
      if (disp_border_plot .eq. 'n') then
c-----------------------------------------------------------------------
c        Plot max and min labels on plot if desired
         if (contlabl .eq. 'y')
     +       call labmm(sngl(xdist(imin)),sngl(ydist(imin)),
     +                  sngl(xdist(imax)),sngl(ydist(imax)),
     +                  xwmin, ywmin, xwmax, ywmax)
c-----------------------------------------------------------------------
         call hf_open_segment('?Picture/plot/contours')
         call hf_set_color('lines = orange yellow, 
     +                         text = white')
         call hf_set_text_alignment('<v')
         call hf_set_text_font('size= 0.015 sru, 
     +                         rotation= follow path')

         do 140 i=1,numcont
            if (conttype .eq. 'a') then
                value = autcontval(i)
            else if (conttype .eq. 'v') then
                value = mancontval(i)
            end if

            call contur(value,itraknum(1),nn,nel,node,tempvar,
     +                  xdist,ydist,xwmin, ywmin, xwmax,ywmax,
     +                  intlab,contlabl,numdecval,
     +                  charlabl(i))
140      continue

         call hf_close_segment()
      end if

c----------------Draw mold wall if requested----------------------------
      if (wallplot .eq. 'y') then
           call plot_original_wall(defmag)
           call plot_cur_wall(zpos, defmag)
      end if
c-----------------------------------------------------------------------

      call display_annotations()

      call hf_close_segment()

      call clear_info_area()
         
      call display_plot_status(PLOTTYPE_DISPLOT)

      if (disp_border_plot .eq. 'n') then
          call display_contour_id(numcont)
      end if

      call update_display()

      return
      end
