*-----------------------------------------------------------------------
      subroutine find_iteration(timetype, time, newtime, zpos, newzpos, 
     +                          casvel, iternum, newitnum,
     +                          t, ut, vt, sigma, epsln, evp, seff, 
     +                          epefft, eeff, epdot, uts, ratio, e0, 
     +                          fa, tle, elastic, eflow, dmgcrt, emod,
     +                          ctime, nn, nns, itnum, 
     +                          czpos, usol, uout)
*-----------------------------------------------------------------------
* timetype: Parameter that indicates how was specified the desired 
*           iteration for which data will be ploted.
*           't' : The time for the iteration was given.
*           'z' : The z position was given
*           'n' : The iteration number was given
*
* According to the timetype parameter, finds the closest iteration 
* in the binary solution file. If timetype = 'z', the time is
* calculated as the quotient of zpos and casvel (casting velocity).
* The search is done looking for the first iteration whose time or
* iteration number is greater or equal to the given time, or iteration 
* number. If the end of the file is reached, the last iteration is
* used. 
* Whatever the timetype is, the corresponding three parameters time,
* zpos and iteration number are returned for the found iteration.
*
* Author: Rodrigo A. Marin
* Date  : November 22 1992
*-----------------------------------------------------------------------

       include 'postconst.h'
       include 'pubfunc.h'

       double precision time, newtime, zpos, newzpos, t(nn), 
     + ut(nns), vt(nns),
     + sigma(4,nns), epsln(4,nns),evp(4,nns),seff(nns),
     + epefft(nns), eeff(nns), epdot(nns),
     + uts(nns), ratio(nns), e0(nns), fa(nns), tle(nns),
     + elastic(4,nns),eflow(4,nns),dmgcrt(4,nns),emod(nns),
     + ctime, czpos

       double precision casvel, tsol, tliq

       integer nn, nns, nbes, iternum, newitnum, itnum, uout, usol 

       logical err, found, rew

       character timetype*1
       character rtitle*(MAXSTRINGLENGTH), mtitle*(MAXSTRINGLENGTH)
                                
       if (timetype .eq. 't') then
           if (dabs(newtime - time) .le. ZERO) then
               return
           end if
       else if (timetype .eq. 'z') then
                if (dabs(newzpos - zpos) .le. ZERO) then
                    return
                end if
       else if (timetype .eq. 'n') then
                if (newitnum .eq. iternum) then
                    return
                end if
       end if
    
       if (timetype .eq. 'u') then
           rew = .TRUE.
       else if (timetype .eq. 't') then
                if (newtime .lt. time) then
                    rew = .TRUE.
                end if
       else if (timetype .eq. 'z') then
                if (newzpos .lt. zpos) then
                    rew = .TRUE.
                end if
       else
           if (newitnum .lt. iternum) then
               rew = .TRUE.
           end if
       end if
       
       if (rew) then
           call binread1(casvel,tsol,tliq, taprn1, taprw1, rmolng,
     +                   rtitle,mtitle,MAXN,nn,nns,nbes, usol,uout)
       end if

       err = .FALSE.
       found = .FALSE.

       do while ((.not. err) .and. (.not. found))
           call binread2(t, ut, vt, sigma, epsln, evp, seff, epefft,
     +                   eeff, epdot, uts, ratio, e0, fa, tle, 
     +                   elastic, eflow, dmgcrt, emod, ctime,
     +                   nn, nns, itnum, err, usol, uout)

           if (.not. err) then
               if ((timetype .eq. 't') .and. (newtime .le. ctime)) then
                   found = .TRUE.
               else if ((timetype .eq. 'n') .and. (newitnum .le. itnum)) 
     +              then
                    found = .TRUE.
               else if ((timetype .eq. 'z') .and. (newtime .le. ctime))
     +              then
                    found = .TRUE.
               else if ((timetype .eq. 'u') .and. (newtime .le. ctime))
     +              then
                    found = .TRUE.
               end if
           end if 
       end do

       if (timetype .eq. 't') then
           time = newtime
           zpos = time * casvel
           iternum = itnum
       else if (timetype .eq. 'z') then
           zpos = newzpos
           time = zpos / casvel
           iternum = itnum
       else
           zpos = ctime * casvel
           time = ctime
           iternum = newitnum
       end if

       czpos = ctime * casvel

       return
       end
