*-----------------------------------------------------------------------
* interact.f
*
* Subroutines for user interaction management
*
* readdouble
* readinteger
* readmenu
* readndouble
* readopt
* readstring
*-----------------------------------------------------------------------
c
c
*---------------------------------------------------------------------
      subroutine clear_info_area
*---------------------------------------------------------------------

      call hf_flush_contents('?Picture/info...', 'text')

      return
      end
 
*---------------------------------------------------------------------
      subroutine deactivate_menu(indx)
*---------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      integer indx

      integer i
      character str*(MAXSTRINGLENGTH)

      if (menu_specs(indx)%level .eq. 0) then
          call hf_open_segment('?Picture/menu_bar')
      else
          call hf_open_segment('?Picture/menu')
      end if

      if (menu_specs(indx)%level .ne. 0) then
          call hf_flush_contents(menu_specs(indx)%name(1:
     +                           strlen(menu_specs(indx)%name)) //
     +                           '...', 'everything')

      else
          i = 1

          call hf_open_segment(menu_specs(indx)%name(1:
     +                         strlen(menu_specs(indx)%name)))

          do while (i .le. menu_specs(indx)%numopt)
             call itoa(i, str)

             call hf_open_segment(str(1:strlen(str)))
             call hf_set_selectability('windows = off')
             call hf_close_segment()

             i = i + 1
          end do

          call hf_close_segment()
      end if

      call hf_close_segment()

      return
      end

*-----------------------------------------------------------------------
      subroutine graphic_interaction(init_state, init_opt)
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      integer init_state
      integer init_opt

      integer option

      real ref_pt(2)

      curstate = init_state
      option = init_opt

      if (curstate .eq. STATE_NONGRAPH) then
          if (option .eq. 2) then
              call conplot(.FALSE., .FALSE.)
          else if (option .eq. 3) then
                   call displot
          else if (option .eq. 4) then
                   call crossplot
          else if (option .eq. 6) then
                   call meshplt
          end if
          call message(1)
      end if

      call set_menu(MENU_NO_MENU, ref_pt)

      return
      end

*-----------------------------------------------------------------------
      subroutine open_mesh_file(umesh, uin, uout)
*-----------------------------------------------------------------------
*     Asks the user for a file name for the mesh file and tries to
*     open it until it can be done successfully.
*
*     umesh: Mesh file unit.
*     uin  : Standard input unit.
*     uout : Standard output unit.
*
*     Author: Rodrigo A. Marin
*     Date  : January 1 1993
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      integer umesh, uin, uout
      integer meshfstat

      logical quit
      character meshfil*(MAXSTRINGLENGTH)
      character prompt*(MAXPROMPTLENGTH) 

      quit = .FALSE.

      write(uout, *)

      do while (.not. quit)
         prompt = 
     +   'Please enter the name of the Mesh File (without .msh):'
         meshfil = readstring(prompt, strlen(prompt), .FALSE., ' ', 
     +                        uin, uout)
         meshfil = meshfil(1:strlen(meshfil)) // MESHFILEEXT

         open(unit=umesh,file=meshfil, status='old',
     +        iostat = meshfstat)

         if (meshfstat .gt. 0) then
             call reperror(1, uout)
         else
           quit = .TRUE.
         endif
      end do

      call meshin1(mtitle,isp,isp2, nn,nnodes,nel,nhbw,nbe, nnodel,
     +             icorn,imesh,iunit,iout,iecho)

      call meshin2(x,y,node,nbnc,nnodes,nel,nnodel,nbe,umesh,uout)

      write(uout, *)
      write(uout, *)'Title of the Mesh File:'
      write(uout, *)mtitle
      write(uout, *)

      mesh_file_open = .TRUE.

      return
      end

*-----------------------------------------------------------------------
      subroutine open_solution_file(usol, uin, uout)
*-----------------------------------------------------------------------
*     Asks the user for a file name for the solution file and tries to
*     open it until it can be done successfully.
*
*     usol : Solution file unit.
*     uin  : Standard input unit.
*     uout : Standard output unit.
*
*     Author: Rodrigo A. Marin
*     Date  : January 1 1993
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      integer usol, uin, uout
      integer solfstat

      logical quit
      character solfil*(MAXSTRINGLENGTH)
      character prompt*(MAXPROMPTLENGTH) 

      quit = .FALSE.

      write(uout, *)

      do while (.not. quit)
         prompt = 
     +   'Please enter the name of the Solution File (without .sol):'
         solfil = readstring(prompt, strlen(prompt), .FALSE., ' ',
     +                       uin, uout)
         solfil = solfil(1:strlen(solfil)) // BINFILEEXT

         open(unit=usol, file=solfil, form='unformatted',
     +        status='old', iostat = solfstat)

         if (solfstat .gt. 0) then
             call reperror(2, uout)
         else 
           quit = .TRUE.
         end if
      end do

      call binread1(casvel, tsol, tliq, taprn1, taprw1, rmolng,
     +              rtitle, mtitle, MAXN, nn, nns,
     +              nbes, ibin, iout)

      if (nns .ne. nn) then
          write(uout, *)
          write(uout, 100)
          write(uout, 200) nn, nns
      end if

      write(uout, *)
      write(uout, *)'Title of the Solution File:'
      write(uout, *)rtitle
      write(uout, *)

      call find_iteration(timetype,time,newtime,lastzpos,newzpos,
     +                    casvel,lastitnum,newitnum,t,ut,vt,sigma,
     +                    epsln,evp,seff,epefft,eeff,epdot,uts,ratio,
     +                    e0,fa,tle,elastic,eflow,dmgcrt,emod,ctime,
     +                    nn,nns,itnum,zpos,ibin,iout)

      sol_file_open = .TRUE.

      return
100   format('WARNING: Only temperature data is available')
200   format(I8, ' Temperature Nodes, ', I8, ' Stress Nodes')
      end
        
*-----------------------------------------------------------------------
      subroutine pause(prompt, promptlength, uin, uout)

*-----------------------------------------------------------------------
* Display a message and wait for the user to press <RETURN>
*
* prompt      : Message to be displayed
* promptlength: Length of the message
* uin         : Standard Input unit
* uout        : Standard output unit
*
* Author: Rodrigo A. Marin
* Date  : November 12 1992
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      integer promptlength, uin, uout
      character prompt*(MAXPROMPTLENGTH) 

      character dummy*(MAXSTRINGLENGTH)
      
      dummy = readstring(prompt, promptlength, .TRUE., ' ', uin, uout)

      return
      end
 
*-----------------------------------------------------------------------
      subroutine read_line(segname, x0, y0, x1, y1)
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      character segname*(MAXSTRINGLENGTH)

      double precision x0, y0, x1, y1
      real x0r, y0r, x1r, y1r
      real x00, y00
      real xtmp, ytmp
      real dummy


      character event_type*(MAXSTRINGLENGTH)
      character segment*(MAXSTRINGLENGTH)
      character annot_segname*(MAXSTRINGLENGTH)
      character action*(MAXSTRINGLENGTH)
      
      logical quit
      integer status

      
      call hf_enable_selection_events('?locater', '?Picture')
               
      call hf_open_segment(segname)

      call hf_qset_selectability('?Picture/plot', 'v')
      call hf_set_heuristics('quick moves')

      call message(3)
      quit = .FALSE.

      do while (.not. quit)
         call hf_await_event(event_type)
         call ltrim(event_type)

         if (event_type(1:strlen(event_type)) .eq.
     +       'selection') then
             call hf_show_selection_source(segment, segment,
     +                                     action, status)

             if (action(1:strlen(action)) .eq. 'v') then
                call hf_qset_selectability('?Picture/plot', '*^')
                call hf_show_selection_position(x00, 
     +                                          y00, 
     +                                          dummy,
     +                                          dummy, dummy,
     +                                          dummy)

                call adjust_position(x00,y00,x0r,y0r)

                call message(4)
             else if (action(1:strlen(action)) .eq. '*') then
                      call hf_flush_geometry('.')
                      call hf_show_selection_position(xtmp, 
     +                         ytmp, dummy, dummy , dummy, dummy)

                      call adjust_position(xtmp,ytmp,x1r,y1r)

                            
                      call hf_insert_line(x0r, y0r, 0.0, x1r, y1r, 0.0)
             else if (action(1:strlen(action)) .eq. '^') then
                      call hf_flush_geometry('.')
                      quit = .TRUE.
                      call hf_close_segment()
                      call clear_messages()
             end if
         end if
      end do

      x0 = x0r
      x1 = x1r
      y0 = y0r
      y1 = y1r
      
      call hf_qset_selectability('?Picture/plot', 'v') 
      call hf_disable_selection_events('?locater', '?Picture')

      return
      end

*-----------------------------------------------------------------------
      subroutine readmenu(idmenu, choice, uin, uout)
*-----------------------------------------------------------------------
*
* Subroutine to display and read a choice from a menu
* 
* idmenu  : Identifier of the desired menu
* choice  : Option selected from the menu
* uin     : Input unit
* uout    : Output unit
*
* The options for the indicated menu are displayed and the user is 
* prompted to select a valid option. A menu is specified
* by the number of options, the default option, if any, and the
* prompt string. The default option is specified by the value of the 
* default variable. If this value is zero, means no default option. 
* The default option is selected by pressing <RETURN>. The option 
* selectors are the positive integer numbers starting from 1.
* 
* Author: Rodrigo Marin
* Date:   October 25 1992
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars
 
      include 'pubfunc.h'

      integer idmenu, choice, uin, uout
 
      character options(MAXOPTIONS)*(MAXOPTLENGTH), 
     +          prompt*(MAXPROMPTLENGTH)
      integer i, default, numopt
 
      write(uout,*)
 
      if (idmenu .eq. 1) then
          numopt = 2
          default = 1
          prompt = 'Option'
 
          write(uout, *) '************************'
          write(uout, *) '       MAIN MENU'
          write(uout, *) '************************'
          write(uout, *)
          write(uout, *) '  1. Interactive Input'
          write(uout, *) '  2. Specify Input File'
 
      else if (idmenu .eq. 2) then
          numopt = 7
          default = 7
          prompt = 'Option'
 
          write(uout,*)'**********************************************'
          write(uout,*)'          PLOT CONFIGURATION MENU'
          write(uout,*)'**********************************************'
          write(uout,*)
          write(uout,*)'   1. Plot Nodes [',nodplot,']'
          write(uout,*)'   2. Plot Node Numbers [',nodnum,']'
          write(uout,*)'   3. Plot Elements [',elplot,']'
          write(uout,*)'   4. Plot Domain Boundaries [',boundplt,']'
          write(uout,*)'   5. Label Contours [', contlabl, ']'
          write(uout,*)'   6. Increment between Labels [', 
     +                        intlab,']'
          write(uout,*)'   7. Go to next menu'

      else if (idmenu .eq. 3) then
          numopt = 7
          default = 1
          prompt = 'Option'

          write(uout, *)'*****************************************'
          write(uout, *)'           OUTPUT TYPE MENU'
          write(uout, *)'*****************************************'
          write(uout, *)
          write(uout, *)'  1. Printout Tables'
          write(uout, *)'  2. Contour Plot'
          write(uout, *)'  3. Displaced Shape Plot'
          write(uout, *)'  4. Stress Crosses Plot'
          write(uout, *)'  5. G & R Gradients and Velocities Plot'
          write(uout, *)'  6. Mesh Plot'
          write(uout, *)'  7. Quit'

      else if (idmenu .eq. 4) then
          numopt = 37
          default = choice
          prompt = 'Number of variable to output'

          write(uout, *)'**********************************************'
          write(uout, *)'                VARIABLES MENU'
          write(uout, *)'**********************************************'
          write(uout, *)
          write(uout, *)'1. Temperature                    ',
     +                  '31. Plastic strain rate'
          write(uout, *)'2. U-displacement                 ',
     +                  '32. Ult. Ten. Strength'
          write(uout, *)'3. V-displacement                 ',
     +                  '33. Stress / UTS'
          write(uout, *)'4. X-stress                       ',
     +                  '34. Thermal Strain'
          write(uout, *)'5. Y-stress                       ',
     +                  '35. Fraction alpha'
          write(uout, *)'6. Z-stress                       ',
     +                  '36. TLExp. coef.'
          write(uout, *)'7. XY-stress                      ',
     +                  '37. Hydrostatic Stress'
          write(uout, *)'8. X-strain (total)               ',
     +                  '38. Shell Thickness'
          write(uout, *)'9. Y-strain (total)               ',
     +                  '39. Elastic Strain X'
          write(uout, *)'10. Z-strain (total)              ',
     +                  '40. Elastic Strain Y'
          write(uout, *)'11. XY-strain (total)             ',
     +                  '41. Elastic Strain Z'
          write(uout, *)'12. X plastic strain              ',
     +                  '42. Elastic Strain XY'
          write(uout, *)'13. Y-plastic strain              ',
     +                  '43. Flow Strain X'
          write(uout, *)'14. Z-plastic strain              ',
     +                  '44. Flow Strain Y'
          write(uout, *)'15. XY-plastic strain             ',
     +                  '45. Flow Strain Z'
          write(uout, *)'16. Principal stress 1            ',
     +                  '46. Flow Strain XY'
          write(uout, *)'17. Principal stress 2            ',
     +                  '47. Damage criteria1'
          write(uout, *)'18. Principal stress 3            ',
     +                  '48. Damage criteria2'
          write(uout, *)'19. Principal shear stress        ',
     +                  '49. Damage criteria3'
          write(uout, *)'20. Principal strain 1            ',
     +                  '50. Damage criteria4'
          write(uout, *)'21. Principal strain 2            ',
     +                  '51. Elastic Modulus(MPa)'
          write(uout, *)'22. Principal strain 3            ' 
          write(uout, *)'23. Principal shear strain        '
          write(uout, *)'24. Principal plastic strain 1    '
          write(uout, *)'25. Principal plastic strain 2    '
          write(uout, *)'26. Principal plastic strain 3    '
          write(uout, *)'27. Principal plastic shear strain'
          write(uout, *)'28. Effective Stress              '
          write(uout, *)'29. Effective plastic strain      '
          write(uout, *)'30. Effective elastic strain      '
 

      else if (idmenu .eq. 5) then
          numopt = 4
          default = 1
          prompt = 'Option'   

          write(uout, *)'********************************'
          write(uout, *)'  ITERATION SPECIFICATION MENU'
          write(uout, *)'********************************'
          write(uout, *)
          write(uout, *)'   1. Specify Time'
          write(uout, *)'   2. Specify Z-Position'
          write(uout, *)'   3. Specify Iteration Number'
          write(uout, *)'   4. Check available data'

      else if (idmenu .eq. 6) then
          numopt = 2
          default = 1
          prompt = 'Option'

          write(uout, *)'**********************************'
          write(uout, *)' CONTOUR VALUE SPECIFICATION MENU'
          write(uout, *)'**********************************'
          write(uout, *)
          write(uout, *)'    1. Automatic Contours'
          write(uout, *)'    2. Specify Individual Values'

      else if (idmenu .eq. 7) then
          numopt = 2
          default = 1
          prompt = 'Option'

          write(uout, *)'1. More  2. Quit'

      else if (idmenu .eq. 8) then
          numopt = 3
          default = 1
          prompt = 'Option'

          write(uout, *)'************************'
          write(uout, *)'  SELECTION TYPE MENU'
          write(uout, *)'************************'
          write(uout, *)
          write(uout, *)'  1. Single Values'
          write(uout, *)'  2. Interval'
          write(uout, *)'  3. All values available'
          write(uout, *)

      else if (idmenu .eq. 9) then
          numopt = 4
          default = 1
          prompt = 'Option'

          write(uout, *)'******************************'
          write(uout, *)'       TABLE TYPE MENU'
          write(uout, *)'******************************'
          write(uout, *)
          write(uout, *)'  1. History Table'
          write(uout, *)'  2. Nodes vs. variables Table'
          write(uout, *)'  3. Slice Table'
          write(uout, *)'  4. Go to next menu'
          write(uout, *)

      else if (idmenu .eq. 10) then
           numopt = 3
           default = 1
           prompt = 'Option'

           write(uout, *)'*****************************'
           write(uout, *)'    DISPLACED PLOTS MENU'
           write(uout, *)'*****************************'
           write(uout, *)
           write(uout, *)'  1. Displaced Border Plot'
           write(uout, *)'  2. Displaced Contour Plots'
           write(uout, *)'  3. Go back'
           write(uout, *)
      else if (idmenu .eq. 11) then
           numopt = 3
           default = 1
           prompt = 'Option'

           write(uout, *)'*********************************'
           write(uout, *)'    CUTTING PLANE DEFINITION'
           write(uout, *)'*********************************'
           write(uout, *)
           write(uout, *)'  1. Mouse definition'
           write(uout, *)'  2. Keyboard definition'
           write(uout, *)'  3. Do not change cutting plane'
           write(uout, *)
      else if (idmenu .eq. 12) then
           numopt = 3
           default = 1
           prompt = 'Option'

           write(uout, *)'***************************'
           write(uout, *)'     WINDOW DEFINITION' 
           write(uout, *)'***************************'
           write(uout, *)
           write(uout, *)'  1. Mouse definition'
           write(uout, *)'  2. Keyboard definition'
           write(uout, *)'  3. Do not change window'
           write(uout, *)
      else if (idmenu .eq. 13) then
           numopt = 3
           default = 1
           prompt = 'Option'

           write(uout, *)'*********************************'
           write(uout, *)'    SHELL THICKNESS TRAJECTORY'
           write(uout, *)'*********************************'
           write(uout, *)
           write(uout, *)'  1. Mouse definition'
           write(uout, *)'  2. Keyboard definition'
           write(uout, *)'  3. Do not change trajectory'
           write(uout, *)
      end if
 
      write(uout, *)
 
      do i = 1, numopt
         write(options(i), 100) i 
      end do
 
      if (default .ne. 0) then
          prompt = prompt(1:strlen(prompt)) // ' [' // 
     +             char(ichar('0') + default) // ']'
      end if

      prompt = prompt(1:strlen(prompt)) // ':'
 
      call readopt(numopt, options, prompt, strlen(prompt)+1, .FALSE.,
     +             default, choice, uin, uout)
 
      write(uout, *)
      return
100   format(i2)
      end
 
*-----------------------------------------------------------------------
      double precision function readdouble(prompt, promptlength, 
     +                                     nullinput, default,
     +                                     uin, uout)
*-----------------------------------------------------------------------
*
* Displays a prompt and reads a double precision number from the 
* specified unit checking input/output errors.
*
* prompt      : Prompt string to be displayed
* promptlength: Length of the prompt string
* nullinput   : Boolean flag that indicates if a null input is accepted
* default     : Default value for null input, if accepted
* uin         : Standard input unit
* uout        : Standard output unit
*
* Author: Rodrigo A. Marin
* Date  : October 28 1992 
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      integer promptlength, uin, uout 
      character prompt*(MAXPROMPTLENGTH)
      character charrep*(MAXSTRINGLENGTH)

      logical nullinput
      double precision default

      logical quit

      quit = .FALSE.

      do while (.not. quit)
50       if (promptlength .gt. 0) then
             write(uout, 100) prompt(1:promptlength + 1)
         end if

         read(uin, 200, err = 50, end = 50) charrep
         
         if (strlen(charrep) .eq. 0) then
             if (nullinput) then
                 readdouble = default
                 quit = .TRUE.
             end if
         else
             read(charrep, 300, err = 50) readdouble
             quit = .TRUE.
         end if
      end do

      return
100   format(A, $)
200   format(A24)
300   format(d24.9)
      end

*---------------------------------------------------------------------
      double precision function read_double(posx, posy, prompt, 
     +                                      promptlength, nullinput, 
     +                                      defstr, default, def_fmt,
     +                                      strdef)
*---------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      real      posx, posy
      character prompt*(MAXPROMPTLENGTH)
      integer   promptlength
      logical   nullinput
      logical   defstr
      double precision default
      character def_fmt*(*)
      character strdef*(*)

      logical quit
      character str*(MAXSTRINGLENGTH)
      character def_str*(MAXSTRINGLENGTH)

      quit = .FALSE.

      if (defstr) then
          def_str = strdef
      else
          if (nullinput) then
              write(def_str, fmt = def_fmt(1:strlen(fmt))) default
          else
              def_str = ' '
          end if
      end if

      do while (.not. quit)
50       str = read_string(posx, posy, prompt, promptlength, 
     +                     nullinput, MAXSTRINGLENGTH, def_str)

         if (strlen(str) .eq. 0) then
             if (nullinput) then
                 read_double = default
                 quit = .TRUE.
             end if
         else
             read(str, 100, err = 50) read_double
         end if

         quit = .TRUE.
      end do
      
      return
100   format(D24.9)
      end
     
*---------------------------------------------------------------------
      subroutine read_menu(option)
*---------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      integer option

      character segname*(MAXSTRINGLENGTH)
      character prev*(MAXSTRINGLENGTH)
      character menuname*(MAXSTRINGLENGTH)
      character str*(MAXSTRINGLENGTH)

      integer i
      integer status
      integer but_status

      integer STATUS_NOBUTPRESS, STATUS_BUTPRESS

      parameter (STATUS_NOBUTPRESS = 0, STATUS_BUTPRESS = 1)

      logical quit
      logical err

      character event_type*(MAXSTRINGLENGTH)
      character segment*(MAXSTRINGLENGTH)
      character action*(MAXSTRINGLENGTH)

      call hf_enable_selection_events('?locater', '?Picture')
      call hf_flush_all_events()
         
      quit = .FALSE.
      prev = 'NULL'
      but_status = STATUS_NOBUTPRESS

      if (menu_specs(curmenu)%level .eq. 0) then
          call hf_qset_selectability('?Picture/menu_bar', 
     +                               	'windows = on')
      else
          call hf_qset_selectability('?Picture/menu', 'windows = on')
      end if

      do while (.not. quit)
         call hf_await_event(event_type)
         call ltrim(event_type)

         if (event_type(1:strlen(event_type)) .eq. 'selection') then
             call hf_show_selection_source(segment, segment,
     +                                     action, status)

             if (action(1:strlen(action)) .eq. 'v') then
                 call hf_show_selection_pathname(segname)
                 
                 if (index(segname, 'menu') .ne. 0) then
                         call hf_show_selection(segname)
                         option = atoi(segname, err)

                         if (.not. err)  then
                             if (menu_specs(curmenu)%level .eq. 0) then
                               call hf_open_segment('?Picture/menu_bar')
                             else
                               call hf_open_segment('?Picture/menu')
                             end if

                             menuname = menu_specs(curmenu)%name
                             call hf_open_segment(menuname(1:
     +                                            strlen(menuname)))
                             call hf_open_segment(segname(1:
     +                                            strlen(segname)))
                             call hf_set_color('window = white,
     +                                          text = black')
                             call hf_close_segment()
                             call hf_close_segment()
                             call hf_close_segment()
                             prev = segname
                         end if

                         but_status = STATUS_BUTPRESS
                 end if

                 call hf_qset_selectability('?Picture/menu_bar', '^')
                 call hf_qset_selectability('?Picture/menu', '^')

             else if (action(1:strlen(action)) .eq. '*') then
                      call hf_show_selection_pathname(segname)
                 
                      if (index(segname, 'menu') .ne. 0) then
                             call hf_show_selection(segname)
                             if ((prev(1:strlen(prev)) .ne. 'NULL')
     +                           .and. (prev(1:strlen(prev)) .ne.
     +                                 (segname(1:strlen(segname)))))
     +                       then
                                 if (menu_specs(curmenu)%level .eq. 0)
     +                           then
                               call hf_open_segment('?Picture/menu_bar')
                                 else
                                   call hf_open_segment('?Picture/menu')
                                end if

                                 menuname = menu_specs(curmenu)%name
                                 call hf_open_segment(menuname(1:
     +                                                strlen(menuname)))
                                call hf_open_segment(prev(1:
     +                                               strlen(prev)))
                                call hf_set_color('window = black,
     +                                            text = white')
                                call hf_close_segment()
                                call hf_close_segment()
                                call hf_close_segment()
                             end if

                             option = atoi(segname, err)

                             if ((.not. err) .and. prev(1:strlen(prev))
     +                           .ne. segname(1:strlen(segname)))  then
                                 if (menu_specs(curmenu)%level .eq. 0)
     +                           then
                               call hf_open_segment('?Picture/menu_bar')
                                 else
                                   call hf_open_segment('?Picture/menu')
                                 end if

                                 menuname = menu_specs(curmenu)%name
                                 call hf_open_segment(menuname(1:
     +                                                strlen(menuname)))
                                 call itoa(option, str)
                                 call hf_open_segment(str(1:
     +                                                strlen(str)))
                                 call hf_set_color('window = white,
     +                                              text = black')
                                 call hf_close_segment()
                                 call hf_close_segment()
                                 call hf_close_segment()
                                 prev = segname
                             end if
                      end if
             else if ((action(1:strlen(action)) .eq. '^') .and.
     +                (but_status .eq. STATUS_BUTPRESS)) then
                      call hf_show_selection_pathname(segname)
                 
                      if (index(segname, 'menu') .ne. 0) then

                             if (prev(1:strlen(prev)) .ne. 'NULL') then
                                 if (menu_specs(curmenu)%level .eq. 0)
     +                           then
                               call hf_open_segment('?Picture/menu_bar')
                                 else
                                   call hf_open_segment('?Picture/menu')
                                 end if

                                 menuname = menu_specs(curmenu)%name
                                 call hf_open_segment(menuname(1:
     +                                                strlen(menuname)))
                                call hf_open_segment(prev(1:
     +                                               strlen(prev)))
                                call hf_set_color('window = black,
     +                                            text = white')
                                call hf_close_segment()
                                call hf_close_segment()
                                call hf_close_segment()
                             end if

                             call hf_show_selection(segname)
                             option = atoi(segname, err)

                             if (.not. err) then
                                 if (.not. 
     +                               menu_specs(curmenu)%active(option))
     +                           then
                                     option = MENU_NO_OPT_SEL
                                 end if
                             else
                                 option = MENU_NO_OPT_SEL
                             end if
                      end if

                      quit = .TRUE.
             else if (action(1:strlen(action)) .eq. 'o') then

             end if
         end if
      end do

      call hf_disable_selection_events('?locater', '?Picture')

c     call hf_get_selection(segname)

c     i = 1

c     option = MENU_NO_OPT_SEL

c     do while (i .le. menu_specs(curmenu)%numopt)
c        if (menu_specs(curmenu)%active(i)) then
c            call itoa(i, str)

c            if (segname(1:strlen(segname)) .eq.
c    +           str(1:strlen(str))) then
c                option = i
c            end if
c        end if

c        i = i + 1
c     end do

      call deactivate_menu(curmenu)

      call hf_qset_selectability('?Picture/menu_bar', 'windows = off')
      call hf_qset_selectability('?Picture/menu', 'windows = off')

      return
      end

*-----------------------------------------------------------------------
      subroutine readndouble(prompt, promptlength, maxnum, nullinput, 
     +                       nval, values, uin, uout)
*-----------------------------------------------------------------------
*     Reads a sequence of double precision numbers separated by commas,
*     checking input/output errors.
*
*     prompt      : Prompt string to be displayed
*     promptlength: Length of the prompt string
*     maxnum      : Maximum number of numbers to be read
*     nullinput   : TRUE if null input is acceptable
*     nval        : Numbers of values actually read
*     values      : Array with the values read
*     uin         : Input unit number
*     uout        : Output unit number 
*
*     Author: Rodrigo A. Marin
*     Date  : January 19 1993
*-----------------------------------------------------------------------
      include 'postconst.h'
      include 'pubfunc.h'

      character prompt*(MAXPROMPTLENGTH)
      integer   promptlength, maxnum, nval, uin, uout
      double precision values(maxnum)
      logical   nullinput 

      integer i
      integer strbegin
      integer strend
      character charrep*(MAXSTRINGLENGTH)
      
      logical quit
      logical err
      logical end

      quit = .FALSE.

      do while (.not. quit)
50       if (promptlength .gt. 0) then
             write(uout, 100) prompt(1:promptlength + 1)
         end if

         read(uin, 200, err = 50, end = 50) charrep

         if (strlen(charrep) .eq. 0) then
             if (nullinput) then
                 nval = 0
                 quit = .TRUE.
             end if
         else
             i = 0
             
             err = .FALSE.
             end = .FALSE.
             strbegin = 1

             do while ((i .lt. maxnum) .and. (.not. end) .and.
     +                 (.not. err))
                strend = strbegin - 1 +
     +                   index(charrep(strbegin:strlen(charrep)), ',')

                if (strend .eq. strbegin - 1) then
                    strend = strlen(charrep) + 1
                end if

                err = (strend - strbegin) .lt. 1

                if (.not. err) then
                    err = .TRUE.

                    read(charrep(strbegin:strend-1), 300, err = 60,
     +                   end = 60) values(i+1)

                    err = .FALSE.

                    strbegin = strend + 1

60                  i = i + 1

                    end = (strbegin .gt. strlen(charrep)) .or. 
     +                    (strend .gt. strlen(charrep))
                end if
             end do
         end if

         if (.not. err) then
             quit = .TRUE.
             nval = i
         end if
      end do

      return
100   format(A, $)
200   format(A80)
300   format(d24.9)
      end

*-----------------------------------------------------------------------
      subroutine read_ndouble(posx, posy, prompt, promptlength, maxnum, 
     +                        nullinput, defstr, ndef, defval, nval, 
     +                        values, def_fmt, str)
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      real      posx, posy
      character prompt*(MAXPROMPTLENGTH)
      integer   promptlength, maxnum, nval, ndef
      double precision values(maxnum)
      double precision defval(ndef)
      logical   nullinput
      logical   defstr
      character def_fmt*(*)
      character str*(*)

      integer i
      integer j
      integer tlength
      integer strbegin
      integer strend
      character charrep*(MAXSTRINGLENGTH)
      character def_str*(MAXSTRINGLENGTH)
      character def*(MAXSTRINGLENGTH)
      
      logical quit
      logical err
      logical end

      if (defstr) then
          def_str = str
      else
          if (nullinput) then
              def_str = '['

              i = 1
              tlength = 1

              do while ((i .le. ndef) .and. 
     +                  (tlength .lt. MAXSTRINGLENGTH))
                 write(def, fmt = def_fmt(1:strlen(def_fmt))) defval(i)

                 tlength = tlength + strlen(def) + 1

                 if (i .eq. 1) then
                     tlength = tlength - 1
                 end if

                 if (tlength .lt. MAXSTRINGLENGTH) then
                     call ltrim(def)
                     if (i .gt. 1) then
                         def_str = def_str(1:strlen(def_str)) //
     +                             ',' // def(1:strlen(def))
                     else
                         def_str = def_str(1:strlen(def_str)) //
     +                             def(1:strlen(def))
                     end if
                 end if

                 i = i + 1
              end do

              def_str = def_str(1:strlen(def_str)) // ']'
          end if
      end if

      quit = .FALSE.

      do while (.not. quit)
         charrep = read_string(posx, posy, prompt, promptlength, 
     +                         nullinput, MAXSTRINGLENGTH, def_str)

         if (strlen(charrep) .eq. 0) then
             if (nullinput) then
                 nval = 0
                 quit = .TRUE.
             end if
         else
             i = 0
             
             err = .FALSE.
             end = .FALSE.
             strbegin = 1

             do while ((i .lt. maxnum) .and. (.not. end) .and.
     +                 (.not. err))
                strend = strbegin - 1 +
     +                   index(charrep(strbegin:strlen(charrep)), ',')

                if (strend .eq. strbegin - 1) then
                    strend = strlen(charrep) + 1
                end if

                err = (strend - strbegin) .lt. 1

                if (.not. err) then
                    err = .TRUE.

                    read(charrep(strbegin:strend-1), 300, err = 60,
     +                   end = 60) values(i+1)

                    err = .FALSE.

                    strbegin = strend + 1

60                  i = i + 1

                    end = (strbegin .gt. strlen(charrep)) .or. 
     +                    (strend .gt. strlen(charrep))
                end if
             end do

             if (.not. err) then
                 quit = .TRUE.
                 nval = i
             end if
         end if
      end do

      return
300   format(d24.9)
      end

*-----------------------------------------------------------------------
      subroutine readninteger(prompt, promptlength, maxnum, nullinput, 
     +                        nval, values, uin, uout)
*-----------------------------------------------------------------------
*     Reads a sequence of integer numbers separated by commas,
*     checking input/output errors.
*
*     prompt      : Prompt string to be displayed
*     promptlength: Length of the prompt string
*     maxnum      : Maximum number of numbers to be read
*     nullinput   : TRUE if null input is acceptable
*     nval        : Numbers of values actually read
*     values      : Array with the values read
*     uin         : Input unit number
*     uout        : Output unit number 
*
*     Author: Rodrigo A. Marin
*     Date  : January 19 1993
*-----------------------------------------------------------------------
      include 'postconst.h'
      include 'pubfunc.h'

      character prompt*(MAXPROMPTLENGTH)
      integer   promptlength, maxnum, nval, uin, uout
      integer   values(maxnum)
      logical   nullinput 

      integer i
      integer strbegin
      integer strend
      character charrep*(MAXSTRINGLENGTH)
      
      logical quit
      logical err
      logical end

      quit = .FALSE.

      do while (.not. quit)
50       if (promptlength .gt. 0) then
             write(uout, 100) prompt(1:promptlength + 1)
         end if

         read(uin, 200, err = 50, end = 50) charrep

         if (strlen(charrep) .eq. 0) then
             if (nullinput) then
                 nval = 0
                 quit = .TRUE.
             end if
         else
             i = 0
             
             err = .FALSE.
             end = .FALSE.
             strbegin = 1

             do while ((i .lt. maxnum) .and. (.not. end) .and.
     +                 (.not. err))
                strend = strbegin - 1 +
     +                   index(charrep(strbegin:strlen(charrep)), ',')

                if (strend .eq. strbegin - 1) then
                    strend = strlen(charrep) + 1
                end if

                err = (strend - strbegin) .lt. 1

                if (.not. err) then
                    values(i+1) = atoi(charrep(strbegin:strend-1), err)

                    strbegin = strend + 1

60                  i = i + 1

                    end = (strbegin .gt. strlen(charrep)) .or. 
     +                    (strend .gt. strlen(charrep))
                end if
             end do
         end if

         if (.not. err) then
             quit = .TRUE.
             nval = i
         end if
      end do

      return
100   format(A, $)
200   format(A80)
300   format(i12)
      end

*-----------------------------------------------------------------------
      subroutine read_ninteger(posx, posy, prompt, promptlength, maxnum, 
     +                         nullinput, defstr, ndef, defval, nval, 
     +                         values, str)
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      real      posx, posy
      character prompt*(MAXPROMPTLENGTH)
      integer   promptlength, maxnum, nval, ndef
      integer   values(maxnum)
      character str*(*)
      integer   defval(ndef)
      logical   nullinput 
      logical   defstr

      integer i
      integer strbegin
      integer strend
      integer tlength
      character charrep*(MAXSTRINGLENGTH)
      character def_str*(MAXSTRINGLENGTH)
      character def*(MAXSTRINGLENGTH)
      
      logical quit
      logical err
      logical end

      if (defstr) then
          def_str = str
      else 
          if (nullinput) then
              def_str = '['

              i = 1
              tlength = 1

              do while ((i .le. ndef) .and.
     +                  (tlength .lt. MAXSTRINGLENGTH))
                 call itoa(defval(i), def)
                 call ltrim(def)

                 tlength = tlength + strlen(def) + 1

                 if (i .eq. 1) then
                     tlength = tlength - 1
                 end if

                 if (tlength .lt. MAXSTRINGLENGTH) then
                     if (i .gt. 1) then
                         def_str = def_str(1:strlen(def_str)) // 
     +                             ',' // def(1:strlen(def))
                     else
                         def_str = def_str(1:strlen(def_str)) // 
     +                             def(1:strlen(def))
                     end if
                 end if

                 i = i + 1
              end do

              def_str = def_str(1:strlen(def_str)) // ']'
          end if
      end if

      quit = .FALSE.

      do while (.not. quit)
50       charrep = read_string(posx, posy, prompt, promptlength, 
     +                         nullinput, MAXSTRINGLENGTH, def_str)

         if (strlen(charrep) .eq. 0) then
             if (nullinput) then
                 nval = 0
                 quit = .TRUE.
             end if
         else
             i = 0
             
             err = .FALSE.
             end = .FALSE.
             strbegin = 1

             do while ((i .lt. maxnum) .and. (.not. end) .and.
     +                 (.not. err))
                strend = strbegin - 1 +
     +                   index(charrep(strbegin:strlen(charrep)), ',')

                if (strend .eq. strbegin - 1) then
                    strend = strlen(charrep) + 1
                end if

                err = (strend - strbegin) .lt. 1

                if (.not. err) then
                    err = .TRUE.

                    read(charrep(strbegin:strend-1), 300, err = 60,
     +                   end = 60) values(i+1)

                    err = .FALSE.

                    strbegin = strend + 1

60                  i = i + 1

                    end = (strbegin .gt. strlen(charrep)) .or. 
     +                    (strend .gt. strlen(charrep))
                end if
             end do

             if (.not. err) then
                 quit = .TRUE.
                 nval = i
             end if
         end if
      end do

      return
300   format(i12)
      end
*----------------------------------------------------------------------
      integer function readinteger(prompt, promptlength, nullinput, 
     +                             default, uin, uout)
*-----------------------------------------------------------------------
* Displays a prompt and reads an integer from the specified unit checking 
* input/output errors.
*
* prompt      : Prompt string to be displayed
* promptlength: Length of the prompt string
* nullinput   : Boolean flag that indicates if a null input is accepted
* default     : Default value for null input, if accepted
* Author: Rodrigo A. Marin
* Date  : October 28 1992 
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      integer promptlength, uin, uout, default
      logical nullinput
      character prompt*(MAXPROMPTLENGTH)
      character charrep*(MAXSTRINGLENGTH)

      logical quit

      quit = .FALSE.

      do while (.not. quit)
50       if (promptlength .gt. 0) then
             write(uout, 100) prompt(1:promptlength + 1)
         end if

         read(uin, 200, err = 50, end = 50) charrep
         
         if (strlen(charrep) .eq. 0) then
             if (nullinput) then
                 readinteger = default
                 quit = .TRUE.
             end if
         else
             read(charrep, 300, err = 50) readinteger
             quit = .TRUE.
         end if
      end do

      return
100   format(A, $)
200   format(A12)
300   format(I12)
      end

*-----------------------------------------------------------------------
      subroutine read_label_increment(default, newintlab)
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      integer default
      integer newintlab

      logical quit
      character prompt*(MAXPROMPTLENGTH)

      quit = .FALSE.

      do while (.not. quit)
         write(prompt, 100) intlab

         newintlab = read_integer(READ_DEFPOSX,
     +                            READ_DEFPOSY,
     +                            prompt, strlen(prompt),
     +                            .TRUE., .FALSE.,
     +                            default, ' ')

         if (newintlab .gt. 0) then
             quit = .TRUE.
             call clear_errmsg()
         else
             call rep_error(13)
         end if
      end do

      return
100   format('Increment between labels')
      end

*-----------------------------------------------------------------------
      integer function read_integer(posx, posy, prompt, promptlength, 
     +                              nullinput, defaultstr, default,
     +                              defstr)
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      real      posx, posy
      character prompt*(MAXPROMPTLENGTH)
      integer   promptlength
      logical   nullinput
      logical   defaultstr
      integer   default
      character defstr*(*)

      logical quit
      character str*(MAXSTRINGLENGTH)
      character def_str*(MAXSTRINGLENGTH)

      quit = .FALSE.
 
      if (defaultstr) then
          def_str = defstr
      else
          if (nullinput) then
              call itoa(default, def_str)
          else
              def_str = ' '
          end if
      end if

      do while (.not. quit)
50       str = read_string(posx, posy, prompt, promptlength, 
     +                     nullinput, MAXSTRINGLENGTH, def_str)

         if (strlen(str) .eq. 0) then
             if (nullinput) then
                 read_integer = default
                 quit = .TRUE.
             end if
         else
             read(str, 100, err = 50) read_integer
         end if

         quit = .TRUE.
      end do
      
      return
100   format(I12)
      end
 
*-----------------------------------------------------------------------
      subroutine readopt(numopt, options, prompt, promptlength,
     +                   case_sens, default, opt_sel, uin, uout)
      include 'postconst.h'
      include 'pubfunc.h'
*-----------------------------------------------------------------------
*
* Subroutine to ask the user to select an option from a set
* of alternatives. All the options are single characters. 
*        
* numopt:      Number of options to select from
* options:     Array of character with the valid options
* prompt:      Prompt string to be displayed to ask for the option
* promptlength:Length of the prompt string 
* case_sens:   .TRUE. if the option selection must be case sensitive
* default:     Index of the option to be selected by just pressing 
*              <RETURN>
* opt_sel:     Selected option  
* uin:         Input unit
* uout:        Output unit
*
* The following steps are executed repetitively until a valid option is
* read:
*
* - The prompt string is displayed
* - A pause is made to wait for user input of the selected option
* - The selected option is checked to test its validity. i.e. if 
*   appears in the options array, or if a null input was done and
*   a default option was provided. 
* 
* Author: Rodrigo Marin
* Date:   October 23 1992
*-----------------------------------------------------------------------
c
      integer numopt, default, promptlength
      character options(numopt)*(MAXOPTLENGTH), 
     +          prompt*(MAXPROMPTLENGTH)
      logical case_sens
      integer opt_sel
      integer uin, uout
c
      integer i
      logical quit
      character opt*(MAXOPTLENGTH)
      character copt*(MAXOPTLENGTH)
c
      quit = .FALSE.
 




      do while (.not. quit)
         write(uout, 100) prompt(1:promptlength)
         
         read(uin, 200) copt 

         if ((copt .eq. ' ') .and. (default .ne. 0)) then
             quit = .TRUE.
             opt_sel = default
         else
             i = 1 

             do while ((i .le. numopt) .and. (.not. quit))
                if (.not. case_sens) then 
                    call strup(copt, copt)
                    call strup(options(i), opt)
                else
                    opt = options(i)
                end if
         
                opt_sel = i

                call ltrim(opt)
                call ltrim(copt)

                quit = (copt .eq. opt)
                
                i = i + 1
             end do
         end if
      end do

      return
100   format(A, $)
200   format(A)
      end

*-----------------------------------------------------------------------
      subroutine read_pagelength(pagelength, uin, uout)
*-----------------------------------------------------------------------
* Reads the page length to be used when displaying lists of values.
*
* The read value is checked to be a positive integer.
*
* Author: Rodrigo A. Marin
* Date  : November 22 1992
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'
    
      integer pagelength
      integer uin, uout

      character prompt*(MAXSTRINGLENGTH)
      character pagel_str*(MAXSTRINGLENGTH)

      logical quit 

      call itoa(pagelength, pagel_str)
 
      prompt = 'Please enter page length for listings [' 
     +         // pagel_str(1:strlen(pagel_str)) // ']: '

      quit = .FALSE.

      do while (.not. quit)
         pagelength = readinteger(prompt, strlen(prompt)+1, .TRUE.,
     +                            pagelength, uin, uout)
         
         if (pagelength .gt. 0) then
             quit = .TRUE.
         else
           call reperror(5, uout)
         end if
      end do

      return
      end

*-----------------------------------------------------------------------
      character*(*) function readstring(prompt, promptlength, nullinput,
     +                                  default, uin, uout)
*-----------------------------------------------------------------------
* Displays a prompt and reads an integer from the specified unit checking 
* input/output errors.
*
* nullinput   : Boolean flag that indicates if a null input is accepted
* default     : Default value for null input, if accepted
*
* Author: Rodrigo A. Marin
* Date  : October 28 1992 
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'
      
      character*(MAXSTRINGLENGTH) prompt
      integer promptlength, uin, uout
      
      logical nullinput
      character*(*) default

      logical quit

      quit = .FALSE.

      do while (.not. quit)
50       if (promptlength .gt. 0) then
             write(uout, 100) prompt(1:promptlength + 1)
         end if

         read(uin, 200, err = 50, end = 50) readstring
         
         if (strlen(readstring) .eq. 0) then
             if (nullinput) then
                 readstring = default
                 quit = .TRUE.
             end if
         else
           quit = .TRUE.
         end if
      end do

      return
100   format(A, $)
200   format(A)
      end

*-----------------------------------------------------------------------
      subroutine read_scale_position(scale_posx, scale_posy, 
     +                               scale_drawn, xwmin, xwmax,
     +                               ywmin, ywmax)
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      real scale_posx, scale_posy
      double precision xwmin, xwmax
      double precision ywmin, ywmax

      logical scale_drawn
 
      real dummy

      character segment*(MAXSTRINGLENGTH)

      if (.not. scale_drawn) then
          call scalemark(scale_posx, scale_posy, xwmin, xwmax,
     +                    ywmin, ywmax)
      end if

      call message(10)

      call hf_get_selection(segment)

      call clear_messages()

      call hf_show_selection_position(scale_posx, scale_posy, dummy, 
     +                                dummy, dummy, dummy)
      
      call scalemark(scale_posx, scale_posy, xwmin, xwmax,
     +               ywmin, ywmax)

      return
      end

*-----------------------------------------------------------------------
      character*(*) function read_string(posx, posyin, prompt,
     +                                   promptlength, nullinput, 
     +                                   maxlength, default)
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      real    posx, posyin
      character*(MAXPROMPTLENGTH) prompt
      integer promptlength
      logical nullinput
      integer maxlength
      character*(*) default

      logical quit
      integer i
      integer string_length
      integer nlines
      real    xfrac, yfrac
      real    dx1, dy1
      real    left
      real    posy

      quit = .FALSE.

      call hf_open_segment('?Picture')

      call hf_open_segment('string_input')

      string_length = max(promptlength, maxlength)

      if (nullinput) then
          string_length = max(string_length, strlen(default))
      end if

      call hf_set_text_alignment('*')
      call hf_set_text_font('size = 0.015 sru')
      call hf_set_camera_projection('stretched')

      call hf_compute_text_extent('.', 'T', xfrac, yfrac)

      if (nullinput) then
          nlines = 4
      else
          nlines = 2
      end if
 
      dx1 = STR_IN_WIDTH_FRAC * 2.0
      dy1 = yfrac * 2.0
 
      left = posx
      posy = posyin

      call hf_open_segment('prompt')

      call hf_set_window(left, left + dx1, posy - dy1, posy)

      call hf_insert_text(0.0, 0.0, 0.0,
     +                    prompt(1:promptlength))

      call hf_close_segment()

      posy = posy - dy1

      if (nullinput) then
          call hf_open_segment('default')

          call hf_set_window(left, left + dx1, posy - 2 * dy1, posy)

          call hf_insert_text(0.0, 0.5, 0.0, 'Default')

          call hf_insert_text(0.0, -0.5, 0.0,
     +              default(1:strlen(default)))

          call hf_close_segment()

          posy = posy - 2 * dy1
      end if

      call hf_open_segment('input_line')

      call hf_set_window(left, left + dx1, posy - dy1, posy)

      do while (.not. quit)
         call hf_flush_contents('.', 'text')

         call hf_get_string('.', read_string)

         if (strlen(read_string) .eq. 0) then
             if (nullinput) then
                 quit = .TRUE.
             end if
         else
             quit = .TRUE.
         end if
      end do

      call hf_close_segment()

      call hf_close_segment()

      call hf_flush_contents('?Picture/string_input...', 'everything')
      call hf_close_segment()

      return
      end

*---------------------------------------------------------------------
      subroutine read_variables(maxnum, ninv, invalid, ndef, defvar, 
     +                          nvar, vars)
*---------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      integer maxnum
      integer ninv
      integer invalid(ninv)
      integer ndef
      integer defvar(ndef)
      integer nvar
      integer vars(maxnum)
      
      real VAR_SEP_FACX

      parameter (VAR_SEP_FACX = 0.05)

      real VAR_SEP_FACY

      parameter (VAR_SEP_FACY = 1.3)

      integer i
      integer incvar
      integer max_length

      logical quit

      real xfrac, yfrac
      real xwidth, xheight
      real dx1, dy1
      real posx, posy
      real left
      real dy_var
      real posy_var

      real max_width
      real total_height

      character prompt*(MAXPROMPTLENGTH)
      character def_str*(MAXSTRINGLENGTH)
      character def*(MAXSTRINGLENGTH)
      character opt_str*(MAXSTRINGLENGTH)

      call hf_open_segment('?Picture')

      call hf_open_segment('read_var')

      call hf_set_text_font('size = 0.015 sru')
      call hf_set_camera_projection('stretched')

      call hf_compute_text_extent('.', 'T', xfrac, yfrac)

      dy1 = yfrac * 2.0

      call hf_compute_text_extent('.', header(1)(1:strlen(header(1))),
     +                            xfrac, yfrac)

      max_width = yfrac
      max_length = strlen(header(1))

      i = 2

      incvar = MAXNUMVAR / 2 + mod(MAXNUMVAR, 2)

      call hf_compute_text_extent('.', 'X', xwidth, xheight)
      
      total_height = yfrac - xheight * 0.5

      do while (i .le. MAXNUMVAR)
         call hf_compute_text_extent('.', 
     +                               header(i)(1:strlen(header(i))),
     +                               xfrac, yfrac)

         if (xfrac .gt. max_width) then
             max_width = xfrac
         end if

         if (strlen(header(i)) .gt. max_length) then
             max_length = strlen(header(i))
         end if

         if (i .le. incvar) then
             yfrac = (yfrac - xheight * 0.5) * VAR_SEP_FACY
             total_height = total_height + yfrac
         end if

         i = i + 1
      end do

      max_width = max_width * 2.0
      total_height = total_height * 2.0
      dx1 = (max_width * 2.0) * (1.0 + 6.0 * VAR_SEP_FACX)
      left = (2.0 - dx1) / 2.0 - 1.0
      posy = 1.0 - (2.0 - total_height) / 2.0

      call hf_open_segment('variables')
      call hf_set_text_alignment('<v')
      
      call hf_set_window(left, left + dx1, posy - total_height, posy)
      call hf_update_display()

      i = 1
      dy_var = 2.0 / incvar
      posy_var = 1.0

      do while (i .le. incvar)
         write(opt_str, 100) i, header(i)

         call hf_insert_text(-1.0, posy_var - dy_var, 0.0,
     +                       opt_str(1:strlen(opt_str)))

         write(opt_str, 100) i + incvar, header(i + incvar)

         call hf_insert_text(VAR_SEP_FACX, posy_var - dy_var, 0.0,
     +                       opt_str(1:strlen(opt_str)))
         call hf_update_display()
         posy_var = posy_var - dy_var
         i = i + 1
      end do

      call hf_close_segment()
      call hf_close_segment()
      call hf_close_segment()

      posy = posy - total_height

      prompt = 'Select Variables'

      quit = .FALSE.

      do while (.not. quit)
         call read_ninteger(left + 
     +                      (dx1 - STR_IN_WIDTH_FRAC * 2.0) / 2.0, 
     +                      posy, prompt, strlen(prompt), maxnum, 
     +                      .TRUE., .FALSE., ndef, defvar, nvar, vars,
     +                      ' ')
 
         if (nvar .eq. 0) then
             nvar = ndef

             i = 1

             do while (i .le. ndef)
                vars(i) = defvar(i)
                i = i + 1
             end do

             quit = .TRUE.

         else
             i = 1

             quit = .TRUE.

             do while (i .le. nvar)
                j = 1

                do while (j .le. ninv)
                   if ((vars(i) .eq. invalid(j)) .or.
     +                 (vars(i) .gt. MAXNUMVAR) .or. 
     +                 (vars(i) .lt. 1)) then
                       quit = .FALSE.
                   end if
                   j = j + 1
                end do

                i = i + 1
             end do
         end if

         if (.not. quit) then
             call rep_error(8)
         else
             call clear_errmsg()
         end if
      end do

      i = 1

      do while (i .le. nvar)
         defvar(i) = vars(i)
         i = i + 1
      end do

      call hf_flush_contents('?Picture/read_var...', 'everything')

      return
100   format(I2, '. ', A<max_length>)
      end

*---------------------------------------------------------------------
      subroutine set_menu(indx, ref_pt)
*---------------------------------------------------------------------

      use posttypes
      use glob_vars

      integer indx
      real    ref_pt(2)

      integer i

      if (indx .eq. 1) then
          menu_specs(1)%name = 'menubar_conplot'
          menu_specs(1)%level = 0
          menu_specs(1)%numopt = 7
          menu_specs(1)%labels(1) = 'Plot Config'
          menu_specs(1)%labels(2) = 'Iteration'
          menu_specs(1)%labels(3) = 'Variable'
          menu_specs(1)%labels(4) = 'Contour'
          menu_specs(1)%labels(5) = 'Annotate'
          menu_specs(1)%labels(6) = 'Plot'
          menu_specs(1)%labels(7) = 'Window'

          i = 1

          do while (i .le. menu_specs(1)%numopt)
             menu_specs(1)%active(i) = .TRUE. 
             i = i + 1
          end do

      else if (indx .eq. 2) then
          menu_specs(2)%name = 'plot_config'
          menu_specs(2)%level = 1
          menu_specs(2)%numopt = 9

          if (nodplot .eq. 'y') then
              menu_specs(2)%labels(1) = 'Do not Plot Nodes'
          else
              menu_specs(2)%labels(1) = 'Plot Nodes'
          end if
          
          if (nodnum .eq. 'y') then
              menu_specs(2)%labels(2) = 'Do not Plot Node Numbers'
          else
              menu_specs(2)%labels(2) = 'Plot Node Numbers'
          end if

          if (elplot .eq. 'y') then
              menu_specs(2)%labels(3) = 'Do not Plot Elements'
          else
              menu_specs(2)%labels(3) = 'Plot Elements'
          end if

          if (boundplt .eq. 'y') then
              menu_specs(2)%labels(4) = 'Do not Plot Boundary'
          else
              menu_specs(2)%labels(4) = 'Plot Boundary'
          end if
 
          if (wallplot .eq. 'y') then
              menu_specs(2)%labels(5) = 'Do not Plot Mold Walls'
          else
              menu_specs(2)%labels(5) = 'Plot Mold Walls'
          end if

          if (disp_border_plot .eq. 'n') then
              menu_specs(2)%labels(6) = 'Border displaced plot'
          else
              menu_specs(2)%labels(6) = 'Contour displaced plot'
          end if

          if (curstate .eq. STATE_CROSSPLOT_01) then
              menu_specs(2)%labels(7) = 'Scale Factor'
          else
              menu_specs(2)%labels(7) = 'Magnification Factor'
          end if

          menu_specs(2)%labels(8) = 'Nodes to skip between crosses'

          menu_specs(2)%labels(9) = 'Scale Position'

          i = 1

          do while (i .le. menu_specs(2)%numopt)
             menu_specs(2)%active(i) = (curstate .ne.
     +                                  STATE_CROSSPLOT_01)
             i = i + 1
          end do

          menu_specs(2)%active(5) = (curstate .eq. STATE_DISPLOT_01)

          menu_specs(2)%active(6) = (curstate .eq. STATE_DISPLOT_01)

          menu_specs(2)%active(7) = ((curstate .eq. STATE_DISPLOT_01)
     +                               .or.
     +                               curstate .eq. STATE_CROSSPLOT_01) 

          menu_specs(2)%active(8) = (curstate .eq. STATE_CROSSPLOT_01)
          menu_specs(2)%active(9) = (curstate .eq. STATE_CROSSPLOT_01 
     +                          .or. (curstate .eq. STATE_DISPLOT_01))

      else if (indx .eq. 3) then
           menu_specs(3)%name = 'iter_spec'
           menu_specs(3)%level = 1
           menu_specs(3)%numopt = 4
           menu_specs(3)%labels(1) = 'Specify Time'
           menu_specs(3)%labels(2) = 'Specify Z-Pos'
           menu_specs(3)%labels(3) = 'Specify Iteration No.'
           menu_specs(3)%labels(4) = 'Check available data'

          i = 1

          do while (i .le. menu_specs(3)%numopt)
             menu_specs(3)%active(i) = .TRUE.
             i = i + 1
          end do 

      else if (indx .eq. 4) then
           menu_specs(4)%name = 'contval_spec'
           menu_specs(4)%level = 1
           menu_specs(4)%numopt = 4

           menu_specs(4)%labels(1) = 'Automatic Contours'
           menu_specs(4)%labels(2) = 'Input Values'
 
           if (contlabl .eq. 'y') then
               menu_specs(4)%labels(3) = 'Do not Label Contours'
           else
               menu_specs(4)%labels(3) = 'Label Contours'
           end if

           menu_specs(4)%labels(4) = 'Label increment'

           i = 1

           do while (i .le. menu_specs(4)%numopt)
              menu_specs(4)%active(i) = .TRUE.
              i = i + 1
           end do

           menu_specs(4)%active(3) = (curstate .eq. STATE_CONPLOT_03) 
     +                               .or.
     +                               ((curstate .eq. STATE_DISPLOT_03) 
     +                               .and.
     +                                (disp_border_plot .eq. 'n'))
           menu_specs(4)%active(4) = (curstate .eq. STATE_CONPLOT_03) 
     +                                .or.
     +                               ((curstate .eq. STATE_DISPLOT_03) 
     +                                .and.
     +                                (disp_border_plot .eq. 'n'))

      else if (indx .eq. 5) then
           menu_specs(5)%name = 'window_def'
           menu_specs(5)%level = 1
           menu_specs(5)%numopt = 3
           menu_specs(5)%labels(1) = 'Max Window'
           menu_specs(5)%labels(2) = 'Keybd. Def'
           menu_specs(5)%labels(3) = 'Mouse Def'

           i = 1

           do while (i .le. menu_specs(5)%numopt)
              menu_specs(5)%active(i) = .TRUE.
              i = i + 1
           end do

      else if (indx .eq. 6) then
           menu_specs(6)%name = 'menubar_annot'
           menu_specs(6)%level = 0
           menu_specs(6)%numopt = 3
           menu_specs(6)%labels(1) = 'Entity'
           menu_specs(6)%labels(2) = 'Edit'
           menu_specs(6)%labels(3) = 'Quit'

           i = 1

           do while (i .le. menu_specs(6)%numopt)
              menu_specs(6)%active(i) = .TRUE.
              i = i + 1
           end do

      else if (indx .eq. 7) then
           menu_specs(7)%name = 'plot'
           menu_specs(7)%level = 1
           menu_specs(7)%numopt = 5
           menu_specs(7)%labels(1) = 'Update Plot'
           menu_specs(7)%labels(2) = 'Clear Plot'
           menu_specs(7)%labels(3) = 'PS Hardcopy'
           menu_specs(7)%labels(4) = 'PICT File'
           menu_specs(7)%labels(5) = 'Quit'
           
           i = 1

           do while (i .le. menu_specs(7)%numopt)
              menu_specs(7)%active(i) = .TRUE.
              i = i + 1
           end do
      else if (indx .eq. 8) then
          menu_specs(8)%name = 'menubar_displot'
          menu_specs(8)%level = 0
          menu_specs(8)%numopt = 7
          menu_specs(8)%labels(1) = 'Plot Config'
          menu_specs(8)%labels(2) = 'Iteration'
          menu_specs(8)%labels(3) = 'Variable'
          menu_specs(8)%labels(4) = 'Contour'
          menu_specs(8)%labels(5) = 'Annotate'
          menu_specs(8)%labels(6) = 'Plot'
          menu_specs(8)%labels(7) = 'Window'

          i = 1

          do while (i .le. menu_specs(8)%numopt)
             menu_specs(8)%active(i) = .TRUE. 
             i = i + 1
          end do

          if (curstate .eq. STATE_DISPLOT_00) then
              menu_specs(8)%active(4) = disp_border_plot .eq. 'n'
          end if
      else if (indx .eq. 9) then
          menu_specs(9)%name = 'annot_entity'
          menu_specs(9)%level = 1
          menu_specs(9)%numopt = 2
          menu_specs(9)%labels(1) = 'Text'
          menu_specs(9)%labels(2) = 'Line'
          
          i = 1

          do while (i .le. menu_specs(9)%numopt)
             menu_specs(9)%active(i) = .TRUE. 
             i = i + 1
          end do
      else if (indx .eq. 10) then
          menu_specs(10)%name = 'annot_edit'
          menu_specs(10)%level = 1
          menu_specs(10)%numopt = 3
          menu_specs(10)%labels(1) = 'Add'
          menu_specs(10)%labels(2) = 'Delete'
          menu_specs(10)%labels(3) = 'Move'
          
          i = 1

          do while (i .le. menu_specs(10)%numopt)
             menu_specs(10)%active(i) = .TRUE.
             i = i + 1
          end do
      else if (indx .eq. 11) then
          menu_specs(11)%name = 'menubar_meshplot'
          menu_specs(11)%level = 0
          menu_specs(11)%numopt = 4
          menu_specs(11)%labels(1) = 'Plot Config'
          menu_specs(11)%labels(2) = 'Annotate'
          menu_specs(11)%labels(3) = 'Plot'
          menu_specs(11)%labels(4) = 'Window'

          i = 1

          do while (i .le. menu_specs(11)%numopt)
             menu_specs(11)%active(i) = .TRUE. 
             i = i + 1
          end do
 
      else if (indx .eq. 12) then
          menu_specs(12)%name = 'menubar_crossplot'
          menu_specs(12)%level = 0
          menu_specs(12)%numopt = 6
          menu_specs(12)%labels(1) = 'Plot Config'
          menu_specs(12)%labels(2) = 'Iteration'
          menu_specs(12)%labels(3) = 'Variable'
          menu_specs(12)%labels(4) = 'Annotate'
          menu_specs(12)%labels(5) = 'Plot'
          menu_specs(12)%labels(6) = 'Window'

          i = 1

          do while (i .le. menu_specs(12)%numopt)
             menu_specs(12)%active(i) = .TRUE. 
             i = i + 1
          end do
      else if (indx .eq. MENU_NO_MENU) then
           call hf_flush_contents('?Picture/menu_bar...', 
     +                            'segment')
      end if

      num_menus = 11

      curmenu = indx

      if (curmenu .ne. MENU_NO_MENU) then
          call display_menu(menu_specs(curmenu), ref_pt)
      end if

      call hf_update_display()

      return
      end

*---------------------------------------------------------------------
      subroutine window_definition(option)
*---------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      integer option

      character prompt*(MAXSTRINGLENGTH)
      character event_type*(MAXSTRINGLENGTH)
      character segment*(MAXSTRINGLENGTH)
      character action*(MAXSTRINGLENGTH)

      real x0, y0
      real x00, y00
      real x1, y1
      real dummy

      double precision values(4)
      double precision temp
      real   rectangle(3, 5)

      integer nval
      integer status

      logical quit

      if (option .eq. 1) then
          xwmin = xmin
          ywmin = ymin
          xwmax = xmax
          ywmax = ymax
      else if (option .eq. 2) then
               write(iout, *)
               write(iout, 100) xmin, ymin, xmax, ymax
               prompt = 
     +         'Enter window definition (xmin, ymin, xmax, ymax)'
               
               quit = .FALSE.

               values(1) = xwmin
               values(2) = ywmin
               values(3) = xwmax
               values(4) = ywmax

               do while (.not. quit)
                  call read_ndouble(READ_DEFPOSX, READ_DEFPOSY,
     +                              prompt, strlen(prompt), 4,
     +                              .TRUE., .FALSE., 4, values, nval,
     +                              values, '(F8.3)', ' ')

                  if (nval .eq. 0) then
                      quit = .TRUE.
                  else if (nval .eq. 4) then
                           if (values(1) .gt. values(3)) then
                               temp = values(1)
                               values(1) = values(3)
                               values(3) = temp
                           end if

                           if (values(2) .gt. values(4)) then
                               temp = values(2)
                               values(2) = values(4)
                               values(4) = temp
                           end if

                           xwmin = values(1)
                           ywmin = values(2)
                           xwmax = values(3)
                           ywmax = values(4)

                           quit = .TRUE.
                           call clear_errmsg()
                  else
                      call rep_error(16)
                  end if
               end do
      else if (option .eq. 3) then
               call hf_enable_selection_events('?locater', '?Picture')

               call hf_open_segment('?Picture/plot/rubberband box')

               call hf_set_heuristics('quick moves')
               call hf_set_line_pattern('- -')
               call hf_set_color('lines = red')

               call message(8)
               quit = .FALSE.

               do while (.not. quit)

                  call hf_await_event(event_type)

                  call ltrim(event_type)

                  if (event_type(1:strlen(event_type)) .eq.
     +                'selection') then
                      call hf_show_selection_source(segment, segment,
     +                                              action, status)

                      if (action(1:strlen(action)) .eq. 'v') then
                          call hf_qset_selectability('?Picture/plot', 
     +                                               '*^')
                          call hf_show_selection_position(x00,y00, 
     +                                                    dummy,
     +                                                    dummy, 
     +                                                    dummy,
     +                                                    dummy)


                          call adjust_position(x00,y00,x0,y0)

                          call message(9)
                      else if (action(1:strlen(action)) .eq. '*') then
                               call hf_flush_geometry('.')
                               call hf_show_selection_position(x00, 
     +                              y00, dummy, dummy, dummy, dummy)
                            


                             call adjust_position(x00,y00,x1,y1)

                               rectangle(1, 1) = x0
                               rectangle(2, 1) = y0
                               rectangle(3, 1) = 0.0
                               rectangle(1, 2) = x0
                               rectangle(2, 2) = y1
                               rectangle(3, 2) = 0.0
                               rectangle(1, 3) = x1
                               rectangle(2, 3) = y1
                               rectangle(3, 3) = 0.0
                               rectangle(1, 4) = x1
                               rectangle(2, 4) = y0
                               rectangle(3, 4) = 0.0
                               rectangle(1, 5) = x0
                               rectangle(2, 5) = y0
                               rectangle(3, 5) = 0.0

                               call hf_insert_polyline(5, rectangle)
                      else if (action(1:strlen(action)) .eq. '^') then
                           call hf_flush_geometry('.')
                           call hf_qset_selectability('?Picture/plot',
     +                                                'v')
                           quit = .TRUE.
                           call clear_messages()
                      end if
                  end if
               end do

               call hf_close_segment()
               call hf_disable_selection_events('?locater', '?Picture')

               xwmin = amin1(x0, x1)
               ywmin = amin1(y0, y1)
               xwmax = amax1(x0, x1)
               ywmax = amax1(y0, y1)
      end if

      if (option .ne. MENU_NO_OPT_SEL) then
          call setup_window(xwmin, ywmin, xwmax, ywmax)
      end if

      return
100   format('The maximum window is (xmin= ', f8.3, ', ymin= ', f8.3,
     +       ') - (xmax= ', f8.3, ', ymax= ', f8.3, ')')
      end

*---------------------------------------------------------------------
      subroutine adjust_position(xlocal,ylocal,xworld,yworld)
*---------------------------------------------------------------------
c
c     Need for adjusting mouse movements, written June 97, by Jon Parkman
c     Given local position, xlocal,ylocal, returns real world coordinates
c     for '?Picture/plot' window
c
c
c
      implicit real (a-z)

      dimension point1(3),point2(3)


      point1(1)=xlocal
      point1(2)=ylocal
      point1(3)=0.0

      point2(1)=0.0
      point2(2)=0.0
      point2(3)=0.0


      call hf_compute_coordinates('?Picture/plot',
     +       'local window',point1,'world',point2)

      xworld = point2(1)
      yworld = point2(2)


      return
      end
