*-----------------------------------------------------------------------
      subroutine mincoord_mold_wall(czpos, mag_fac, xminwall, yminwall)
*-----------------------------------------------------------------------
* Calculates the minimum x and y coordinate of the plot wall, used in 
* determination of the default window for displaying the displaced shape
* plot.
*
* Author: Rodrigo A. Marin
* Date  : March 4 1993
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      double precision czpos
      double precision mag_fac
      double precision xminwall, yminwall

      double precision wall_disp, distortion

      call taper(x(isp(1)), y(isp(1)), czpos, wall_disp, distortion,
     +           NARROW_WALL_FLAG, 0)

      xminwall = x(isp(1)) + mag_fac * (wall_disp - distortion)
 
      call taper(x(icorn), y(icorn), czpos, wall_disp, distortion,
     +           WIDTH_WALL_FLAG, 0)

      yminwall = y(icorn) + mag_fac * (wall_disp - distortion)

      return
      end

*-----------------------------------------------------------------------
      subroutine plot_cur_wall(zpos, defmag)
*-----------------------------------------------------------------------


      include 'postconst.h'

      double precision zpos
      double precision defmag

      logical scale_drawn

      call hf_open_segment('?Picture/plot/cur_wall')
      call hf_set_color('lines = green')
      call plot_wall(zpos, defmag, LINETYPE_DASHED)
      scale_drawn = .TRUE.
      call hf_close_segment()

      return
      end

*-----------------------------------------------------------------------
      subroutine plot_original_wall(defmag)
*-----------------------------------------------------------------------

      include 'postconst.h'

      double precision defmag
      double precision dummy

      call hf_open_segment('?Picture/plot/orig_wall')
      call hf_set_color('lines = red')
      call plot_wall(0.0, defmag, LINETYPE_DOTTED, dummy)
      call hf_close_segment()

      return
      end
 
*-----------------------------------------------------------------------
      subroutine plot_wall(czpos, mag_fac, linet)
*-----------------------------------------------------------------------
* Plots the distorted walls of the mold for a specific Z-position
*
* czpos: Z-position at which the wall distortion is ploted.
* mag_fac: Magnification factor that must be applied to the plot.
* linet  : Line type: LINETYPE_DASHED, LINETYPE_DOTTED or LINETYPE_CONTINUOUS
* Author: Rodrigo A. Marin
* Date  : January 18 1993
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      double precision czpos, mag_fac
      integer linet

      integer i
      integer j
      double precision x1_w, y1_w, x2_w, y2_w
      double precision xicorn, yicorn
      double precision distortion, wall_disp
       
c     Calculate narrow wall position

      call taper(x(isp(1)), y(isp(1)), czpos, wall_disp, distortion,
     +           NARROW_WALL_FLAG, 0)

      x1_w = x(isp(1)) + mag_fac * (wall_disp - distortion)
      y1_w = y(isp(1))

      call taper(x(icorn), y(icorn), czpos, wall_disp, distortion,
     +           NARROW_WALL_FLAG, 0)

      x2_w = x(icorn) + mag_fac * (wall_disp - distortion)
      xicorn = x2_w
     
      call taper(x(icorn), y(icorn), czpos, wall_disp, distortion,
     +           WIDTH_WALL_FLAG, 0)

      y2_w = y(icorn) + mag_fac * (wall_disp - distortion)
      yicorn = y2_w
      
      if (linet .eq. LINETYPE_DASHED) then
          call dashline(x1_w, y1_w, x2_w, y2_w)
      else if (linet .eq. LINETYPE_DOTTED) then
               call dottedline(x1_w, y1_w, x2_w, y2_w)
      else if (linet .eq. LINETYPE_CONTINUOUS) then
               call line(x1_w, y1_w, x2_w, y2_w)
      end if
      
c     Calculate width face position

      i = 1
      j = 1

      do while (i .le. nbe)
         if (nbnc(5, i) .eq. WIDTH_WALL_NODE_FLAG) then
             if (nbnc(1, i) .eq. icorn) then
                 if (j .eq. 1) then
                     x1_w = xicorn
                 else
                     x2_w = xicorn
                 end if
             else
                 if (j .eq. 1) then
                     x1_w = xicorn + x(nbnc(1, i))
                 else
                     x2_w = xicorn + x(nbnc(1, i))
                 end if
             end if

             call taper(x(nbnc(1, i)), y(nbnc(1, i)), czpos,
     +                  wall_disp, distortion, WIDTH_WALL_FLAG, 0)
                 
             if (j .eq. 1) then
                 y1_w = y(nbnc(1, i)) +
     +                  mag_fac * (wall_disp - distortion)
             else
                 y2_w = y(nbnc(1, i)) +
     +                  mag_fac * (wall_disp - distortion)

                 if (linet .eq. LINETYPE_DASHED) then
                     call dashline(x1_w, y1_w, x2_w, y2_w)
                 else if (linet .eq. LINETYPE_DOTTED) then
                          call dottedline(x1_w, y1_w, x2_w, y2_w)
                 else if (linet .eq. LINETYPE_CONTINUOUS) then
                          call line(x1_w, y1_w, x2_w, y2_w)
                 end if

                 x1_w = x2_w
                 y1_w = y2_w
             end if
             j = j + 1
         end if
         i = i + 1
      end do 

      return
      end

*-----------------------------------------------------------------------
      subroutine read_mold_params
*-----------------------------------------------------------------------
*     Reads from the user the mold parameters for the mold wall position
*     calculation, verifying input errors.
*
*     Author: Rodrigo A. Marin
*     Date  : January 25 1993
*-----------------------------------------------------------------------
 
      use posttypes
      use glob_vars

      include 'pubfunc.h'

      character prompt*(MAXPROMPTLENGTH)
      logical quit 

      write(iout, *)
      write(prompt, 100) taprw1

      taprw1 = readdouble(prompt, strlen(prompt), .TRUE., taprw1,
     +                    iunit, iout)

      write(iout, *)
      write(prompt, 200) taprn1

      taprn1 = readdouble(prompt, strlen(prompt), .TRUE., taprn1,
     +                    iunit, iout)

      write(iout, *)
      write(prompt, 300) rmolng

      quit = .FALSE.

      do while (.not. quit)
         rmolng = readdouble(prompt, strlen(prompt), .TRUE., rmolng,
     +                       iunit, iout)

        if (dabs(rmolng) .le. ZERO) then
            call reperror(17, iout)
        else
            quit = .TRUE.
        end if
      end do

      return
100   format(
     +'Please enter the taper percentage per unit for the width wall [', 
     + f8.3, ']: ')
200   format(
     +'Please enter the taper percentage per unit for the narrow wall ['
     + ,f8.3, ']: ')
300   format('Please enter the mold length [', f8.3, ']: ')
      end
