*-----------------------------------------------------------------------
* postconst.h
* 
* Definition of global constants
* 
* Author: Rodrigo Marin
*-----------------------------------------------------------------------
      parameter(MAXN=5000,   MAXEL=9000,   MAXB=3000)
c               max nodes   max elements, max boundaries

      parameter(MAXOPTIONS = 100, MAXPROMPTLENGTH = 80)
c
c     MAXOPTIONS : Maximum number of options in a choice
c     MAXPROMPTLENGTH : Maximum length of an option choice prompt string
c
      integer MAXOPTLENGTH
      
      parameter (MAXOPTLENGTH = 16)

c     MAXOPTLENGTH : Maximum length for a string used to choose an option
c                    by the user.    

      parameter(MAXCONTOURS = 52)

c     MAXCONTOURS : Maximum number of cotours

      integer MAXNUMVAR

      parameter (MAXNUMVAR = 52)

c     Maximum number of variables in binary solution file

      integer MAXSTRINGLENGTH

      parameter (MAXSTRINGLENGTH = 80)

c     MAXSTRINGLENGTH : Maximum length of a string

      integer MAXNUMVAL

      parameter (MAXNUMVAL = 60)

c     Maximum number of values selected by the user to be output in Table Form

      parameter (SELTYPE_SINGVAL = 1, SELTYPE_INTERVAL = 2, 
     +           SELTYPE_ALLVALUES = 3)

c     Value selection types: single and by intervals

      parameter (TIMETYPE_TVAL = 1, TIMETYPE_ZPOS = 2, 
     +           TIMETYPE_ITNUM = 3)

c     Iteration specification methods: By time values, by z-positions, by
c     iteration numbers

      character*(MAXSTRINGLENGTH) MESHFILEEXT

      parameter (MESHFILEEXT = '.msh')

c     MESHFILEEXT : Extension for Mesh Files.

      character*(MAXSTRINGLENGTH) BINFILEEXT

      parameter (BINFILEEXT = '.sol')

c     BINFILEEXT : Extension for Binary Solution Files.

      parameter(MAXITERATIONS = 3000)

c     MAXITERATIONS : Maximum number of iterations in solution file

      character*(MAXSTRINGLENGTH) TABLEFILEEXT 

      parameter (TABLEFILEEXT = '.tbl')

c     TABLEFILEEXT : Extension for the Table Output File

      integer TABLEFILEUNIT

      parameter (TABLEFILEUNIT = 38)
     
c     Unit for the Table File

      double precision ZERO

      parameter (ZERO = 1.D-09)

c     Zero value to be used in double precision comparisons

      double precision CROSSPLOT_DEFSCALE

      parameter (CROSSPLOT_DEFSCALE = 0.1)

c     Default value for the crossplot scale factor

      double precision DISPLOT_DEFMAG

      parameter (DISPLOT_DEFMAG = 1.0)

c     Default value for the displace plot magnification factor

      integer WIDTH_WALL_FLAG

      parameter (WIDTH_WALL_FLAG = 1)

c     Flag to identify the width mold wall

      integer NARROW_WALL_FLAG

      parameter (NARROW_WALL_FLAG = 2)

c     Flag to identify the narrow mold wall

      integer WIDTH_WALL_NODE_FLAG

      parameter (WIDTH_WALL_NODE_FLAG = 4)

c     Flag to identify nodes belonging to the width mold wall

      character*(MAXSTRINGLENGTH) MOLDTYPE

      parameter (MOLDTYPE = 'ARMCO Butler Caster')

c     Mold Type for which wall deformation is calculated

      double precision TAPER_WIDTH_DEFAULT, TAPER_NARROW_DEFAULT,
     +                 MOLD_LENGTH_DEFAULT

      parameter (TAPER_WIDTH_DEFAULT = 0.0010, 
     +           TAPER_NARROW_DEFAULT = 0.0070,
     +           MOLD_LENGTH_DEFAULT = 0.65)

c     TAPER_WIDTH_DEFAULT = Default taper percentage per unit for the width wall
c     TAPER_NARROW_DEFAULT = Default taper percentage per unit for the narrow wall
c     MOLD_LENGTH_DEFAULT  = Default mold length

      integer TLINE_LT45, TLINE_GT45

      parameter (TLINE_LT45 = 0, TLINE_GT45 = 1)

c     Line types, according to inclination; less than 45 degrees, or more than
c     45 degrees.

      integer NUMCONT_DEFVAL

      parameter (NUMCONT_DEFVAL = 10)

c     Default number of contours.

      integer LINETYPE_DASHED, LINETYPE_DOTTED, LINETYPE_CONTINUOUS
 
      parameter (LINETYPE_DASHED = 0, LINETYPE_DOTTED = 1, 
     +           LINETYPE_CONTNUOUS = 2)
c
c     Line types according to the way in which are displayed in the screen.

      integer TINTER_PLANE_EDGE, TINTER_PLANE_ELEM

      parameter (TINTER_PLANE_EDGE = 0, TINTER_PLANE_ELEM = 1)
c
c     Slice plot plane intersection type with the dominium:
c
c     TINTER_PLANE_EDGE : Plane - Edge intersection
c     TINTER_PLANE_ELEM : Plane - Element intersection.
c
      integer CONTDATAUNIT

      parameter (CONTDATAUNIT = 39)

c     CONTDATAUNIT : Output unit to write contour plot data file

      character*(MAXSTRINGLENGTH) CONTDATAFILEEXT

      parameter (CONTDATAFILEEXT = '.contdat')

c     CONTDATAFILEEXT : Extension of contour plot data file

      integer VARFMT_TYPE_1, VARFMT_TYPE_2, VARFMT_TYPE_3

      parameter (VARFMT_TYPE_1 = 0, VARFMT_TYPE_2 = 1, 
     +           VARFMT_TYPE_3 = 2)

c     Types of formats for variable value output

      integer MAXMENUOPTS 

      parameter (MAXMENUOPTS = 10)

c     Maximum number of options in graphics window pull down menus.

      integer MENU_NO_OPT_SEL

      parameter (MENU_NO_OPT_SEL = -1)

c     Value for no option selection in a graphics menu

      integer MENU_NO_MENU

      parameter (MENU_NO_MENU = -1)

c     Value for no menu in graphics display

      integer MAX_MENUBAR_OPTS

      parameter (MAX_MENUBAR_OPTS = 7)

c     Maximum number of options in graphics window menu bar.

      real MENUBAR_OPT_WIDTH_FRAC

      parameter (MENUBAR_OPT_WIDTH_FRAC = 2.0 / MAX_MENUBAR_OPTS)

c     Fraction in x occupied by each option of the menu bar.

      integer MAXMENUS

      parameter (MAXMENUS = 20)

c     Maximum number of graphics window menus.

      real READ_DEFPOSX, READ_DEFPOSY

      parameter (READ_DEFPOSX = -0.5, READ_DEFPOSY = 0.0)

c     Default position for read dialog boxes in graphics interface

      real STR_IN_WIDTH_FRAC

      parameter (STR_IN_WIDTH_FRAC = 2.0/3.0)

c     Width of the string input dialog box as a Fraction of the global window

      real INFO_MARGINX_FRAC 

      parameter (INFO_MARGINX_FRAC = 0.05)

c     Fraction of the info window width left as left and right margin 

      real STATUS_SEP_FACY

      parameter (STATUS_SEP_FACY = 1.3)

c     Separation factor between lines in the plot status display

      real PLOT_STATUS_MARGINY_SUP, PLOT_STATUS_MARGINY_INF

      parameter (PLOT_STATUS_MARGINY_SUP = 0.01,
     +           PLOT_STATUS_MARGINY_INF = 0.85)

c     Margins of the plot status window with respect to the info area window.

      integer STATE_NONGRAPH,
     +        STATE_CONPLOT_00, STATE_CONPLOT_01, STATE_CONPLOT_02,
     +        STATE_CONPLOT_03, STATE_CONPLOT_04, STATE_CONPLOT_05,
     +        STATE_MESHPLOT_00, 
     +        STATE_DISPLOT_00,
     +        STATE_CROSSPLOT_00,
     +        STATE_ANNOT_00,
     +        STATE_ANNOT_01,
     +        STATE_ANNOT_02


      parameter (STATE_NONGRAPH = 0, 
     +           STATE_CONPLOT_00 = 1, STATE_CONPLOT_01 = 2,
     +           STATE_CONPLOT_02 = 3, STATE_CONPLOT_03 = 4,
     +           STATE_CONPLOT_04 = 5, STATE_CONPLOT_05 = 6,
     +           STATE_CONPLOT_06 = 7,
     +           STATE_MESHPLOT_00 = 8, 
     +           STATE_DISPLOT_00 = 9, 
     +           STATE_DISPLOT_01 = 10,
     +           STATE_DISPLOT_02 = 11,
     +           STATE_DISPLOT_03 = 12,
     +           STATE_DISPLOT_04 = 13,
     +           STATE_DISPLOT_05 = 14,
     +           STATE_DISPLOT_06 = 15,
     +           STATE_CROSSPLOT_00 = 16,
     +           STATE_ANNOT_00 = 17,
     +           STATE_ANNOT_01 = 18,
     +           STATE_ANNOT_02 = 19,
     +           STATE_MESHPLOT_01 = 20,
     +           STATE_MESHPLOT_02 = 21,
     +           STATE_MESHPLOT_03 = 22,
     +           STATE_CROSSPLOT_01 = 23,
     +           STATE_CROSSPLOT_02 = 24,
     +           STATE_CROSSPLOT_03 = 25,
     +           STATE_CROSSPLOT_04 = 26)

c     Interaction States in the Graphics Interface.

      integer MAXNUMANNOT

      parameter (MAXNUMANNOT = 100)

c     Maximum number of annotations in a plot

      integer ENTTYPE_TEXT, ENTTYPE_LINE

      parameter (ENTTYPE_TEXT = 0, ENTTYPE_LINE = 1)

c     Entity types used in the annotation of plots.

      integer PLOTTYPE_CONPLOT, PLOTTYPE_DISPLOT, 
     +        PLOTTYPE_CROSSPLOT, PLOTTYPE_MESHPLOT

      parameter (PLOTTYPE_CONPLOT = 1, PLOTTYPE_DISPLOT = 2,
     +           PLOTTYPE_CROSSPLOT = 3, PLOTTYPE_MESHPLOT = 4)

c     Colors for active and inactive menu options

      character*(MAXSTRINGLENGTH) ACTIVE_OPTION_COLOR

      parameter (ACTIVE_OPTION_COLOR = 'white')

      character*(MAXSTRINGLENGTH) NONACTIVE_OPTION_COLOR

      parameter (NONACTIVE_OPTION_COLOR = 'gray')
