*-----------------------------------------------------------------------
      subroutine assign_variables
*-----------------------------------------------------------------------
* Assigns the variables from the current iteration in the binary solution
* file to the arrays indexed by variable number and node number, so that
* they are suitable to table format print-out.
*
* variables : Array with the indices of the variables selected by the user
* varvals   : Array of variable values for each selected variable at each
*             node at the current iteration from the binary solution file.
*
* Author: Rodrigo A. Marin
* Date  : November 26 1992
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars
 
      integer i
      integer j

      i = 1

      do while (i .le. nn)
         varvals(1, i) = t(i)
         i = i + 1
      end do 

      i = 1

      do while (i .le. nns) 
         varvals(2,i)=ut(i)
         varvals(3,i)=vt(i)
         varvals(4,i)=sigma(1,i)
         varvals(5,i)=sigma(2,i)
         varvals(6,i)=sigma(3,i)
         varvals(7,i)=sigma(4,i)
         varvals(8,i)=epsln(1,i)
         varvals(9,i)=epsln(2,i)
         varvals(10,i)=epsln(3,i)
         varvals(11,i)=epsln(4,i)
         varvals(12,i)=evp(1,i)
         varvals(13,i)=evp(2,i)
         varvals(14,i)=evp(3,i)
         varvals(15,i)=evp(4,i)
         varvals(28,i)=seff(i)
         varvals(29,i)=epefft(i)
         varvals(30,i)=eeff(i)
         varvals(31,i)=epdot(i)
         varvals(32,i)=uts(i)
         varvals(33,i)=ratio(i)
         varvals(34,i)=e0(i)
         varvals(35,i)=fa(i)
         varvals(36,i)=tle(i)
         varvals(37,i)=(sigma(1,i)+sigma(2,i)+sigma(3,i))/3.0
         varvals(39,i)=elastic(1,i)
         varvals(40,i)=elastic(2,i)
         varvals(41,i)=elastic(3,i)
         varvals(42,i)=elastic(4,i)
         varvals(43,i)=eflow(1,i)
         varvals(44,i)=eflow(2,i)
         varvals(45,i)=eflow(3,i)
         varvals(46,i)=eflow(4,i)
         varvals(47,i)=dmgcrt(1,i)
         varvals(48,i)=dmgcrt(2,i)
         varvals(49,i)=dmgcrt(3,i)
         varvals(50,i)=dmgcrt(4,i)
         varvals(51,i)=emod(i)

         i = i + 1
      end do

      i = 1

      do while (i .le. numvar)
        if ((variables(i) .ge. 16) .and. (variables(i) .le. 19)) then
            call prst(sigma, sigma2, nns, nns)

            j = 1

            do while (j .le. nns)
                  varvals(variables(i), j) = 
     +            sigma2(variables(i) - 15, j)

            j = j + 1 
            end do
        end if

        if ((variables(i) .ge. 20) .and. (variables(i) .le. 23)) then
            call prst(epsln, epsln2, nns, nns)

            j = 1

            do while (j .le. nns)
               varvals(variables(i), j) = 
     +         epsln2(variables(i) - 19, j)
               j = j + 1 
            end do
        end if

        if ((variables(i) .ge. 24) .and. (variables(i) .le. 27)) then
            call prst(evp, evp2, nns, nns)

            j = 1

            do while (j .le. nns)
               varvals(variables(i), j) = 
     +         evp2(variables(i) - 23, j)
               j = j + 1 
            end do
        end if

        if (variables(i) .eq. 38) then
            call calc_shell_thick(varvals(38, 1))

            j = 2

            do while (j .le. nn)
               varvals(38, j) = varvals(38, 1)

               j = j + 1
            end do
        end if

        i = i + 1
      end do
     
      return
      end

*-----------------------------------------------------------------------
      logical function check_variables(nvar, vars, numnonaccept,
     +                                 nonaccept, uin, uout)
*-----------------------------------------------------------------------
* nvar        : Number of variables to be checked
* vars        : Array with the indexes of the variables to be checked.
* numnonaccept: Number of non-acceptable variables
* nonaccept   : Array with the indexes of no-acceptable variables
* uin, uout   : Standard input and standard ouput unit numbers.
*
* This function returns .TRUE. if none of the indexes of the variables
* included in the vars array is present in the nonaccept array.
* Additionally, if the previous condition is met, it checks the vars 
* array looking for indexes of variables tha require special input data.
*
* Author: Rodrigo A. Marin
* Date  : March 19 1993
*-----------------------------------------------------------------------

      integer nvar
      integer vars(nvar)
      integer numnonaccept
      integer nonaccept(numnonaccept)
      integer uin, uout

      integer i
      integer j
      logical accept

      i = 1
      accept = .TRUE.

      do while (i .le. nvar)
         j = 1
 
         do while (j .le. numnonaccept)
            if (vars(i) .eq. nonaccept(j)) then
                write(uout, *)'Variable number ', vars(i)
                call reperror(21, uout)
                accept = .FALSE.
            end if

            j = j + 1
         end do

         i = i + 1
      end do
      
      if (accept) then
          i = 1

          do while (i .le. nvar)
             if (vars(i) .eq. 38) then
                 call read_shell_thick_input_data
             end if

             i = i + 1
          end do
 
          check_variables = .TRUE.
      else
          check_variables = .FALSE.
      end if

      return
      end

*-----------------------------------------------------------------------
      subroutine history_table(usol, uin, uout)
*-----------------------------------------------------------------------
* Prints binary data in table format.
* 
* Author: Rodrigo A. Marin
* Date  : November 25 1992
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      integer uin, uout, usol

      integer i
      integer j
      integer k
      integer l
      integer numval
      integer ttype
      integer seltype
      double precision values(MAXNUMVAL)
      double precision dval
      integer ival
      integer numnonaccept
      integer nonaccept(MAXNUMVAR)


      logical err
      logical quit
      logical found


      numnonaccept = 0
      quit = .FALSE.

      call open_tables_file(uin, uout)

      do while (.not. quit)
         call select_variables(numvar, variables, uin, uout)
   
         quit = check_variables(numvar, variables, numnonaccept, 
     +                          nonaccept, iunit, iout)
      end do

      call select_time(numtimevals, numzposvals, numitervals,
     +                 timevals, zposvals, itervals, timeinterval,
     +                 zposinterval, iternuminterval, 
     +                 seltype, ttype, numval, values, usol, uin, uout)
         
      if (seltype .eq. SELTYPE_SINGVAL) then
          if (ttype .eq. TIMETYPE_TVAL) then
              i = 1

              do while (i .le. numval)
                 timevals(i) = values(i)
                 i = i + 1
              end do

              numtimevals = numval
          else if (ttype .eq. TIMETYPE_ZPOS) then
                   i = 1

                   do while (i .le. numval)
                      zposvals(i) = values(i)
                      i = i + 1
                   end do

                   numzposvals = numval
          else if (ttype .eq. TIMETYPE_ITNUM) then
                   i = 1

                   do while (i .le. numval)
                      itervals(i) = idint(values(i))
                      i = i + 1
                   end do

                   numitervals = numval
          end if
      else
          if (ttype .eq. TIMETYPE_TVAL) then
              timeinterval(1) = values(1)
              timeinterval(2) = values(2)
          else if (ttype .eq. TIMETYPE_ZPOS) then
                   zposinterval(1) = values(1)
                   zposinterval(2) = values(2)
          else if (ttype .eq. TIMETYPE_ITNUM) then
                   iternuminterval(1) = idint(values(1))
                   iternuminterval(2) = idint(values(2))
          end if
      end if

      if (.not. ((numvar .eq. 1) .and. 
     +           (variables(1) .eq. 38))) then
          call select_single_nodes(nn, numnodes, nodes, uin, uout)
      end if

      call binread1(casvel, tsol, tliq, taprn1, taprw1, rmolng,
     +              rtitle, mtitle, MAXN, nn,
     +              nns, nbes, usol, uout)

      if (seltype .eq. SELTYPE_SINGVAL) then
          call sort_double_array(numval, values)
      end if

      write(TABLEFILEUNIT, 10) rtitle
      write(TABLEFILEUNIT, 5)

      call special_titles_history_table

      write(TABLEFILEUNIT, 5)

      j = 1

      write(TABLEFILEUNIT, 50)

      do while (j .le. numvar)
         k = 1

         if (.not. ((numvar .eq. 1) .and. 
     +       (variables(1) .eq. 38))) then
             do while (k .le. numnodes)
                write(TABLEFILEUNIT, 100)tbltitle1(variables(j))

                k = k + 1
             end do
         else
            write(TABLEFILEUNIT, 100)tbltitle1(variables(j))
         end if

         j = j + 1
      end do

      write(TABLEFILEUNIT, *)

      j = 1

      write(TABLEFILEUNIT, 50)

      do while (j .le. numvar)
         k = 1

         if (.not. ((numvar .eq. 1) .and. 
     +       (variables(1) .eq. 38))) then
             do while (k .le. numnodes)
                write(TABLEFILEUNIT, 100)tbltitle2(variables(j))

                k = k + 1
             end do
         else
            write(TABLEFILEUNIT, 100)tbltitle2(variables(j))
         end if

         j = j + 1
      end do

      write(TABLEFILEUNIT, *)
      write(TABLEFILEUNIT, 5)

      if (.not. ((numvar .eq. 1) .and. 
     +          (variables(1) .eq. 38))) then
          j = 1

          write(TABLEFILEUNIT, 60)
      
          do while (j .le. numvar)
             k = 1

             do while (k .le. numnodes)
                write(TABLEFILEUNIT, 200)nodes(k)

                k = k + 1
            end do 

            j = j + 1
          end do
      end if

      write(TABLEFILEUNIT, *)
      write(TABLEFILEUNIT, 5)
      write(TABLEFILEUNIT, 70)
      write(TABLEFILEUNIT, 5)

      i = 1
      err = .FALSE.
      quit = .FALSE.

      if (seltype .eq. SELTYPE_SINGVAL) then
          l = 1
      end if

      do while ((i .le. MAXITERATIONS) .and. (.not. err) .and. 
     +          (.not. quit))
         found = .FALSE.

         call binread2(t, ut, vt, sigma, epsln, evp, seff, epefft, eeff,
     +                 epdot, uts, ratio, e0, fa, tle, 
     +                 elastic, eflow, dmgcrt, emod, ctime, nn, nns,
     +                 itnum, err, usol, uout)

         if (.not. err) then
             if (ttype .eq. TIMETYPE_TVAL) then
                 dval = ctime
             else if (ttype .eq. TIMETYPE_ZPOS) then
                 dval = ctime * casvel
             else if (ttype .eq. TIMETYPE_ITNUM) then
                 ival = itnum
             end if

             if (seltype .eq. SELTYPE_SINGVAL) then
                 if ((ttype .eq. TIMETYPE_TVAL) .or.
     +               (ttype .eq. TIMETYPE_ZPOS)) then
                     if (dval .ge. values(l)) then
                         found = .TRUE.
                         l = l + 1
                         quit = (l .gt. numval)
                     end if
                 else
                     if (ival .ge. int(values(l))) then
                         found = .TRUE.
                         l = l + 1
                         quit = (l .gt. numval)
                     end if
                 end if
             else if (seltype .eq. SELTYPE_INTERVAL) then
                 if ((ttype .eq. TIMETYPE_TVAL) .or.
     +               (ttype .eq. TIMETYPE_ZPOS)) then
                     found = (dval .ge. values(1)) .and.  
     +                       (dval .le. values(2))

                     if (.not. found) then
                         quit = dval .gt. values(2) 
                     end if
                 else
                     found = (ival .ge. int(values(1))) .and.
     +                       (ival .le. int(values(2)))

                     if (.not. found) then
                         quit = ival .gt. int(values(2))
                     end if
                 end if
             else if (seltype .eq. SELTYPE_ALLVALUES) then
                      found = .TRUE.
             end if

             if (found) then
                 call assign_variables

                 write(TABLEFILEUNIT, 300) ctime, ctime*casvel
                 j = 1

                 do while (j .le. numvar)
                    k = 1

                    if (.not. ((numvar .eq. 1) .and.
     +                         (variables(1) .eq. 38))) then
                    do while (k .le. numnodes)
                       if (((variables(j) .ge. 4) .and. 
     +                      (variables(j) .le. 7)) .or.
     +                     ((variables(j) .ge. 16) .and. 
     +                      (variables(j) .le. 19)) .or.
     +                     (variables(j) .eq. 28) .or. 
     +                     (variables(j) .eq. 35) .or.
     +                     (variables(j) .eq. 37) .or. 
     +                     (variables(j) .eq. 1) .or.
     +                     (variables(j) .eq. 38) .or.
     +                     (variables(j) .eq. 51)) then
                           write(TABLEFILEUNIT, 400)
     +                     varvals(variables(j), nodes(k))
                       else if (((variables(j) .ge. 2) .and. 
     +                           (variables(j) .le. 3)) .or.
     +                          ((variables(j) .ge. 8) .and. 
     +                           (variables(j) .le. 15)) .or.
     +                          ((variables(j) .ge. 20) .and. 
     +                           (variables(j) .le. 27)) .or.
     +                          ((variables(j) .ge. 29) .and. 
     +                           (variables(j) .le. 30)) .or.
     +                           (variables(j) .eq. 32) .or.
     +                           (variables(j) .eq. 33) .or.
     +                          (variables(j) .eq. 34) .or. 
     +                          (variables(j) .eq. 36) .or.
     +                          (variables(j) .eq. 39) .or.
     +                          (variables(j) .eq. 40) .or.
     +                          (variables(j) .eq. 41) .or.
     +                          (variables(j) .eq. 42) .or.
     +                          (variables(j) .eq. 43) .or.
     +                          (variables(j) .eq. 44) .or.
     +                          (variables(j) .eq. 45) .or.
     +                          (variables(j) .eq. 46) .or.
     +                          (variables(j) .eq. 47) .or.
     +                          (variables(j) .eq. 48) .or.
     +                          (variables(j) .eq. 49) .or.
     +                          (variables(j) .eq. 50)) then
                                write(TABLEFILEUNIT, 500)
     +                          varvals(variables(j), nodes(k))
                       else
                           write(TABLEFILEUNIT, 600)
     +                     varvals(variables(j), nodes(k))
                       end if

                       k = k + 1
                    end do
                    else
                       if (((variables(j) .ge. 4) .and. 
     +                      (variables(j) .le. 7)) .or.
     +                     ((variables(j) .ge. 16) .and. 
     +                      (variables(j) .le. 19)) .or.
     +                     (variables(j) .eq. 28) .or. 
     +                     (variables(j) .eq. 37) .or. 
     +                     (variables(j) .eq. 1) .or.
     +                     (variables(j) .eq. 35) .or.
     +                     (variables(j) .eq. 38) .or.
     +                     (variables(j) .eq. 51)) then
                           write(TABLEFILEUNIT, 400)
     +                     varvals(variables(j), 1)
                       else if (((variables(j) .ge. 2) .and. 
     +                           (variables(j) .le. 3)) .or.
     +                          ((variables(j) .ge. 8) .and. 
     +                           (variables(j) .le. 15)) .or.
     +                          ((variables(j) .ge. 20) .and. 
     +                           (variables(j) .le. 27)) .or.
     +                          ((variables(j) .ge. 29) .and. 
     +                           (variables(j) .le. 30)) .or.
     +                          (variables(j) .eq. 32) .or.
     +                          (variables(j) .eq. 33) .or.
     +                          (variables(j) .eq. 34) .or. 
     +                          (variables(j) .eq. 36) .or.
     +                          (variables(j) .eq. 39) .or.
     +                          (variables(j) .eq. 40) .or.
     +                          (variables(j) .eq. 41) .or.
     +                          (variables(j) .eq. 42) .or.
     +                          (variables(j) .eq. 43) .or.
     +                          (variables(j) .eq. 44) .or.
     +                          (variables(j) .eq. 45) .or.
     +                          (variables(j) .eq. 46) .or.
     +                          (variables(j) .eq. 47) .or.
     +                          (variables(j) .eq. 48) .or.
     +                          (variables(j) .eq. 49) .or.
     +                          (variables(j) .eq. 50)) then
                                write(TABLEFILEUNIT, 500)
     +                          varvals(variables(j), 1)
                       else
                           write(TABLEFILEUNIT, 600)
     +                     varvals(variables(j), 1)
                       end if
                    end if

                    j = j + 1
                 end do
                 
                 write(TABLEFILEUNIT, *)
             end if
         end if
         i = i + 1
      end do  

      close(TABLEFILEUNIT)

      return
5     format('#')
10    format('# ', A80)
50    format('#                             ', $)
60    format('# Nodes                       ', $)
70    format('#    Time            Z-Pos')
100   format('  ', A14, $)
200   format('      ', '(', i4, ')', '    ', $)
300   format(E14.5, '  ', E14.5, $)
400   format('  ', f14.4, $)
500   format('  ', f15.9, $)
600   format('  ', e14.4, $)
      end

*-----------------------------------------------------------------------
      subroutine nodes_table(usol, umesh, uin, uout)
*-----------------------------------------------------------------------
* Generates a node vs variables values table, at a user specified time.
* 
* Author: Rodrigo A. Marin
* Date  : December 14 1992
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      integer uin, uout, usol, umesh

      integer j
      integer k
      integer seltype
      integer antnode
      integer nextnode
      integer singnode
      integer step
      integer numnonaccept
      integer nonaccept(MAXNUMVAR)
      double precision distance

      logical quit

      numnonaccept = 1
      nonaccept(1) = 38
      quit = .FALSE.

      call open_tables_file(uin, uout)

      do while (.not. quit)
         call select_variables(numvar, variables, uin, uout) 

         quit = check_variables(numvar, variables, numnonaccept, 
     +                          nonaccept, iunit, iout)
      end do

      call askiter(time,newtime,lastzpos,newzpos,casvel,lastitnum,
     +             newitnum,iunit,iout,ibin,iecho,timetype)

      call select_nodes(nn, seltype, numnodes, nodes, nodeinterval, 
     +                  step, uin, uout)

      call find_iteration(timetype, time, newtime, lastzpos, newzpos,
     +                    casvel, lastitnum, newitnum,
     +                    t,ut,vt,sigma,epsln,evp,seff,epefft,eeff,
     +                    epdot,uts,ratio,e0,fa,tle,elastic,eflow,
     +                    dmgcrt,emod,ctime,nn,nns,
     +                    itnum,zpos,ibin,iout)

      write(uout, *)
      write(uout,*)'Printout table will be done for the following data:'
      write(uout, *)
      write(uout, 700)'Iteration : ', itnum
      write(uout, 800)'Time      : ', ctime
      write(uout, 800)'Z-position: ', zpos

      write(TABLEFILEUNIT, 10) rtitle
      write(TABLEFILEUNIT, 5)

      write(TABLEFILEUNIT, 40)ctime, zpos, itnum
      write(TABLEFILEUNIT, 5)

      write(TABLEFILEUNIT, 70)

      j = 1

      do while (j .le. numvar)
         write(TABLEFILEUNIT, 100)tbltitle1(variables(j))

         j = j + 1
      end do

      write(TABLEFILEUNIT, *)
      write(TABLEFILEUNIT, 50)

      j = 1

      do while (j .le. numvar)
         write(TABLEFILEUNIT, 100)tbltitle2(variables(j))

         j = j + 1
      end do

      write(TABLEFILEUNIT, *)
      write(TABLEFILEUNIT, 5)

      call assign_variables

      j = 1
      distance = 0.0

      if (seltype .eq. SELTYPE_SINGVAL) then
          singnod = 1
          nextnode = nodes(singnod)
      else if (seltype .eq. SELTYPE_INTERVAL) then
               nextnode = nodeinterval(1)
      else if (seltype .eq. SELTYPE_ALLVALUES) then
               nextnode = 1
      end if

      antnode = nextnode

      quit = .FALSE.

      do while (.not. quit)
         k = 1

         write(TABLEFILEUNIT, 300) nextnode, distance

         do while (k. le. numvar)
            if (((variables(k) .ge. 4) .and. 
     +          (variables(k) .le. 7)) .or.
     +          ((variables(k) .ge. 16) .and. 
     +           (variables(k) .le. 19)) .or.
     +          (variables(k) .eq. 28) .or. 
     +          (variables(k) .eq. 35) .or.
     +          (variables(k) .eq. 37) .or. 
     +          (variables(k) .eq. 51) .or.
     +          (variables(k) .eq. 1)) then
                write(TABLEFILEUNIT, 400)
     +          varvals(variables(k), nextnode)
            else if (((variables(k) .ge. 2) .and. 
     +               (variables(k) .le. 3)) .or.
     +               ((variables(k) .ge. 8) .and. 
     +               (variables(k) .le. 15)) .or.
     +               ((variables(k) .ge. 20) .and. 
     +                (variables(k) .le. 27)) .or.
     +               ((variables(k) .ge. 29) .and. 
     +                (variables(k) .le. 30)) .or.
     +                (variables(k) .eq. 32) .or.
     +                (variables(k) .eq. 33) .or.
     +               (variables(k) .eq. 34) .or. 
     +               (variables(k) .eq. 36) .or.
     +               (variables(k) .eq. 39) .or.
     +               (variables(k) .eq. 40) .or.
     +               (variables(k) .eq. 41) .or.
     +               (variables(k) .eq. 42) .or.
     +               (variables(k) .eq. 43) .or.
     +               (variables(k) .eq. 44) .or.
     +               (variables(k) .eq. 45) .or.
     +               (variables(k) .eq. 46) .or.
     +               (variables(k) .eq. 47) .or.
     +               (variables(k) .eq. 48) .or.
     +               (variables(k) .eq. 49) .or.
     +               (variables(k) .eq. 50)) then
                     write(TABLEFILEUNIT, 500)
     +               varvals(variables(k), nextnode)
            else
                write(TABLEFILEUNIT, 600)
     +          varvals(variables(k), nextnode)
            end if
            
            k = k + 1
         end do 
        
         write(TABLEFILEUNIT, *)

         antnode = nextnode

         if (seltype .eq. SELTYPE_SINGVAL) then
             if (singnod .eq. numnodes) then
                 quit = .TRUE.
             else
                 singnod = singnod + 1
                 nextnode = nodes(singnod)
             end if
         else if (seltype .eq. SELTYPE_INTERVAL) then
                  if ((nextnode + step - nodeinterval(1)) *
     +                (nextnode + step - nodeinterval(2)) .gt. 0) then
                      quit = .TRUE.
                  else
                      nextnode = nextnode + step
                  end if
         else if (seltype .eq. SELTYPE_ALLVALUES) then
                  if (nextnode + 1 .gt. nn) then
                      quit = .TRUE.
                  else
                      nextnode = nextnode + 1
                  end if
         end if
 
         if (.not. quit) then
             distance = distance + dabs(sqrt((x(nextnode) - 
     +                                        x(antnode))**2 +
     +                                       (y(nextnode) -
     +                                        y(antnode))**2))
         end if
      end do

      close(TABLEFILEUNIT)

      return
5     format('#')
10    format('# ', A80)
40    format('# Time= ', E14.5, '  Z-pos= ', E14.5,
     +       '  Iteration No.= ', i8)
50    format('#                      ', $)
70    format('# Node      Distance   ', $)
100   format('  ', A14, $)
300   format(i5, '    ', f14.4, $)
400   format('  ', f14.4, $)
500   format('  ', f15.9, $)
600   format('  ', e14.4, $)
700   format(A, I8)
800   format(A, E14.5)
      end

*-----------------------------------------------------------------------
      subroutine open_tables_file(uin, uout)
*-----------------------------------------------------------------------
* Reads from the user the filename for table form output and opens it
*
* Author: Rodrigo A. Marin
* Date  : November 25 1992
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      integer uin, uout

      character filename*(MAXSTRINGLENGTH)
      character prompt*(MAXPROMPTLENGTH)

      logical quit 

      write(uout, *)

      quit = .FALSE.

      do while (.not. quit)
50       prompt =
     +   'Please enter the name of the Output File (without .tbl):'
         
         filename = readstring(prompt, strlen(prompt), .FALSE., ' ',
     +                         uin, uout)

         filename = filename(1:strlen(filename)) // TABLEFILEEXT

         open(unit = TABLEFILEUNIT, file = filename, 
     +        status = 'unknown', err = 50)

         quit = .TRUE.

         if (.not. quit) then
             call reperror(6, uout)
         end if
      end do 

      write(uout, *)

      return
      end

*-----------------------------------------------------------------------
      subroutine print_tables(usol, umesh, uin, uout, sol_file_open)
*-----------------------------------------------------------------------
* Asks the user to select a table type, and dispatchs the corresponding
* subroutine to generate the table. This is done until the user selects
* to go on to the next menu.
*
* Author: Rodrigo A. Marin
* Date  : December 14 1992
*-----------------------------------------------------------------------

      integer uin, usol, umesh, uout
      logical sol_file_open

      integer option
      logical quit

      if (.not. sol_file_open) then
          call open_solution_file(usol, uin, uout)
      end if

      quit = .FALSE.

      do while (.not. quit)
         call readmenu(9, option, uin, uout)

         if (option .eq. 1) then
             call history_table(usol, uin, uout)
         else if (option .eq. 2) then
             call nodes_table(usol, umesh, uin, uout)
         else if (option .eq. 3) then
             call sliceplot
         else if (option .eq. 4) then
             quit = .TRUE.
         end if
      end do

      return
      end

*-----------------------------------------------------------------------
      subroutine select_nodes(nn, seltype, numnodsel, nodsel, 
     +                        nodeinterval, step, uin, uout)
*-----------------------------------------------------------------------
* Aks the user to select the nodes included in the table output.
*
* nn        : Number of nodes in problem topology
* seltype   : Node number selection type.
* numnodsel : Number of nodes selected by the user
* nodsel    : Array with numbers of selected nodes if single nodes were 
*             selected.
* nodeinterval: Interval of desired nodesi, if an interval was chosen.
* step      : Step between node numbers if interval was specified.
*
* Author: Rodrigo A. Marin
* Date  : December 30 1992
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      integer nn, numnodsel, uin, uout
      integer nodsel(nn)
      integer nodeinterval(2)
      
      integer option
      integer i
      integer j
      integer seltype
      integer step 
      integer firstlim
      integer secondlim
      logical quit

      character prompt*(MAXPROMPTLENGTH) 
      
      write(uout, *)
      write(uout, *)'      NODE SELECTION'
      write(uout, *)'      **************'
      write(uout, *)

      step = 1
      call readmenu(8, seltype, uin, uout)

      if (seltype .eq. SELTYPE_SINGVAL) then
          call select_single_nodes(nn, numnodsel, nodsel, uin, uout)
      else if (seltype .eq. SELTYPE_INTERVAL) then
          quit = .FALSE.

          do while (.not. quit)
             write(prompt, 100) nn
        
             firstlim = readinteger(prompt, strlen(prompt), .TRUE.,
     +                              nodeinterval(1), uin, uout)

             if ((firstlim .ge. 1) .and. (firstlim .le. nn)) then
                 write(prompt, 200) nn

                 secondlim = readinteger(prompt, strlen(prompt),
     +                                   .TRUE., nodeinterval(2), 
     +                                   uin, uout)

                 if ((secondlim .ge. 1) .and.
     +               (secondlim .le. nn)) then

                     do while (.not. quit)
                        write(prompt, 300) step 

                        step = readinteger(prompt,
     +                                     strlen(prompt),
     +                                     .TRUE., step,
     +                                     uin, uout)

                        if (step .le. 0) then
                            call reperror(15, uout)
                            step = 1
                        else
                            quit = .TRUE.
                        end if
                     end do

                     if (secondlim .lt. firstlim) then
                         step = -step
                     end if
                 else
                     call reperror(9, uout)
                 end if
             else
                 call reperror(11, uout)
             end if
          end do

          nodeinterval(1) = firstlim
          nodeinterval(2) = secondlim
      else if (seltype .eq. SELTYPE_ALLVALUES) then

      end if 

      return
100   format('First limit of node interval (between 1 and ',
     +       i5, '): ')
200   format('Second limit of node interval (between 1 and ',
     +       i5, '): ')
300   format('Step between nodes [',i5,']: ')
      end
          
*-----------------------------------------------------------------------
      subroutine select_single_nodes(nn, numnodsel, nodsel, uin, uout)
*-----------------------------------------------------------------------
* Aks the user to select the nodes included in the table output.
*
* nn        : Number of nodes in problem topology
* numnodsel : Number of nodes selected by the user
* nodsel    : Array with numbers of selected nodes
*
* Author: Rodrigo A. Marin
* Date  : November 26 1992
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      integer nn, numnodsel, uin, uout
      integer nodsel(nn)
      integer newnodsel(MAXN)
      integer newnumnodsel
      
      integer option
      integer i
      logical quit
      logical err

      character prompt*(MAXPROMPTLENGTH) 
      
      write(uout, *)
      prompt = ' '

      quit = .FALSE.

      write(uout, *) 
     +'Please enter the desired node numbers separated by commas'

      i = 1

      do while (i .le. numnodsel)
         if (i .eq. 1) then
             write(uout, 100) nodsel(i)
         else
             write(uout, 200) nodsel(i)
         end if

         if (i .eq. numnodsel) then
             write(uout, 300)
         end if

         i = i + 1
      end do

      do while (.not. quit)
         i = 1

         do while (i .le. numnodsel)
            newnodsel(i) = nodsel(i)
            i = i + 1
         end do

         call readninteger(prompt, strlen(prompt), nn, .TRUE.,
     +                     newnumnodsel, newnodsel, uin, uout)
             
         if ((newnumnodsel .eq. 0) .and. (numnodsel .eq. 0)) then
             call reperror(10, uout)
         else
             err = .FALSE.
             i = 1

             if (newnumnodsel .eq. 0) then
                 newnumnodsel = numnodsel
             end if

             do while ((.not. err) .and. (i .le. newnumnodsel))
                if ((newnodsel(i) .le. 0) .or. 
     +              (newnodsel(i) .gt. nn)) then
                    call reperror(11, uout)
                    err = .TRUE.
                end if

                i = i + 1 
             end do

             quit = .not. err
         end if
      end do

      numnodsel = newnumnodsel

      i = 1

      do while (i .le. numnodsel)
         nodsel(i) = newnodsel(i)
         i = i + 1
      end do

      return
100   format('[', i4, $)
200   format(', ', i4, $)
300   format(']:')
      end

*-----------------------------------------------------------------------
      subroutine select_time(numtimevals, numzposvals, numitervals,
     +                       timevals, zposvals, itervals, timeinterval,
     +                       zposinterval, iternuminterval, 
     +                       seltype, timetype, numval, values, ubin,
     +                       uin, uout)
*-----------------------------------------------------------------------
* Asks the user to select the time for which variables will be output
*
* numtimevals: Number of default single time values.
* numzposvals: Number of default z-position single values.
* numitervals: Number of default iteration single numbers.
* timevals   : Array with default single time values.
* zposvals   : Array with default z-position values.
* itervals   : Array with default single iteration numbers.
* timeinterval   : Default time interval.
* zposinterval   : Default z-position interval.
* iternuminterval: Default iteration number interval.
* seltype : Indicates how the time was specified:
*           SELTYPE_SINGVAL  : Single values 
*           SELTYPE_INTERVAL : Interval 
*
* timetype: Type of value specified:
*           TIMETYPE_TVAL  : Time values
*           TIMETYPE_ZPOS  : Z-positions
*           TIMETYPE_ITNUM : Iteration numbers
*
* numval  : Number of values selected if timetype = SELTYPE_SINGVAL 
* values  : Array of selected values
*
* Author: Rodrigo A. Marin
* Date  : November 25 1992
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      integer numtimevals, numzposvals, numitervals
      double precision timevals(MAXNUMVAL), zposvals(MAXNUMVAL)
      integer itervals(MAXNUMVAL)
      double precision timeinterval(2), zposinterval(2)
      integer iternuminterval(2)
      integer seltype, numval, timetype, ubin, uin, uout

      integer numdefval
      
      double precision values(MAXNUMVAL)

      integer option 
      integer i
      logical quit

      character prompt*(MAXPROMPTLENGTH)

      quit = .FALSE.

      write(uout, *)'       TIME SELECTION'
      write(uout, *)'       **************'
      write(uout, *)

      do while (.not. quit)
         call readmenu(5, timetype, uin, uout)

         if ((timetype .ge. TIMETYPE_TVAL) .and. 
     +       (timetype .le. TIMETYPE_ITNUM)) then
             quit = .TRUE.
         else 
             call showiter(ubin, uin, uout)
         end if 
      end do

      call readmenu(8, seltype, uin, uout)

      if (seltype .eq. SELTYPE_SINGVAL) then
          if (timetype .eq. TIMETYPE_TVAL) then
              write(uout, *) 
     +        'Please enter the time values separated by commas '
              numdefval = numtimevals
          else if (timetype .eq. TIMETYPE_ZPOS) then
               write(uout, *)
     +         'Please enter the Z-position values separated by '
     +         //'commas '
               numdefval = numzposvals
          else if (timetype .eq. TIMETYPE_ITNUM) then
               write(uout, *) 
     +         'Please enter the iteration numbers separated by '
     +         // 'commas: '
               numdefval = numitervals
          end if

          i = 1
          do while (i .le. numdefval)
             if (i .eq. 1) then
                 if (timetype .eq. TIMETYPE_TVAL) then
                     write(uout, 100) timevals(1)
                 else if (timetype .eq. TIMETYPE_ZPOS) then
                          write(uout, 100) zposvals(1)
                 else if (timetype .eq. TIMETYPE_ITNUM) then
                          write(uout, 400) itervals(1)
                 end if
             else
                 if (timetype .eq. TIMETYPE_TVAL) then
                     write(uout, 200) timevals(i)
                 else if (timetype .eq. TIMETYPE_ZPOS) then
                          write(uout, 200) zposvals(i)
                 else if (timetype .eq. TIMETYPE_ITNUM) then
                          write(uout, 500) itervals(i)
                 end if
             end if

             if (i .eq. numtimevals) then
                 write(uout, 300)
             end if

             i = i + 1
          end do

          if (timetype .eq. TIMETYPE_TVAL) then
              i = 1

              do while (i .le. numtimevals)
                  values(i) = timevals(i)
                  i = i + 1
              end do
          else if (timetype .eq. TIMETYPE_ZPOS) then
                   i = 1

                   do while (i .le. numzposvals)
                       values(i) = zposvals(i)
                       i = i + 1
                   end do
          else if (timetype .eq. TIMETYPE_ITNUM) then
                   i = 1

                   do while (i .le. numitervals)
                       values(i) = itervals(i)
                       i = i + 1
                   end do
          end if
 
          prompt = ' '

          quit = .FALSE.

          do while (.not. quit)
             call readndouble(prompt, strlen(prompt), MAXNUMVAL,
     +                        .TRUE., numval, values, uin, uout)

             if (timetype .eq. TIMETYPE_TVAL) then
                 if ((numtimevals .eq. 0) .and. (numval .eq. 0)) then
                     call reperror(22, uout)
                 else
                     if (numval .eq. 0) then
                         numval = numtimevals
                     end if
                     quit = .TRUE.
                 end if 
             else if (timetype .eq. TIMETYPE_ZPOS) then
                      if ((numzposvals .eq. 0) .and. (numval .eq. 0)) 
     +                then
                          call reperror(22, uout)
                      else
                          if (numval .eq. 0) then
                              numval = numzposvals
                          end if
                          quit = .TRUE.
                      end if
             else if (timetype .eq. TIMETYPE_ITNUM) then
                      if ((numitervals .eq. 0) .and. (numval .eq. 0))
     +                then
                          call reperror(22, uout)
                      else
                          if (numval .eq. 0) then
                              numval = numitervals
                          end if
                          quit = .TRUE.
                      end if
             end if
          end do
      else if (seltype .eq. SELTYPE_INTERVAL) then
          quit = .FALSE.

          do while (.not. quit)
             if (timetype .eq. TIMETYPE_TVAL) then
                 write(prompt, 600) timeinterval(1)
                 values(1) = readdouble(prompt, strlen(prompt), .TRUE.,
     +                                  timeinterval(1), uin, uout)
             else if (timetype .eq. TIMETYPE_ZPOS) then
                      write(prompt, 600) zposinterval(1)
                      values(1) = readdouble(prompt, strlen(prompt), 
     +                                       .TRUE., zposinterval(1),
     +                                       uin, uout)
             else if (timetype .eq. TIMETYPE_ITNUM) then
                      write(prompt, 700) iternuminterval(1)
                      values(1) = dble(readinteger(prompt, 
     +                                 strlen(prompt), .TRUE.,
     +                                 iternuminterval(1), uin, uout))
             end if

             prompt = 'Enter upper limit of time interval:'

             if (timetype .eq. TIMETYPE_TVAL) then
                 write(prompt, 800) timeinterval(2)
                 values(2) = readdouble(prompt, strlen(prompt), .TRUE.,
     +                                  timeinterval(2), uin, uout)
             else if (timetype .eq. TIMETYPE_ZPOS) then
                      write(prompt, 800) zposinterval(2)
                      values(2) = readdouble(prompt, strlen(prompt), 
     +                                       .TRUE., zposinterval(2),
     +                                       uin, uout)
             else if (timetype .eq. TIMETYPE_ITNUM) then
                      write(prompt, 900) iternuminterval(2)
                      values(2) = dble(readinteger(prompt, 
     +                                 strlen(prompt),
     +                                 .TRUE., iternuminterval(2),
     +                                 uin, uout))
             end if

             if (values(1) .gt. values(2)) then
                 call reperror(9, uout)
             else
                 quit = .TRUE.
             end if
          end do
      else if (seltype .eq. SELTYPE_ALLVALUES) then
           
      end if 

      return
100   format('[', E14.5, $)
200   format(', ', E14.5, $)
300   format(']: ', $)
400   format('[', i4, $)
500   format(', ', i4, $)
600   format('Please enter the lower limit of interval [', E14.5, ']:')
700   format('Please enter the lower limit of interval [', I4, ']:')
800   format('Please enter the upper limit of interval [', E14.5, ']:')
900   format('Please enter the upper limit of interval [', I4, ']:')
      end

*-----------------------------------------------------------------------
      subroutine select_variables(numvar, variables, uin, uout)
*-----------------------------------------------------------------------
* Asks the user to select the variables to be output in table form
*
* numvar   : Number of variables selected
* variables: Array with indices of the variables to output
*
* Author: Rodrigo A. Marin
* Date  : November 25 1992
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      integer numvar, uin, uout
      integer variables(MAXNUMVAR)
      integer newvariables(MAXNUMVAR)

      character prompt*(MAXPROMPTLENGTH)
      character nvarstr*(MAXSTRINGLENGTH)

      integer i
      integer newnumvar

      logical quit
      logical err

      write(uout, *)'**********************************************'
      write(uout, *)'                VARIABLES MENU'
      write(uout, *)'**********************************************'
      write(uout, *)
          write(uout, *)'1. Temperature                    ',
     +                  '31. Plastic strain rate'
          write(uout, *)'2. U-displacement                 ',
     +                  '32. Ult. Ten. Strength'
          write(uout, *)'3. V-displacement                 ',
     +                  '33. Stress / UTS'
          write(uout, *)'4. X-stress                       ',
     +                  '34. Thermal Strain'
          write(uout, *)'5. Y-stress                       ',
     +                  '35. Fraction alpha'
          write(uout, *)'6. Z-stress                       ',
     +                  '36. TLExp. coef.'
          write(uout, *)'7. XY-stress                      ',
     +                  '37. Hydrostatic Stress'
          write(uout, *)'8. X-strain (total)               ',
     +                  '38. Shell Thickness'
          write(uout, *)'9. Y-strain (total)               ',
     +                  '39. Elastic Strain X'
          write(uout, *)'10. Z-strain (total)              ',
     +                  '40. Elastic Strain Y'
          write(uout, *)'11. XY-strain (total)             ',
     +                  '41. Elastic Strain Z'
          write(uout, *)'12. X plastic strain              ',
     +                  '42. Elastic Strain XY'
          write(uout, *)'13. Y-plastic strain              ',
     +                  '43. Flow Strain X'
          write(uout, *)'14. Z-plastic strain              ',
     +                  '44. Flow Strain Y'
          write(uout, *)'15. XY-plastic strain             ',
     +                  '45. Flow Strain Z'
          write(uout, *)'16. Principal stress 1            ',
     +                  '46. Flow Strain XY'
          write(uout, *)'17. Principal stress 2            ',
     +                  '47. Damage criteria1'
          write(uout, *)'18. Principal stress 3            ',
     +                  '48. Damage criteria2'
          write(uout, *)'19. Principal shear stress        ',
     +                  '49. Damage criteria3'
          write(uout, *)'20. Principal strain 1            ',
     +                  '50. Damage criteria4'
          write(uout, *)'21. Principal strain 2            ',
     +                  '51. Elastic Modulus(MPa)'
          write(uout, *)'22. Principal strain 3            ' 
          write(uout, *)'23. Principal shear strain        '
          write(uout, *)'24. Principal plastic strain 1    '
          write(uout, *)'25. Principal plastic strain 2    '
          write(uout, *)'26. Principal plastic strain 3    '
          write(uout, *)'27. Principal plastic shear strain'
          write(uout, *)'28. Effective Stress              '
          write(uout, *)'29. Effective plastic strain      '
          write(uout, *)'30. Effective elastic strain      '


      write(uout, *)
      write(uout, *) 
     +'Please enter the variables to output separated by commas'

      if (numvar .gt. 0) then
          i = 1
          prompt = '['

          do while (i .le. numvar)
             if (i .eq. 1) then
                 write(nvarstr, 100) variables(i)
             else
                 write(nvarstr, 200) variables(i)
             end if
            
             prompt = prompt(1:strlen(prompt)) // nvarstr
             i = i + 1
          end do

          prompt = prompt(1:strlen(prompt)) // ']: '
          write(uout, *) prompt
      end if

      quit = .FALSE.
      prompt = ' '

      do while (.not. quit)
         i = 1

         do while (i .le. numvar)
            newvariables(i) = variables(i)
            i = i + 1
         end do
      
             call readninteger(prompt, strlen(prompt), MAXNUMVAR, 
     +                         .TRUE., newnumvar, newvariables, 
     +                         uin, uout)

             if ((newnumvar .le. 0) .and. (numvar .eq. 0)) then
                 call reperror(7, uout)
             else
                 i = 1
                 err = .FALSE.

                 if (newnumvar .eq. 0) then
                     newnumvar = numvar
                 end if

                 do while ((i .le. newnumvar) .and. (.not. err))
                    if ((newvariables(i) .lt. 1) .or. 
     +                  (newvariables(i) .gt. MAXNUMVAR)) then
                        call reperror(8, uout)
                        err = .TRUE.
                    end if
                    i = i + 1
                 end do
                 quit = .not. err
             end if
      end do

      numvar = newnumvar

      i = 1

      do while (i .le. numvar)
         variables(i) = newvariables(i)
         i = i + 1
      end do

      write(uout, *)
      return
100   format(i2)
200   format(' ,', i2)
      end

*-----------------------------------------------------------------------
      subroutine special_titles_history_table
*-----------------------------------------------------------------------

*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      integer i

      i = 1

      do while (i .le. numvar)
         if (variables(i) .eq. 38) then
             write(TABLEFILEUNIT, 100) x1st, y1st, x2st, y2st
             write(TABLEFILEUNIT, 200) frac_tsol
         end if

         i = i + 1
      end do

      return
5     format('#')
100   format('# Shell thickness calculated on ', F8.3, ', ', F8.3, 
     +       ' - ', F8.3, ', ', F8.3)
200   format('# Fraction of solid temperature: ', F8.3)
      end
