*-----------------------------------------------------------------------
      subroutine calc_intersections(node, nel, x, y, nnodes, tline, 
     +                              x1, y1, x2, y2, a, b, c,
     +                              intersecs, nedgeinter)
*-----------------------------------------------------------------------
* node     : Array with connectivity information.
* nel      : Number of finite elements in the domain.
* x, y     : x and y coordinates of the domain nodes.
* nnodes   : Number of nodes in the domain.
* tline    : Type of line that defines the cutting plane.
* x1, y1   : Coordinates of the first end-point of the line defining the
*            cutting plane.
* x2, y2   : Coordinates of the second end-point of the line defining the
*            cutting plane.
* a, b, c  : Coefficients of the implicit equation of the line.
* intersecs: Array of intersections between the plane and the domain.
* nedgeinter: Number of intersections.
*
* This subroutine stores in the nedgeinter parameter the number of intersections
* found between the cutting plane and the edges of the domain. Three node elements
* are assumed. The intersecs array is filled with the data corresponding to each
* of the intersections.
*
* Author: Rodrigo A. Marin
* Date  : March 14 1993
*-----------------------------------------------------------------------

      use posttypes

      include 'postconst.h'

      integer nel, nnodes
      integer node(3, nel)
      double precision x(nnodes), y(nnodes)
      integer tline
      double precision x1, y1, x2, y2
      double precision a, b, c

      type (intersection) intersecs(nel+1)

      integer nedgeinter

      integer i, j, k, l
      integer nsing
      integer ninter
      integer n1, n2
      
      integer single_ints(MAXEL+1)
      integer nsingle_ints
      integer numint

      double precision t11, t12, t21, t22

      i = 1
      nedgeinter = 0
      nsingle_ints = 0

      do while (i .le. nel)
         j = 1

         do while (j .le. 3)
            n1 = node(j, i)
            n2 = node(mod(j, 3) + 1, i)

            if (n1 .gt. n2) then
                call calc_inter(x(n1), y(n1), x(n2), y(n2),
     +                          x1, y1, x2, y2, tline,
     +                          a, b, c, ninter,
     +                          t11, t12, t21, t22)

                if (ninter .eq. 1) then
                    if ((t11 .ge. 0.0) .and.
     +                  (t11 .le. 1.0)) then
                        if ((t11 .le. ZERO) .or.
     +                      ((1.0 - t11) .le. ZERO)) then
                            if (t11 .le. ZERO) then
                                nsing = n1
                            else
                                nsing = n2
                            end if
                                 
                            k = 1

                            do while ((k .le. nsingle_ints) .and.
     +                                (single_ints(k) .ne. nsing))
                               k = k + 1
                            end do

                            if (k .gt. nsingle_ints) then
                                nsingle_ints = nsingle_ints + 1
                                single_ints(nsingle_ints) = nsing
                                nedgeinter = nedgeinter + 1

                                call insert_edge_intersection(intersecs,
     +                               nedgeinter, i, j, 1, t11, 0.0, t21, 
     +                               0.0)
                            end if
                        else
                            nedgeinter = nedgeinter + 1

                            call insert_edge_intersection(intersecs,
     +                           nedgeinter, i, j, 1, t11, 0.0, t21, 
     +                           0.0)
                        end if
                    end if
                else 
                    if (ninter .eq. 2) then
                        l = 1

                        numint = 0

                        do while (l .le. 2)
                           if (l .eq. 1) then
                               nsing = n1
                           else
                               nsing = n2
                           end if

                           k = 1

                           do while ((k .le. nsingle_ints) .and.
     +                               (single_ints(k) .ne. nsing))
                              k = k + 1
                           end do

                           if (k .gt. nsingle_ints) then
                               nsingle_ints = nsingle_ints + 1
                               single_ints(nsingle_ints) = nsing
                               
                               numint = numint + 1
                           end if    

                           l = l + 1
                        end do

                        if (numint .gt. 0) then
                            nedgeinter = nedgeinter + 1

                            call  insert_edge_intersection(intersecs,
     +                            nedgeinter, i, j, numint, t11, t12, 
     +                            t21, t22)
                        end if
                    end if
                end if
            end if

            j = j + 1
          end do
      
          i = i + 1
      end do
      
      return
      end
      
*-----------------------------------------------------------------------
      subroutine classify_line(x1, y1, x2, y2, tline, m, a, b, c)
*-----------------------------------------------------------------------
* x1, y1, x2, y2: x and y coordinates of the endpoints of the line
*                 defining the cutting plane.
* tline         : Type of line according to its slope.
* m             : Slope of the line.
* a, b, c       : Coefficients of the implicit equation of the line.
* 
* This subroutine classifies the line defined by two endpoints according
* to its slope, and calculates the values of the slope, and the coefficients
* of the corresponding implicit equation.
*
* Author: Rodrigo A. Marin
* Date  : March 14 1993
*-----------------------------------------------------------------------

      include 'postconst.h'


      double precision x1, y1, x2, y2
      integer tline
      double precision m, a, b, c

      double precision dx, dy

      dx = x2 - x1
      dy = y2 - y1

      if (dabs(dx) .le. ZERO) then
          tline = TLINE_GT45
          m = 0.0
      else 
          if (dabs(dy) .le. ZERO) then
              tline = TLINE_LT45
              m = 0.0
          else
              m = dy/dx
              if ((m .le. 1.0) .and. (m .ge. -1.0)) then
                  tline = TLINE_LT45
              else
                  tline = TLINE_GT45
                  m = dx/dy
              end if
          end if
      end if

      a = dy
      b = -dx

      if (tline .eq. TLINE_GT45) then
          c = -(x1 - m * y1) * dy
      else
          c = (y1 - m * x1) * dx
      end if

      return
      end

*-----------------------------------------------------------------------
      subroutine sliceplot
*-----------------------------------------------------------------------
* Subroutine that calculates the slice profile generated by the intersection
* of a cutting plane defined by the user and the solution field of the
* variables specified by the user. 
*
* Author: Rodrigo A. Marin
* Date  : March 14 1993
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      double precision x1, y1, x2, y2
      real             x1r, y1r, x2r, y2r
      double precision x1d, y1d, x2d, y2d
      double precision dx, dy
      double precision a, b, c
      double precision m

      type (intersection) intersecs(MAXEL+1)
      
      integer err
      integer i
      integer j
      integer nval
      integer tline
      integer nedgeinter
      integer option

      character options(2)*(MAXOPTLENGTH), prompt*(MAXPROMPTLENGTH)

      logical quit1
      logical quit2
      logical quit3

      double precision values(4)

      integer nonaccept(MAXNUMVAR)
      integer numnonaccept

      data options /'y', 'n'/

      quit1 = .FALSE.

      call minmax(x,nnodes,xmin,xmax)
      call minmax(y,nnodes,ymin,ymax)

      xwmin = xmin
      ywmin = ymin
      xwmax = xmax
      ywmax = ymax

      x1 = xwmin
      y1 = ywmin
      x2 = xwmax
      y2 = ywmax

      numnonaccept = 1
      nonaccept(1) = 38

      do while (.not. quit1)
         call setplotconfig(mtitle, 'n', 'n', 'n', 'y', 'n', 'n', 
     +                      intlab, disp_border_plot)
         call asklab(ptitle,nodplot,nodnum,elplot,boundplt,
     +               contlabl, intlab, iunit,iout,iecho)

         quit2 = .FALSE.

         do while (.not. quit2)
            call select_variables(numvar, variables, iunit, iout)

            quit2 = check_variables(numvar, variables, numnonaccept,
     +                              nonaccept, iunit, iout)
         end do

         call askiter(time,newtime,lastzpos,newzpos,casvel,lastitnum,
     +                newitnum,iunit,iout,ibin,
     +                iecho,timetype)

         call find_iteration(timetype, time, newtime, lastzpos, newzpos, 
     +                       casvel, lastitnum, newitnum,
     +                       t,ut,vt,sigma,epsln,evp,seff,epefft,eeff,
     +                       epdot,uts,ratio,e0,fa,tle,elastic,eflow,
     +                       dmgcrt,emod,ctime,nn,nns,
     +                       itnum,zpos,ibin,iout)

         call assign_variables

         write(iout, *)
         write(iout,*)
     +   'Printout table will be done for the following data:'
         write(iout, *)
         write(iout, 900)'Iteration : ', itnum
         write(iout, 1000)'Time      : ', ctime
         write(iout, 1000)'Z-position: ', zpos

c-----------------------------------------------------------------------
c     Initialize terminal
         call hf_open_segment('?Picture/plot')

c----------------Plot nodes and nodes numbers if requested--------------
         if (nodplot .eq. 'y') call plotn(x,y,xwmin, ywmin,
     +                                    xwmax, ywmax, nn,nodnum)
c
c----------------Draw elements if requested-----------------------------
         if (elplot .eq. 'y')
     +       call eplot(x,y,nn,nel,node,intrash)
c
c----------------Draw boundary if requested-----------------------------
         if (boundplt .eq. 'y')
     +       call border(x,y,nn,nbe,nbnc,0)

         call show_cut_plane(x1, y1, x2, y2)

         call update_display()

         quit2 = .FALSE.

         do while (.not. quit2)
            write(iout, *)
            write(iout, 1150) x1, y1, x2, y2
            call readmenu(11, option, iunit, iout)
            
            if (option .eq. 1) then
                write(iout, *)
                write(iout, *)'Please use the graphics window'
                write(iout, *)
                call clear_messages()
                call hf_flush_contents('?Picture/plot/cutplane', 
     +                                 'lines')
                call read_line('?Picture/plot/cutplane', x1, y1, x2, y2)
                call message(1)
            else if (option .eq. 2) then
                values(1) = x1
                values(2) = y1
                values(3) = x2
                values(4) = y2

                quit3 = .FALSE.

                do while (.not. quit3)
                   write(iout, *)
                   write(iout, 1200) xmin, ymin, xmax, ymax
                   write(iout, *)
                   write(prompt, 1100) x1, y1, x2, y2

                   call readndouble(prompt, strlen(prompt), 4, .TRUE.,
     +                              nval, values, iunit, iout)

                   if ((nval .eq. 4) .or. (nval .eq. 0)) then
                       x1 = values(1)
                       y1 = values(2)
                       x2 = values(3)
                       y2 = values(4)

                       dx = x2 - x1
                       dy = y2 - y1 

                       if ((dabs(dx) .le. ZERO) .and. 
     +                     (dabs(dy) .le. ZERO)) then
                          call reperror(12, iout)
                       else
                           quit3 = .TRUE.
                       end if
                   else
                       call reperror(18, iout)
                   end if
                end do
            else  if (option .eq. 3) then
                      quit2 = .TRUE.
            end if

            call show_cut_plane(x1, y1, x2, y2)

            call update_display()

            call classify_line(x1, y1, x2, y2, tline, m, a, b, c)

            write(iout, *)
            write(iout, 1300) xwmin, ywmin, xwmax, ywmax
            write(iout, *)

            call readmenu(12, option, iunit, iout)

            if (option .eq. 1) then
                write(iout, *)
                write(iout, *)'Please use the graphics window'
                write(iout, *)
                call clear_messages()
                call window_definition(3)
                call message(1)
            else if (option .eq. 2) then
                call define_new_window(xmin, ymin, xmax, ymax,
     +                                 xwmin, ywmin, xwmax, ywmax,
     +                                 iunit, iout)
                call setup_window(xwmin,ywmin,xwmax,ywmax)
            end if

            call hf_close_segment()
            call update_display()

            call calc_intersections(node, nel, x, y, nnodes, tline,
     +                              x1, y1, x2, y2, a, b, c, intersecs, 
     +                              nedgeinter)

           call open_tables_file(iunit, iout)

           if (nedgeinter .le. 1) then
               call reperror(17, iout)
           else   
               write(TABLEFILEUNIT, 10) rtitle
               write(TABLEFILEUNIT, 5)

               write(TABLEFILEUNIT, 40) ctime, zpos, itnum
               write(TABLEFILEUNIT, 5)

               write(TABLEFILEUNIT, 60) x1, y1, x2, y2
               write(TABLEFILEUNIT, 5)

               write(TABLEFILEUNIT, 70)

               i = 1

               do while (i .le. numvar)
                  write(TABLEFILEUNIT, 100) tbltitle1(variables(i))

                  i = i + 1
               end do

               write(TABLEFILEUNIT, *)
               write(TABLEFILEUNIT, 50)

               i = 1

               do while (i .le. numvar)
                  write(TABLEFILEUNIT, 100) tbltitle2(variables(i))

                  i = i + 1
               end do
       
               write(TABLEFILEUNIT, *)
               write(TABLEFILEUNIT, 5)

               call filter_intersecs(x1, y1, x2, y2, intersecs,
     +                               nedgeinter)

               call sort_intersecs(intersecs, nedgeinter)

               call interp_variables(x1, y1, x2, y2, nedgeinter,
     +                               intersecs)
           end if

           close(TABLEFILEUNIT)

           write(iout, *)

           prompt = 'Change cutting plane and/or window? (y/n) [y]: '

           call readopt(2, options, prompt, strlen(prompt)+1, .FALSE.,
     +                  1, option, iunit, iout)

           quit2 = option .eq. 2
         end do

         prompt = 'Do you want another slice table? (y/n) [y]: '

         call readopt(2, options, prompt, strlen(prompt)+1, .FALSE.,
     +                1, option, iunit, iout)

         quit1 = option .eq. 2
      end do

      return
5     format('#')
10    format('# ', A80)
40    format('# Time= ', E14.5, ' Z-pos= ', E14.5,
     +       '  Iteration No.= ', i8)
50    format('#            ', $)
60    format('# Cutting Plane: (',f10.5,', ',f10.5,')-(',f10.5,', ',
     +       f10.5, ')')
70    format('# Distance   ', $)
100   format('  ', A14, $)
300   format(E14.5, $)
400   format('  ', F14.4, $)
500   format('  ', F14.9, $)
600   format('  ', E14.4, $)
900   format(A, I8)
1000  format(A, E14.5)
1100  format('Please define the cutting plane [',
     +       F10.5, ', ', F10.5, ', ', F10.5, ', ', F10.5, ']: ')
1150  format('The current cutting plane is (', F10.5, ', ', F10.5,
     +       ') - (', F10.5, ', ', F10.5, ')')
1200  format('The minimum and maximum coordinates are ',
     +       '(', F10.6, ', ', F10.6, ') - (', F10.6, ', ', F10.6, ')')
1300  format('The current window is (xmin= ', f10.5, ', ymin= ', f10.5,
     +       ') - (xmax= ', f10.5, ', ymax= ', f10.5, ')')

      end

*-----------------------------------------------------------------------
      subroutine calc_inter(xe1, ye1, xe2, ye2, x1, y1, x2, y2, 
     +                      tline, a, b, c, ninter, t11, t12, t21, t22)
*-----------------------------------------------------------------------
* xe1, ye1, xe2, ye2: Coordinates of the endpoints of the edge.
* x1, y1, x2, y2    : Coordinates of the endpoints of the line defining the
*                     cutting plane.
* tline             : Type of line defining the cutting plane.
* a, b, c           : Coefficients of the implicit equation of the line
*                     defining the cutting plane.
* m                 : Slope of the line defining the cutting plane.
* ninter            : Number of intersections found between the cutting plane 
*                     and the given edge.
* t11, t12          : Value(s) of the parameter on the intesection point(s) on the
*                     line defining the cutting plane.
* t21, t22          : Value(s) of the parameter on the intersection point(s) on the 
*                     line segment defined by the given edge.
*
* This subroutine calculates the intersection(s) between the cutting plane and the
* given edge of the domain. Zero, one or two intersections can be found and this
* value is dtored in the ninter parameter, and the value of the parameter corresponding
* to the intersection points on the edge and the cutting plane defining line are
* calculated.
*
* Author: Rodrigo A. Marin
* Date  : March 14 1993
*-----------------------------------------------------------------------

      include 'postconst.h'
 
      double precision xe1, ye1, xe2, ye2
      double precision x1, y1, x2, y2
      double precision a, b, c
      integer ninter
      integer tline
      double precision t11, t12, t21, t22
      double precision a11, a12, a21, a22

      double precision z1, z2
      double precision det

      z1 = a * xe1 + b * ye1 + c
      z2 = a * xe2 + b * ye2 + c

      if ((dabs(z1) .le. ZERO) .and. (dabs(z2) .le. ZERO)) then
          ninter = 2
          t11 = 0.0
          t12 = 1.0
          
          if (tline .eq. TLINE_LT45) then
              t21 = (xe1 - x1) / (x2 - x1)
              t22 = (xe2 - x1) / (x2 - x1)
          else
             t21 = (ye1 - y1) / (y2 - y1)
             t22 = (ye2 - y1) / (y2 - y1)
          end if
      else
c
c         Next statement should be a check for a negative number,
c         and not a small number (like ZERO)
c
          if (z1 * z2 .le. 0.0) then
              a11 = xe2 - xe1
              a12 = x2 - x1
              a21 = ye2 - ye1
              a22 = y2 - y1
              det = (a11 * a22) - (a12 * a21)
              
              t11 = a22 * (xe1 - x1) - a12 * (ye1 - y1)
              t11 = -t11 / det

              t21 = -a21 * (xe1 - x1) + a11 * (ye1 - y1)
              t21 = t21 / det
              ninter = 1
          else
              ninter = 0
          end if
      end if

      return
      end

*-----------------------------------------------------------------------
      subroutine filter_intersecs(x1, y1, x2, y2, intersecs, nedgeinter)
*-----------------------------------------------------------------------

      use posttypes

      include 'postconst.h'

      double precision x1, y1, x2, y2

      type (intersection) intersecs(MAXEL+1)

      integer nedgeinter

      integer i
      integer j
      integer k

      logical elemfound
      logical inter_zero_found
      logical inter_one_found
      
      integer elemnum
      double precision lambda(3)

      i = 0
      j = 1

      do while (j .le. nedgeinter)
         if (intersecs(j)%numint .eq. 2) then
             if ((intersecs(j)%t2(1) .ge. 0.0) .and.
     +           (intersecs(j)%t2(1) .le. 1.0)) then
                 if ((intersecs(j)%t2(2) .lt. 0.0) .or.
     +               (intersecs(j)%t2(2) .gt. 1.0)) then
                     intersecs(j)%numint = 1
                 end if

                 i = i + 1

                 if (i .ne. j) then
                     call move_intersecs(intersecs, i, j)
                 end if

             else
                 if ((intersecs(j)%t2(2) .ge. 0.0) .and.
     +               (intersecs(j)%t2(2) .le. 1.0)) then
                     intersecs(j)%t2(1) = intersecs(j)%t2(2)
                     intersecs(j)%t1(1) = intersecs(j)%t1(2)
                     intersecs(j)%numint = 1

                     i = i + 1

                     if (i .ne. j) then
                         call move_intersecs(intersecs, i, j)
                     end if
                 
                 end if
             end if
         else
             if ((intersecs(j)%t2(1) .ge. 0.0) .and.
     +           (intersecs(j)%t2(1) .le. 1.0)) then

                 i = i + 1

                 if (i .ne. j) then
                     call move_intersecs(intersecs, i, j)
                 end if

             end if
         end if

         j = j + 1
      end do

      nedgeinter = i

      k = 1
      inter_zero_found = .FALSE.
      inter_one_found = .FALSE.

      do while ((k .le. nedgeinter) .and. ( .not. inter_zero_found) 
     +          .and. (.not. inter_one_found))
         l = 1

         do while (l .le. intersecs(k)%numint)
            if (dabs(intersecs(k)%t2(l)) .le. ZERO) then
                inter_zero_found = .TRUE.
            else if (dabs(1.0 - intersecs(k)%t2(l)) .le. ZERO) then
                     inter_one_found = .TRUE.
            end if

            l = l + 1
         end do

         k = k + 1     
      end do 

      if (.not. inter_zero_found) then
          call find_element_inc_point(x1, y1, elemfound, elemnum, 
     +                                lambda)

          if (elemfound) then
              nedgeinter = nedgeinter + 1
              call insert_elem_intersection(intersecs, nedgeinter,
     +                                      elemnum, dble(0.0), lambda)
          end if
      end if

      if (.not. inter_one_found) then
          call find_element_inc_point(x2, y2, elemfound, elemnum, 
     +                                lambda)

          if (elemfound) then
              nedgeinter = nedgeinter + 1
              call insert_elem_intersection(intersecs, nedgeinter,
     +                                      elemnum, dble(1.0), lambda)
          end if
      end if

      return
      end

*-----------------------------------------------------------------------
      subroutine find_element_inc_point(px, py, elemfound, elemnum, 
     +                                  lambda)
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      double precision px, py
      integer elemnum
      logical elemfound
      double precision lambda(3)

      integer i
      integer j
      integer k

      double precision det
      double precision Q(3, 3)
      double precision pt(3)
    
      i = 1
      elemnum = 0
      elemfound = .FALSE.

      pt(1) = px
      pt(2) = py
      pt(3) = 1.0

      do while ((i .le. nel) .and. (.not. elemfound))
         det = -(y(node(1, i)) * x(node(2, i))) + (x(node(1, i)) * 
     +         y(node(2, i))) + (y(node(1, i)) * x(node(3, i))) -
     +         (y(node(2, i)) * x(node(3, i))) - (x(node(1, i)) *
     +         y(node(3, i))) + (x(node(2, i)) * y(node(3, i)))

         Q(1, 1) = y(node(2, i)) - y(node(3, i))
         Q(1, 2) = -x(node(2, i)) + x(node(3, i))
         Q(1, 3) = -(y(node(2, i)) * x(node(3, i))) +
     +             (x(node(2, i)) * y(node(3, i)))

         Q(2, 1) = -y(node(1, i)) + y(node(3, i))
         Q(2, 2) = x(node(1, i)) - x(node(3, i))
         Q(2, 3) = (y(node(1, i)) * x(node(3, i))) -
     +             (x(node(1, i)) * y(node(3, i)))

         Q(3, 1) = y(node(1, i)) - y(node(2, i))
         Q(3, 2) = -x(node(1, i)) + x(node(2, i))
         Q(3, 3) = -(y(node(1, i)) * x(node(2, i))) +
     +             (x(node(1, i)) * y(node(2, i)))

         call matvec(Q, 3, 3, pt, lambda)

         lambda(1) = lambda(1) / det
         lambda(2) = lambda(2) / det
         lambda(3) = lambda(3) / det

         elemfound = (lambda(1) .ge. 0.0) .and. 
     +               (lambda(2) .ge. 0.0) .and. (lambda(3) .ge. 0.0)

         i = i + 1
      end do

      if (elemfound) then
          elemnum = i - 1
      end if

      return
      end

*-----------------------------------------------------------------------
      subroutine insert_edge_intersection(intersecs, nedgeinter, elnum,
     +                                    edgenum, numint, t11, t12, 
     +                                    t21, t22)
*-----------------------------------------------------------------------

      use posttypes

      include 'postconst.h'

      type (intersection) intersecs(MAXEL+1)
      integer nedgeinter, edgenum, elnum, numint
      double precision t11, t12, t21, t22
   
      intersecs(nedgeinter)%elnum = elnum 
      intersecs(nedgeinter)%edgenum = edgenum
      intersecs(nedgeinter)%numint = numint
      intersecs(nedgeinter)%intertype = TINTER_PLANE_EDGE

      if (numint .eq. 2) then
          if (t21 .lt. t22) then
              intersecs(nedgeinter)%t1(1) = t11
              intersecs(nedgeinter)%t2(1) = t21
              intersecs(nedgeinter)%t1(2) = t12
              intersecs(nedgeinter)%t2(2) = t22
          else
              intersecs(nedgeinter)%t1(1) = t12
              intersecs(nedgeinter)%t2(1) = t22
              intersecs(nedgeinter)%t1(2) = t11
              intersecs(nedgeinter)%t2(2) = t21
          end if
      else
          intersecs(nedgeinter)%t1(1) = t11
          intersecs(nedgeinter)%t2(1) = t21
      end if

      return
      end
       
*-----------------------------------------------------------------------
      subroutine insert_elem_intersection(intersecs, neleminter, elnum,
     +                                    t2, lambda)
*-----------------------------------------------------------------------

      use posttypes

      include 'postconst.h'

      type (intersection) intersecs(MAXEL+1)

      integer neleminter
      integer elnum
      double precision lambda(3)
      double precision t2
      integer i


      intersecs(neleminter)%elnum = elnum
      intersecs(neleminter)%numint = 1
      intersecs(neleminter)%intertype = TINTER_PLANE_ELEM
      
      i = 1

      do while (i .le. 3)
         intersecs(neleminter)%lambda(i) = lambda(i)
         i = i + 1
      end do

      intersecs(neleminter)%t2(1) = t2

      return
      end

*-----------------------------------------------------------------------
      subroutine interp_variables(x1, y1, x2, y2, nedgeinter, intersecs)
*-----------------------------------------------------------------------
* x1, y1, x2, y2: Coordinates of the endpoints of the line defining
*                 the cutting plane.
* nedgeinter    : Number of edge and element intersections.
* intersecs     : Array with the cutting plane - domain intersections.
* variables     : Array with the indices of the variables to interpolate.
*
* This function interpolates the values of the given variables, on the
* intersection points found between the cutting plane and the corresponding
* solution fields. The results are written to the current table file in the
* appropriate format.
*
* Author: Rodrigo A. Marin
* Date  : March 14 1993
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      double precision x1, y1, x2, y2
      integer nedgeinter
      type (intersection) intersecs(nedgeinter)

      integer i
      integer j
      integer k
      integer n1, n2, n3
      double precision distance
      double precision xcur, ycur
      double precision xlast, ylast
      double precision v1, v2, v3, vint
      double precision alpha
       
      distance = 0.0

      i = 1

      do while (i .le. nedgeinter)
         if (intersecs(i)%intertype .eq. TINTER_PLANE_EDGE) then
             n1 = node(intersecs(i)%edgenum, intersecs(i)%elnum)
             n2 = node(mod(intersecs(i)%edgenum, 3) + 1, 
     +                 intersecs(i)%elnum)

             j = 1

             do while (j .le. intersecs(i)%numint)
                alpha = intersecs(i)%t2(j)
                beta  = intersecs(i)%t1(j)

                if ((i .eq. 1) .and. (j .eq. 1)) then
                    xant = alpha * x2 + (1.0 - alpha) * x1
                    yant = alpha * y2 + (1.0 - alpha) * y1
                else
                    xcur = alpha * x2 + (1.0 - alpha) * x1
                    ycur = alpha * y2 + (1.0 - alpha) * y1

                    distance = distance + dsqrt((xcur - xant)**2.0 +
     +                                          (ycur - yant)**2.0)

                    xant = xcur
                    yant = ycur
                end if
            
                write(TABLEFILEUNIT, 300) distance

                k = 1

                do while (k .le. numvar)
                   v1 = varvals(variables(k), n1)
                   v2 = varvals(variables(k), n2)

                   vint = beta * v2 + (1.0 - beta) * v1

                   if (((variables(k) .ge. 4) .and.
     +                 (variables(k) .le. 7)) .or.
     +                 ((variables(k) .ge. 16) .and.
     +                  (variables(k) .le. 19)) .or.
     +                 (variables(k) .eq. 28) .or.
     +                 (variables(k) .eq. 35) .or. 
     +                 (variables(k) .eq. 37) .or.
     +                 (variables(k) .eq. 51) .or.
     +                 (variables(k) .eq. 1)) then
                       write(TABLEFILEUNIT, 400) vint
                   else if (((variables(k) .ge. 2) .and.
     +                      (variables(k) .le. 3)) .or.
     +                      ((variables(k) .ge. 8) .and.
     +                      (variables(k) .le. 15)) .or.
     +                      ((variables(k) .ge. 20) .and.
     +                       (variables(k) .le. 27)) .or.
     +                      ((variables(k) .ge. 29) .and.
     +                       (variables(k) .le. 30)) .or.
     +                     (variables(k) .eq. 32) .or.
     +                     (variables(k) .eq. 33) .or.
     +                      (variables(k) .eq. 34) .or.
     +                    (variables(k) .eq. 36) .or.
     +                    (variables(k) .eq. 39) .or.
     +                    (variables(k) .eq. 40) .or.
     +                    (variables(k) .eq. 41) .or.
     +                    (variables(k) .eq. 42) .or.
     +                    (variables(k) .eq. 43) .or.
     +                    (variables(k) .eq. 44) .or.
     +                    (variables(k) .eq. 45) .or.
     +                    (variables(k) .eq. 46) .or.
     +                    (variables(k) .eq. 47) .or.
     +                    (variables(k) .eq. 48) .or.
     +                    (variables(k) .eq. 49) .or.
     +                      (variables(k) .eq. 50)) then
                            write(TABLEFILEUNIT, 500) vint
                   else
                       write(TABLEFILEUNIT, 600) vint
                   end if

                   k = k + 1
                end do

                write(TABLEFILEUNIT, *)

                j = j + 1
             end do
         else
             n1 = node(1, intersecs(i)%elnum)
             n2 = node(2, intersecs(i)%elnum)
             n3 = node(3, intersecs(i)%elnum)
             alpha = intersecs(i)%t2(1)

             if (i .eq. 1) then
                 xant = alpha * x2 + (1.0 - alpha) * x1
                 yant = alpha * y2 + (1.0 - alpha) * y1
             else
                 xcur = alpha * x2 + (1.0 - alpha) * x1
                 ycur = alpha * y2 + (1.0 - alpha) * y1

                 distance = distance + dsqrt((xcur - xant)**2.0 +
     +                                       (ycur - yant)**2.0)

                 xant = xcur
                 yant = ycur
             end if

             write(TABLEFILEUNIT, 300) distance

             k = 1

             do while (k .le. numvar)
                v1 = varvals(variables(k), n1)
                v2 = varvals(variables(k), n2)
                v3 = varvals(variables(k), n3)

                vint = intersecs(i)%lambda(1) * v1 + 
     +                 intersecs(i)%lambda(2) * v2 +
     +                 intersecs(i)%lambda(3) * v3

                if (((variables(k) .ge. 4) .and.
     +              (variables(k) .le. 7)) .or.
     +              ((variables(k) .ge. 16) .and.
     +              (variables(k) .le. 19)) .or.
     +              (variables(k) .eq. 28) .or.
     +              (variables(k) .eq. 37) .or.
     +              (variables(k) .eq. 1) .or.
     +              (variables(k) .eq. 33).or. 
     +              (variables(k) .eq. 35) .or.
     +              (variables(k) .eq. 51)) then
                    write(TABLEFILEUNIT, 400) vint
                else if (((variables(k) .ge. 2) .and.
     +                   (variables(k) .le. 3)) .or.
     +                   ((variables(k) .ge. 8) .and.
     +                   (variables(k) .le. 15)) .or.
     +                   ((variables(k) .ge. 20) .and.
     +                   (variables(k) .le. 27)) .or.
     +                   ((variables(k) .ge. 29) .and.
     +                   (variables(k) .le. 30)) .or.
     +                   (variables(k) .eq. 32) .or.
     +                   (variables(k) .eq. 34) .or.
     +                   (variables(k) .eq. 36) .or.
     +                   (variables(k) .eq. 39) .or.
     +                   (variables(k) .eq. 40) .or.
     +                   (variables(k) .eq. 41) .or.
     +                   (variables(k) .eq. 42) .or.
     +                   (variables(k) .eq. 43) .or.
     +                   (variables(k) .eq. 44) .or.
     +                   (variables(k) .eq. 45) .or.
     +                   (variables(k) .eq. 46) .or.
     +                   (variables(k) .eq. 47) .or.
     +                   (variables(k) .eq. 48) .or.
     +                   (variables(k) .eq. 49) .or.
     +                   (variables(k) .eq. 50)) then
                         write(TABLEFILEUNIT, 500) vint
                else
                    write(TABLEFILEUNIT, 600) vint
                end if

                k = k + 1
             end do

             write(TABLEFILEUNIT, *)
         end if

         i = i + 1
      end do

      return
300   format(E14.4, $)
400   format('  ', f14.4, $)
500   format('  ', f15.9, $)
600   format('  ', e14.4, $)
      end

*-----------------------------------------------------------------------
      subroutine move_intersecs(intersecs, i, j)
*-----------------------------------------------------------------------

      use posttypes

      include 'postconst.h'

      type (intersection) intersecs(MAXEL + 1)

      integer k

      intersecs(i)%elnum = intersecs(j)%elnum
      intersecs(i)%intertype = intersecs(j)%intertype
      intersecs(i)%numint = intersecs(j)%numint

      if (intersecs(j)%intertype .eq. TINTER_PLANE_EDGE) then
          intersecs(i)%edgenum = intersecs(j)%edgenum
       
          k = 1

          do while (k .le. intersecs(j)%numint)
             intersecs(i)%t1(k) = intersecs(j)%t1(k)
             k = k + 1
          end do
      else
          k = 1

          do while (k .le. 3)
             intersecs(i)%lambda(k) = intersecs(j)%lambda(k)
             k = k + 1
          end do
      end if

      k = 1

      do while (k .le. intersecs(j)%numint)
         intersecs(i)%t2(k) = intersecs(j)%t2(k)

         k = k + 1
      end do

      return
      end
      
*-----------------------------------------------------------------------
      subroutine sort_intersecs(intersecs, nedgeinter)
*-----------------------------------------------------------------------
* intersecs: Array with the intersecstions between the cutting plane and 
*            the domain edges.
* nedgeinter: Number of intersections.
*
* This subroutine sorts the found intersections in increasing order on the
* value of the parameter at the intersection points on the line dfining 
* the cutting plane.
*
* Author: Rodrigo A. Marin
* Date  : March 15 1993
*-----------------------------------------------------------------------

      use posttypes

      include 'postconst.h'

      integer nedgeinter
      type (intersection) intersecs(nedgeinter)

      integer i
      integer j
      integer k
      integer l
      logical swap

      double precision tmplambda(3)

      i = 1

      do while (i .lt. nedgeinter)
         j = i + 1

         if (intersecs(i)%intertype .eq. TINTER_PLANE_EDGE) then
             if (intersecs(i)%numint .eq. 2) then
                 if (intersecs(i)%t2(1) .gt. intersecs(i)%t2(2)) then
                     call dswap(intersecs(i)%t2(1), intersecs(i)%t2(2))
                     call dswap(intersecs(i)%t1(1), intersecs(i)%t1(2))
                 end if
             end if
         end if

         do while (j .le. nedgeinter)
            swap = .FALSE.

            if (intersecs(j)%intertype .eq. TINTER_PLANE_EDGE) then
                if (intersecs(j)%numint .eq. 2) then
                    if (intersecs(j)%t2(1) .gt. intersecs(j)%t2(2)) then
                        call dswap(intersecs(j)%t2(1), 
     +                             intersecs(j)%t2(2))
                        call dswap(intersecs(j)%t1(1), 
     +                             intersecs(j)%t1(2))
                    end if
                end if
            end if

            if (intersecs(i)%numint .eq. 1) then
                if (intersecs(j)%numint .eq. 1) then
                    if (intersecs(j)%t2(1) .lt. intersecs(i)%t2(1))
     +              then
                        swap = .TRUE.
                    end if
                else
                    if (intersecs(j)%t2(2) .lt. intersecs(i)%t2(1))
     +              then
                        swap = .TRUE.
                    end if
                end if
            else
                if (intersecs(j)%numint .eq. 1) then
                    if (intersecs(j)%t2(1) .lt. intersecs(i)%t2(1)) then
                        swap = .TRUE.
                    end if
                else
                    if (intersecs(j)%t2(2) .lt. intersecs(i)%t2(1)) then
                        swap = .TRUE.
                    end if
                end if
            end if

            if (swap) then
                call iswap(intersecs(i)%elnum, intersecs(j)%elnum)
                call iswap(intersecs(i)%numint, intersecs(j)%numint)
                call dswap(intersecs(i)%t2(1), intersecs(j)%t2(1))
                call dswap(intersecs(i)%t2(2), intersecs(j)%t2(2))

                if (intersecs(i)%intertype .eq. intersecs(j)%intertype) 
     +          then
                    if (intersecs(i)%intertype .eq. TINTER_PLANE_EDGE)
     +              then
                        call iswap(intersecs(i)%edgenum, 
     +                             intersecs(j)%edgenum)
                        call dswap(intersecs(i)%t1(1), 
     +                             intersecs(j)%t1(1))
                        call dswap(intersecs(i)%t1(2), 
     +                             intersecs(j)%t1(2))
                    else
                        call dswap(intersecs(i)%lambda(1), 
     +                             intersecs(j)%lambda(1))
                        call dswap(intersecs(i)%lambda(2), 
     +                             intersecs(j)%lambda(2))
                        call dswap(intersecs(i)%lambda(3), 
     +                             intersecs(j)%lambda(3))
                    end if
                else
                    if (intersecs(i)%intertype .eq. TINTER_PLANE_EDGE) 
     +              then
                        k = j
                        l = i
                    else
                        k = i
                        l = j
                    end if

                    tmplambda(1) = intersecs(k)%lambda(1)
                    tmplambda(2) = intersecs(k)%lambda(2)
                    tmplambda(3) = intersecs(k)%lambda(3)

                    intersecs(k)%edgenum = intersecs(l)%edgenum
                    intersecs(k)%t1(1) = intersecs(l)%t1(1)
                    intersecs(k)%t1(2) = intersecs(l)%t1(2)

                    intersecs(l)%lambda(1) = tmplambda(1)
                    intersecs(l)%lambda(2) = tmplambda(2)
                    intersecs(l)%lambda(3) = tmplambda(3)
                        
                end if
                call iswap(intersecs(i)%intertype, 
     +                     intersecs(j)%intertype)
            end if

            j = j + 1
         end do 

         i = i + 1
      end do

      return
      end
