*-----------------------------------------------------------------------
* util.f
* Utility functions
*
* strlen
*-----------------------------------------------------------------------

*-----------------------------------------------------------------------
      subroutine add_line_annotation(posx1, posy1, posx2, posy2)
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      real posx1, posy1
      real posx2, posy2

      integer i

      annotation_tbl(nextannot)%available = .FALSE.
      annotation_tbl(nextannot)%ent_type = ENTTYPE_LINE
      annotation_tbl(nextannot)%posx1 = posx1
      annotation_tbl(nextannot)%posy1 = posy1
      annotation_tbl(nextannot)%posx2 = posx2
      annotation_tbl(nextannot)%posy2 = posy2

      if (nextannot .eq. MAXNUMANNOT) then
          nextannot = MAXNUMANNOT + 1
      else
          nextannot = nextannot + 1
          numannot = numannot + 1

          do while ((nextannot .le. MAXNUMANNOT) .and.
     +              (.not. annotation_tbl(nextannot)%available))
             nextannot = nextannot + 1
          end do
      end if

      return
      end

*-----------------------------------------------------------------------
      subroutine add_text_annotation(posx, posy, str)
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      real posx, posy
      character str*(MAXSTRINGLENGTH)

      integer i

      annotation_tbl(nextannot)%available = .FALSE.
      annotation_tbl(nextannot)%ent_type = ENTTYPE_TEXT
      annotation_tbl(nextannot)%posx = posx
      annotation_tbl(nextannot)%posy = posy
      annotation_tbl(nextannot)%str = str

      if (nextannot .eq. MAXNUMANNOT) then
          nextannot = MAXNUMANNOT + 1
      else
          nextannot = nextannot + 1
          numannot = numannot + 1

          do while ((nextannot .le. MAXNUMANNOT) .and.
     +              (.not. annotation_tbl(nextannot)%available))
             nextannot = nextannot + 1
          end do
      end if

      return
      end

*-----------------------------------------------------------------------
      integer function atoi(str, err)
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      character*(*) str
      logical err
      
      character str1*(MAXSTRINGLENGTH)

      integer i
      integer l
      integer sign

      str1 = str

      call ltrim(str1)

      l = strlen(str1)

      if (l .eq. 0) then
          atoi = -1
          err = .TRUE.
      else
          if (str1(1:1) .eq. '-') then
              sign = -1
              i = 2
          else
              sign = 1
              i = 1
          end if

          err = .FALSE.
          atoi = 0
 
          do while ((i .le. l) .and. (.not. err))
             err = (((ichar(str1(i:i)) .lt. (ichar('0'))) .or.
     +              (ichar(str1(i:i)) .gt. (ichar('9')))) .and.
     +              (str1(i:i) .ne. ' '))

             if (.not. err) then
                 atoi = atoi * 10 + (ichar(str1(i:i)) - ichar('0'))
             end if

             i = i + 1
          end do
      end if

      if (.not. err) then
          atoi = sign*atoi
      end if

      return
      end
 
*-----------------------------------------------------------------------
      subroutine clear_errmsg()
*-----------------------------------------------------------------------


      include 'postconst.h'
      include 'pubfunc.h'

      call hf_flush_contents('?Picture/messages', 'text')

      return
      end

*-----------------------------------------------------------------------
      subroutine clear_messages()
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      call hf_flush_contents('?Picture/messages', 'text')

      return
      end

*-----------------------------------------------------------------------
      subroutine delete_annotation(indx)
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      integer indx

      annotation_tbl(indx)%available = .TRUE.

      if (indx .lt. nextannot) then
          nextannot = indx
      end if

      numannot = numannot - 1
      return
      end

*-----------------------------------------------------------------------
      subroutine dswap(d1, d2)
*-----------------------------------------------------------------------
* Swaps two double precision values.
*
* Author: Rodrigo A. Marin
* Date  : Februery 2 1993
*-----------------------------------------------------------------------

      double precision d1, d2

      double precision temp

      temp = d1
      d1 = d2
      d2 = temp

      return
      end

*-----------------------------------------------------------------------
      subroutine iswap(i1, i2)
*-----------------------------------------------------------------------
* Swaps two integer values.
*
* Author: Rodrigo A. Marin
* Date  : Februery 2 1993
*-----------------------------------------------------------------------

      integer i1, i2

      integer temp

      temp = i1
      i1 = i2
      i2 = temp

      return
      end

*-----------------------------------------------------------------------
      recursive subroutine itoa(n, str)
*-----------------------------------------------------------------------
* Converts an integer into its character representation
*
* Author: Rodrigo A. Marin
* Date  : October 30 1992
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      integer n
      character*(*) str
      character*(MAXSTRINGLENGTH) str1
      character*(MAXSTRINGLENGTH) str2

      integer m 
      integer i,iones,itens 

      if (n .lt. 0) then
          call itoa(-n, str1)
          str = '-' // str1
      else if (n .le. 9) then
               str = char(ichar('0') + n)
      else
          iones = MOD(n,10)
          itens = (n-iones)/10 
          call itoa(itens, str1)
          call itoa(iones, str2)
          str = str1(1:strlen(str1)) // str2(1:strlen(str2)) 
      end if    

      return
      end

*-----------------------------------------------------------------------
      subroutine ltrim(str1)
*-----------------------------------------------------------------------
* Trims left blank spaces from str1.
*
* Author: Rodrigo A. Marin
* Date  : December 5 1992
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      character*(*) str1

      character c*1

      integer i
      integer j
      integer l

      l = strlen(str1)

      i = 1

      do while ((i .le. l) .and. (str1(i:i) .eq. ' '))
         i = i + 1
      end do

      if (i .le. l) then
          j = 1

          do while (i .le. l)
             c = str1(i:i)
             str1(i:i) = ' '
             str1(j:j) = c
             i = i + 1
             j = j + 1
          end do
      end if

      return
      end
      
*-----------------------------------------------------------------------
      subroutine matvec(Q, m, n, v, w)
*-----------------------------------------------------------------------

      integer m, n
      double precision Q(m, n)
      double precision v(n)
      double precision w(n)

      integer i, j

      i = 1

      do while (i .le. m)
         w(i) = 0.0

         j = 1
         
         do while (j .le. n)
            w(i) = w(i) + Q(i, j) * v(j)
            j = j + 1
         end do

         i = i + 1
      end do

      return
      end

*-----------------------------------------------------------------------
      subroutine message(indx)
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      integer indx

      character msg*(MAXSTRINGLENGTH)

      if (indx .eq. 1) then
          msg = 'Please use the text window'
      else if (indx .eq. 2) then
               msg = 'Please select text position'
      else if (indx .eq. 3) then
               msg = 'Click on first endpoint of line. ' //
     +               'Keep mouse button pressed'
      else if (indx .eq. 4) then
               msg = 'Select second endpoint releasing mouse button'
      else if (indx .eq. 5) then
               msg = 'Click on annotation to be deleted'
      else if (indx .eq. 6) then
               msg = 'Click on annotation to be moved'
      else if (indx .eq. 7) then
               msg = 'Drag annotation keeping mouse button pressed'
      else if (indx .eq. 8) then
               msg = 'Click on first window corner. ' //
     +               'Keep mouse button pressed'
      else if (indx .eq. 9) then
               msg = 'Select opposite corner releasing mouse button'
      else if (indx .eq. 10) then
               msg = 'Click on the new scale position'
      end if

      call clear_messages()
      call hf_open_segment('?Picture/messages')

      call hf_insert_text(-1.0, -1.0, 0.0, msg)
      call hf_close_segment()
      call update_display()

      return
      end

*-----------------------------------------------------------------------
      logical function positive_orientation(x1, y1, x2, y2, x3, y3)
*-----------------------------------------------------------------------
* Returns .TRUE. if the triangle defined by the three given points is
* positively oriented (counterclockwise).
*
* Author: Rodrigo A. Marin
* Date  : February 2 1993
*-----------------------------------------------------------------------

      double precision x1, y1, x2, y2, x3, y3

      double precision vx1, vx2, vy1, vy2

      vx1 = x2 - x1
      vx2 = x3 - x2
      
      vy1 = y2 - y1
      vy2 = y3 - y2
      
      positive_orientation = (vx1 * vy2 - vx2 * vy1) .gt. 0.0

      return
      end
 
*-----------------------------------------------------------------------
      subroutine reperror(n, uout)
*-----------------------------------------------------------------------
* Displays the error message with the given code on the specified unit
*
* Author: Rodrigo A. Marin
* Date  : October 30 1992
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      integer n, uout

      character*(MAXSTRINGLENGTH) msg 

      if (n .eq. 1) then
          msg = 'Error opening Mesh File'
      else if (n .eq. 2) then
               msg = 'Error opening Binary Solution File'
      else if (n .eq. 3) then
               msg = 'Error reading Binary Solution File'
      else if (n .eq. 4) then
               msg = 'Contour value out of range'
      else if (n .eq. 5) then
               msg = 'Page Length must be positive'
      else if (n .eq. 6) then
               msg = 'Error opening Table Output File'
      else if (n .eq. 7) then
               msg = 'Invalid number of variables'
      else if (n .eq. 8) then
               msg = 'Invalid variable number'
      else if (n .eq. 9) then
               msg = 'Invalid interval'
      else if (n .eq. 10) then
               msg = 'Invalid number of nodes'
      else if (n .eq. 11) then
               msg = 'Invalid node number'
      else if (n. eq. 12) then
               msg = 'Invalid line definition'
      else if (n .eq. 13) then
               msg = 'Invalid increment'
      else if (n .eq. 14) then
               msg = 'Invalid skip value between crosses'
      else if (n .eq. 15) then
               msg = 'Invalid step between node numbers'
      else if (n .eq. 16) then
               msg = 'Invalid window definition'
      else if (n .eq. 17) then
               msg = 'Cutting plane does not intersect the domain'
      else if (n .eq. 18) then
               msg = 'Invalid cutting plane definition'
      else if (n .eq. 19) then
               msg = 'Invalid number of contours'
      else if (n .eq. 20) then
               msg = 'Invalid shell thickness calculation trajectory'
      else if (n .eq. 21) then
               msg = 'Invalid variable for this type of table'
      else if (n .eq. 22) then
               msg = 'Invalid time specification'
      else if (n .eq. 23) then
               msg = 'Error opening contour data file'
      else if (n .eq. 24) then
               msg = 'Invalid number of decimal places'
      end if

      write(uout, *)
      write(uout, *) msg

      return
      end
          
*-----------------------------------------------------------------------
      subroutine rep_error(indx)
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      integer indx

      character errmsg*(MAXSTRINGLENGTH)

      if (indx .eq. 4) then
          errmsg = 'Contour value out of range'
      else if (indx .eq. 8) then
          errmsg = 'Invalid variable(s) specified for this type of plot' 
      else if (indx .eq. 13) then
          errmsg = 'Invalid increment between labels'
      else if (indx .eq. 14) then
               errmsg = 'Invalid skip value between crosses'
      else if (indx .eq. 16) then
               errmsg = 'Invalid window definition'
      else if (indx .eq. 19) then
               errmsg = 'Invalid number of contours'
      else if (indx .eq. 20) then
               errmsg = 'Maximum number of annotations exceded'
      end if

      call clear_errmsg()

      call hf_open_segment('?Picture/messages')
      
      call hf_insert_text(-1.0, -1.0, 0.0, errmsg)
  
      call hf_close_segment()

      return
      end

*-----------------------------------------------------------------------
      subroutine reset_annotations
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      integer i

      i = 1

      do while (i .le. MAXNUMANNOT)
         annotation_tbl(i)%available = .TRUE.
         i = i + 1
      end do

      nextannot = 1
      numannot = 0

      return
      end

*-----------------------------------------------------------------------
      double precision function sqrdist(x1, y1, x2, y2)
*-----------------------------------------------------------------------
* Returns the square of the distance between (x1, y1) and (x2, y2).
*
* Author: Rodrigo A. Marin
* Date  : July 28 1993
*-----------------------------------------------------------------------

      double precision x1, y1, x2, y2

      sqrdist = ((x1 - x2) ** 2.0) + ((y1 - y2) ** 2.0)

      return
      end

*-----------------------------------------------------------------------
      integer function strlen(string)
*-----------------------------------------------------------------------
* Returns the length of the given character string; an integer number
* Author: Rodrigo Marin
* Date:   October 24 1992
*-----------------------------------------------------------------------
c
      character string*(*)
c
      strlen = len(string)
c
      do while ((string(strlen:strlen) .eq. ' ') .and. (strlen .gt. 0))
         strlen = strlen - 1
      end do
c
      return
      end

*-----------------------------------------------------------------------
      subroutine strlow(string1, string2)
*-----------------------------------------------------------------------
* Returns string2 such that each character is the corresponding
* lowercase character of string1.
* Author: Rodrigo A. Marin
* Date  : December 5 1992
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      character string1*(*)
      character string2*(*)

      integer i
      integer l

      l = strlen(string1)

      i = 1

      do while (i .le. l)
         if ((ichar(string1(i:i)) .ge. ichar('A')) .and.
     +       (ichar(string1(i:i)) .le. ichar('Z'))) then
             string2(i:i) = char(ichar(string1(i:i)) - ichar('A') + 
     +                           ichar('a'))
         else
             string2(i:i) = string1(i:i)
         end if

         i = i + 1
      end do
      
      return
      end

*-----------------------------------------------------------------------
      subroutine sort_double_array(numval, values)
*-----------------------------------------------------------------------
* numval : Number of elements
* values : Array of values to be sorted.
* Sorts an array of double precision values in ascending order.
*-----------------------------------------------------------------------

      integer numval
      double  precision values(numval)

      integer i
      integer j

      double precision temp

      i = 1

      do while (i .lt. numval)
         j = i + 1

         do while (j .le. numval)
            if (values(j) .lt. values(i)) then
                temp = values(i)
                values(i) = values(j)
                values(j) = temp
            end if

            j = j + 1
         end do

         i = i + 1
      end do

      return
      end

*-----------------------------------------------------------------------
      subroutine strup(string1, string2)
*-----------------------------------------------------------------------
* Returns string2 such that each character is the corresponding
* uppercase character of string1.
* Author: Rodrigo A. Marin
* Date  : December 5 1992
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      character string1*(*)
      character string2*(*)

      integer i
      integer l

      l = len(string1)

      i = 1

      do while (i .le. l)
         if ((ichar(string1(i:i)) .ge. ichar('a')) .and.
     +       (ichar(string1(i:i)) .le. ichar('z'))) then
             string2(i:i) = char(ichar(string1(i:i)) + ichar('A') - 
     +                           ichar('a'))
         else
             string2(i:i) = string1(i:i)
         end if

         i = i + 1
      end do
      
      return
      end

*-----------------------------------------------------------------------
      character*(*) function var_fmt(indx)
*-----------------------------------------------------------------------

      integer indx

      if (((indx .ge. 4) .and.
     +     (indx .le. 7)) .or.
     +     ((indx .ge. 16) .and.
     +      (indx .le. 19)) .or.
     +      (indx .eq. 28) .or.
     +      (indx .eq. 32) .or.
     +      (indx .eq. 37) .or.
     +      (indx .eq. 1) .or.
     +      (indx .eq. 33) .or.
     +      (indx .eq. 35) .or.
     +      (indx .eq. 38)) then
            var_fmt = '(f14.4)'
      else if (((indx .ge. 2) .and.
     +          (indx .le. 3)) .or.
     +         ((indx .ge. 8) .and.
     +          (indx .le. 15)) .or.
     +         ((indx .ge. 20) .and.
     +          (indx .le. 27)) .or.
     +         ((indx .ge. 29) .and.
     +          (indx .le. 30)) .or.
     +          (indx .eq. 34) .or.
     +          (indx .eq. 36)) then
                var_fmt = '(f14.9)' 
      else
          var_fmt = '(e14.4)'
      end if

      return
      end

*-----------------------------------------------------------------------
      subroutine generate_hardcopy(iunit, iout, hardcpf)
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      integer iunit, iout
      character hardcpf*(MAXSTRINGLENGTH)
      character driverf*(MAXSTRINGLENGTH)

      character prompt*(MAXPROMPTLENGTH)
      character psf*(MAXSTRINGLENGTH)
      character ximagf*(MAXSTRINGLENGTH)
      character comstr*(MAXSTRINGLENGTH)
 
      prompt = 
     + 'Name of Postscript file (with .ps or .eps extension)'
      hardcpf = read_string(READ_DEFPOSX, READ_DEFPOSY,
     +                      prompt, strlen(prompt), .FALSE.,
     +                      MAXSTRINGLENGTH, ' ')

      psf = hardcpf(1:strlen(hardcpf))
      driverf = '?driver/postscript/' // psf(1:strlen(psf))

      call hf_copy_segment('?Picture/plot', driverf)
      call hf_open_segment(driverf)
      call hf_set_color('window = white, lines = black, markers = black, 
     +                   text = black')
      call hf_update_display()
      call hf_close_segment()
      call hf_delete_segment(driverf)
 
      return
      end

*-----------------------------------------------------------------------
      subroutine generate_pict_file(iunit, iout, hardcpf)
*-----------------------------------------------------------------------

      include 'postconst.h'
      include 'pubfunc.h'

      integer iunit, iout
      character hardcpf*(MAXSTRINGLENGTH)
      character driverf*(MAXSTRINGLENGTH)

      character prompt*(MAXPROMPTLENGTH)
      character psf*(MAXSTRINGLENGTH)
      character ximagf*(MAXSTRINGLENGTH)
      character comstr*(MAXSTRINGLENGTH)
 
      prompt = 'Name of PICT file (without .pict)'
      hardcpf = read_string(READ_DEFPOSX, READ_DEFPOSY,
     +                      prompt, strlen(prompt), .FALSE.,
     +                      MAXSTRINGLENGTH, ' ')

      psf = hardcpf(1:strlen(hardcpf)) // '.pict'
      driverf = '?driver/pict/' // psf(1:strlen(psf))

      call hf_copy_segment('?Picture/plot', driverf)
      call hf_open_segment(driverf)
      call hf_set_color('window = white, lines = black, markers = black, 
     +                   text = black')
      call hf_update_display()
      call hf_close_segment()
      call hf_delete_segment(driverf)
 
      return
      end
